/*
 * vi:ts=4:sw=4
 * $Id: archive_ex.c,v 1.2 2001/11/29 15:23:06 kgt Exp $
 */
/*
 * archive_ex.c - Archive library extention for "Angel Ladder for MGL2"
 * Copyright (c) 2001 Hiroshi Kuwagata <kgt@topaz.ocn.ne.jp>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by Hiroshi Kuwagata.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "archive_ex.h"
#include "compress_lz77.h"

ArchiveEx *
make_archive( int type, u_char* name)
{
	ArchiveEx* arc;

	arc = ( ArchiveEx*)malloc( sizeof( ArchiveEx));
	if( arc == NULL){
		fprintf( stderr, "Can't malloc Archive control block.\n");
		exit( 1);
	}

	strncpy( arc->name, name, FNAME_LEN);
	arc->fp = fopen( name, "w");
	if( arc->fp == NULL){
		free( arc);
		fprintf(stderr, "Can't open %s\n", name);
		return NULL;
	}

	arc->num  = 0;
	arc->item = NULL;
	arc->last = NULL;

	if( type == ARC_BMP || type == ARC_SCN){
		arc->type = type;
	} else {
		free( arc);
		fprintf(stderr, "̤бΥ֤Ǥ\n");
		return NULL;
	}

	return arc;
}

void
append_item( ArchiveEx* arc, u_char* name,
				 void* data, int esize, int width, int height)
{
	int	  size;
	void* cdata;
	ArchiveExItem* item;

	item = (ArchiveExItem*)malloc( sizeof( ArchiveExItem));
	if( item == NULL){
		fprintf( stderr, "Can't malloc Item block.\n");
		exit( 1);
	}

	cdata = (void*)malloc( esize);
	if( cdata == NULL){
		fprintf( stderr, "Can't malloc compress data area.\n");
		exit( 1);
	}

	/*
	 * ȥꥢƥκ
	 */

	// ե̾Υԡ
	strncpy( item->name, name, FNAME_LEN);

	// ǡΰ
	size = compress_lz77( cdata, data, esize);
	if( size == 0){
		// ̤ʤ縵ǡޤޤ륳ԡ
		memcpy( cdata, data, esize);
		size = esize;
	}

	item->osize  = size;
	item->esize  = esize;

	item->width  = width;
	item->height = height;
	item->next = NULL;

	arc->num++;

	/*
	 * ȥꥹȤؤΥڥ
	 */
	if( arc->item == NULL){
		// ꥹȤξ
		arc->item = item;
		arc->last = item;
	} else {
		// ˲äƤ
		arc->last->next = item;
		arc->last = item;
	}

	fwrite( cdata, size, 1, arc->fp);
	free( cdata);
}

static void
set_int32_be( u_char* buf, int val)
{
	buf[ 3] = (val >> 24) & 0xff;
	buf[ 2] = (val >> 16) & 0xff;
	buf[ 1] = (val >> 8) & 0xff;
	buf[ 0] = val & 0xff;
}

static void
set_int32_le( u_char* buf, int val)
{
	buf[ 0] = val & 0xff;
	buf[ 1] = (val >> 8) & 0xff;
	buf[ 2] = (val >> 16) & 0xff;
	buf[ 3] = (val >> 24) & 0xff;
}

static void
set_str( u_char* buf, u_char* str, int len)
{
	strncpy( buf, str, len);
}

/* 32bitǡǤΥӥåȥơ
	width > 0 , width < 0  */
static void
bit_rotate32le( u_char* buf, int width) 
{
	unsigned int	x;

	x = buf[3] << 24 | buf[2] << 16 | buf[1] << 8 | buf[0];
	if( width > 0){
		x = (x << width) | ( x >> (32 - width));
	} else {
		width *= -1;
		x = (x >> width) | ( x << (32 - width));
	}
	buf[3] = (x >> 24) &0xff;
	buf[2] = (x >> 16) &0xff;
	buf[1] = (x >> 8) &0xff;
	buf[0] = x &0xff;
}

/* lenǡǤΥХȥơ
	width > 0 , width < 0 , ͭwidthĹ255ХȤޤ */
static void
byte_rotate( u_char* buf, int len, int width) 
{
	u_char tmp[255];

	if( width > 0){
		memcpy( tmp, buf, width);
		memcpy( buf, buf + width, len - width);
		memcpy( buf + len - width, tmp, width);
	} else {
		width *= -1;
		memcpy( tmp, buf + len - width, width);
		memcpy( buf + width, buf, len - width);
		memcpy( buf, tmp, width);
	}
}

static void
scramble_block( u_char* buf, int len, int salt)
{
	int	i;

	for( i = 0; i < len; i++){
		buf[ i] = buf[ i] ^ salt;
	}
}

void
finish_archive( ArchiveEx* arc)
{
	int		i, offset, bsize;
	ArchiveExItem *item;
	u_char	buf[ ARC_MAX_ITEMSZ];

	offset = 0;

	switch( arc->type){
	case ARC_BMP:
		bsize = ARC_BMP_ITEMSZ;
		break;

	case ARC_SCN:
		bsize = ARC_SCN_ITEMSZ;
		break;

	default:
		fprintf( stderr, "Unknown archive type.\n");
		fclose( arc->fp);
		remove( arc->name);
		exit(1);
	}

	for( item = arc->item, offset = 0;
	  item != NULL;  offset += item->osize, item = item->next){
		memset( buf, 0, ARC_MAX_ITEMSZ);

		switch( arc->type){
		case ARC_BMP:
			set_int32_le( buf + 4, item->osize);
			set_int32_le( buf + 8, item->esize);
			set_int32_le( buf + 12, offset);
			set_int32_le( buf + 16, item->width);
			set_int32_le( buf + 20, item->height);
			set_str( buf + 24, item->name, 16);
			break;

		case ARC_SCN:
			set_int32_le( buf + 0x18, offset);
			set_str( buf + 0x1c, item->name, 16);
			break;

		default:
			fprintf( stderr, "Unknown archive type.\n");
			fclose( arc->fp);
			remove( arc->name);
			exit(1);
		}

		scramble_block( buf, bsize, 0xd6);

		byte_rotate( buf, bsize, 7);

		for( i = 0; i < bsize; i += 4){
			bit_rotate32le( buf + i, -13);
		}

		fwrite( buf, bsize, 1, arc->fp);
	}

	set_int32_le( buf, arc->num);
	fwrite( buf, 4, 1, arc->fp);

	fclose( arc->fp);
	arc->fp = NULL;
	free( arc);
}
