/*
 * vi:ts=4:sw=4
 * $Id: compress_lz77.c,v 1.1 2001/11/29 13:06:26 kgt Exp $
 */
/*
 * compress_lz77 - LZ77 compress module for "Angel Ladder for MGL2"
 * Copyright (c) 2001 Hiroshi Kuwagata <kgt@topaz.ocn.ne.jp>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by Hiroshi Kuwagata.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>

#define LZ77_DICSIZE		(0xfff + 1)
#define LZ77_MATCHMIN		3
#define LZ77_MATCHMAX		(0x0f + 3)

#define RBUF_ERR_OK			0x0000
#define RBUF_ERR_NOTENOUGH	0x8000
#define RBUF_ERR_NOTFOUND	0x8001

typedef struct {
	u_char* buff;
	int	size;
	int	max;

	int top;
	int bottom;
} RCB;	// RCB == RingBuffer Control Block


/*
 * rbuf_new	- 󥰥Хåե
 */
RCB*
rbuf_new( int size)
{
	RCB* ret;

	ret = (RCB*)malloc( sizeof( RCB));
	if( ret == NULL){
		fprintf( stderr, "Can't malloc ring buffer control block.\n");
		exit( 1);
	}

	ret->buff = (u_char*)malloc( sizeof( u_char) * size);
	if( ret->buff == NULL){
		fprintf( stderr, "Can't malloc ring buffer (size=%d).\n", size);
		exit( 1);
	}
	memset( ret->buff, 0, size);

	ret->max    = size;
	ret->size   = 0;
	ret->top    = 0;
	ret->bottom = 0;

	return ret;
}

/*
 * rbuf_free	- 󥰥Хåե˴/
 */
void
rbuf_free( RCB* rcb)
{
	free( rcb->buff);
	free( rcb);
}

/*
 * rbuf_que	- 󥰥ХåեؤΥǡ()
 */
void
rbuf_que( RCB* rcb, u_char* dat, int size)
{
	int	i;

	rcb->size += size;

	/*
     * ܤ˥󥰥ХåեδԤʤ顢ǡԡ
	 * ϰȽԤmemcpy()ǤƤӽФƤΨɤ
	 * ξ硢塼̤ʤΤǰХȤĥԡ
	 * Ǥ
	 *
	 * [Memo]
     *  ˡΥ⡣󥰥Хåե礭
	 *  ä
	 *
	 *  
     *	*1	ԡǡХåե꤭ʤ
	 *		ǡƬڼΤƤơΥǡХåե˻Ĥ褦
	 *		ǡΥեåȤĴ
	 *
	 *  *2	ԡѥȽ
	 *
	 *	ͤ륳ԡѥ
	 *	*Pattern-1
	 *		Хåեޤ̵֤
	 *		+-----------------------+
	 *		|                       |
	 *		| - - - - - - - - - - - +---------> bottom 
	 *		|                       |    A
	 *		|                       |    |
	 *		|       dat򥳥ԡ     |    |  size
	 *		|                       |    |
	 *		|                       |    V
	 *		| - - - - - - - - - - - +----------
	 *		+-----------------------+
     *
	 *	*Pattern-2
	 *		Хåեޤ֤
   	 *		+-----------------------+    | 
	 *		|   datȾ򥳥ԡ     |    | sizeȾ( size - sizeȾ)
	 *		|                       |    V
	 *		| - - - - - - - - - - - +----------
	 *		|                       |    
	 *		|                       |     
	 *		| - - - - - - - - - - - +---------> bottom 
	 *		|                       |    A
	 *		|    datȾ򥳥ԡ    |    | sizeȾ( max - bottom)
	 *		+-----------------------+    |
	 *
	 *  
	 *		Υǡ¸̤Ĵtop֤ĴϸƱOK
	 *
	 *	¾
	 *		quedequesearch⤪ʤͤ˹ͤ롣
	 */
	if( dat != NULL){
		while( size-- > 0){
			rcb->buff[ rcb->bottom++] = *dat++;
			if( rcb->bottom >= rcb->max) rcb->bottom = 0;
		}
	} else {
		while( size-- > 0){
			rcb->buff[ rcb->bottom++] = 0;
			if( rcb->bottom >= rcb->max) rcb->bottom = 0;
		}
	}

	if( rcb->size > rcb->max){
		rcb->size = rcb->max;
		rcb->top  = rcb->bottom;
	}
}

/*
 * rbuf_deque	- 󥰥ХåեΥǡȴ(Ƭ)
 */
int
rbuf_deque( RCB* rcb, u_char* dat, int size)
{
	if( rcb->size < size){
		return RBUF_ERR_NOTENOUGH;
	}

	rcb->size -= size;	//˥Хåեλĥǡ̤򸺤餷Ƥ

	while( size-- > 0){
		*dat++ = rcb->buff[ rcb->top++];

		if( rcb->top == rcb->max){
			rcb->top = 0;
		}
	}

	return RBUF_ERR_OK;
}

/*
 * rbuf_search	- 󥰥Хåեθ(פõ)
 *
 * 	RBUF_ER_OK			դä
 *			RBUF_ER_NOTFOUND	פʪ̵
 *
 * 		rcb		оݥ󥰥Хåե
 *			str		ǡ
 *			fpos	դäΥХåեΰ
 *			fsize	դäΰץХȿ
 *			limit	ΥΥߥå(Ͱʾΰפ
 *					դäǸλ)
 */
int
rbuf_search( RCB* rcb, u_char* str, int* fpos, int* fsize, int limit)
{
	int	i, j, p;
	int	max_pos, max_size;

	max_pos  = -1;
	max_size = -1;

#if 0
	for( i = 0, p = rcb->top; i < rcb->size; i++){
		if( rcb->buff[ p] == *str){
			int	p2 = p + 1, size = 1;
			if( p2 >= rcb->max) p2 = 0;

			while( rcb->buff[ p2] == str[ size]){
				if( p2 == rcb->bottom) break;

				if( ++p2 >= rcb->max) p2 = 0;
				size++;
			}

			if( size > max_size){
				max_size = size;
				max_pos  = i;

				if( limit <= max_size) break;
			}
		}

		if( ++p >= rcb->max) p = 0;
	}
#else
	for( i = 0; i < rcb->max; i++){
		if( rcb->buff[ i] == *str){
			int	p2 = i, size = 0;

			do {
				size++;
				if( ++p2 >= rcb->max) break;
			} while( rcb->buff[ p2] == str[ size]);

			if( size > max_size &&
			  !(( i < rcb->bottom)&&( (i+size) > rcb->bottom))   ){
				max_size = size;
				max_pos  = i;

				if( limit <= max_size) break;
			}
		}
	}
#endif

	*fpos  = max_pos;
	*fsize = max_size;

	if( max_size < 0){
		return RBUF_ERR_NOTFOUND;
	} else {
		return RBUF_ERR_OK;
	}
}


/*
 * compress_lz77 - LZ77̴ؿ
 *			̸Υǡ(0ξϰ̤ʤäؤ)
 *			dst		̸ΥǡǼɥ쥹
 *				src		ǡγǼɥ쥹
 *				ssz		ǡΥ
 */
int
compress_lz77( void *dst, void* src, int ssz)
{
	int    pos, width, f, dsz;
	RCB*   dic;
	u_char *sp, *dp, *fp;

	dic = rbuf_new( LZ77_DICSIZE);
	if( dic == NULL){
		fprintf( stderr, "Can't malloc dictionary buffer.\n");
		exit( 1);
	}

	rbuf_que( dic, NULL, LZ77_DICSIZE - LZ77_MATCHMAX);

	sp  = src;
	dp  = dst;
	dsz = ssz;
	f   = 0x0080;

	while( ssz > 0){
		if( (f <<= 1) & 0x0100){
			f  = 0x01;
			fp = dp++;
			*fp = 0x00;
			dsz--;
		}

		/* ǡ̤ХȿˤʤޤǤޤ魯 */

		if( dsz <= 2) {
			// ̥ǡΰλ̤2ХȤڤäϰ̸̵̤Ȥߤʤ
			rbuf_free( dic);
			return 0;		
		}

		rbuf_search( dic, sp, &pos, &width, LZ77_MATCHMAX);
		if( ssz < width){
			//ǡλ̰ʾΥޥåɬ̵(ưθˤʤ)Τǥå
			width = ssz;
		}

		if( width >= LZ77_MATCHMIN){
			/* ˥ޥåǡä */

			if( width > LZ77_MATCHMAX) width = LZ77_MATCHMAX;
	
			// ޥåʬɤФ
			rbuf_que( dic, sp, width);

			// ݥι
			sp += width;
			ssz -= width;

			// dstؤΥǡϿ
			*dp++ = pos & 0xff;
			*dp++ = ((pos >> 4) & 0xf0) | (width - LZ77_MATCHMIN);

			dsz -= 2;
			
		} else {
			/* ˥ޥåǡ̵ä */

			// ХȤɤФ
			rbuf_que( dic, sp, 1);

			// dstؤΥǡϿ
			*dp++ = *sp++;
			ssz--;
			dsz--;

			// ե饰
			*fp |= f;
		}
	}

	rbuf_free( dic);

	return (dp - (u_char*)dst);
}

#if 0

#define RBUFSZ    (0xfff + 1)
#define LEN       (0xf + 3)

static void
decode_lz77(FILE *fp, u_char * data, int osize, int esize)
{
    char rbuf[RBUFSZ + LEN - 1];
    u_int ch, pos, c1, c2, offset, count, i;
    u_int bits = 0;
    int tpos = 0;

    memset(rbuf, 0, RBUFSZ - LEN);
    pos = RBUFSZ - LEN;

    while (!feof(fp)) {
        /* ե饰 (1B) ϲ̤鸫Ƥ */
        if (!((bits >>= 1) & 0x100)) {
            ch = fgetc(fp);
            bits = ch | 0xff00;
        }
        /* ӥåȤΩäƤ٥ 1B */
        if (bits & 1) {
            ch = fgetc(fp);
            rbuf[pos] = ch;
			pos = (pos + 1) & ( RBUFSZ - 1);
            data[tpos] = ch;
            if (++tpos >= esize)
                return;
        } else {
            /* 2B (ab cd) ǰ cab Ĺ d + 3 */
            c1 = fgetc(fp);
            c2 = fgetc(fp);
            offset = c1 | ((c2 & 0xf0) << 4);
            count = (c2 & 0x0f) + 3;
            for (i = 0; i < count; i++) {
                ch = rbuf[(offset + i) & (RBUFSZ - 1)];
                data[tpos] = ch;
                rbuf[pos] = ch;
				pos = (pos + 1) & ( RBUFSZ - 1);
                if (++tpos >= esize)
                    return;
			}
        }
    }
}

u_char	a[100 * 1024];
u_char	b[100 * 1024];

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
main()
{
#if 0
	RCB*	x;
	u_char	a[11];
	int		pos, size;

	x = rbuf_new( 10);

	rbuf_que( x, "0123456789abcdef", 16);

	memset( a, 0, 11);
	rbuf_deque( x, a, 4);
	printf( "%s %d\n", a, x->size);

	rbuf_que( x, "ABCDEF", 6);

	memset( a, 0, 11);
	rbuf_deque( x, a, 10);
	printf( "%s %d\n", a, x->size);

	rbuf_que( x, "abcdabcdef", 10);
	rbuf_search( x, "abcded", &pos, &size, 4);

	printf( "%d %d\n", pos, size);
#endif
#if 0
	int		sz;
	FILE	*fp;
	int		i;

	for( i = 0; i < sizeof( a); i++){
		a[ i] = ((u_char*)rbuf_new)[i];
	}

	sz = compress_lz77( b, a, sizeof( a));
	printf( "%d\n", sz);

	fp = fopen( "bfo", "w");
	fwrite( b, sz, 1, fp);
	fclose( fp);
	
	fp = fopen( "bfo", "r");
	decode_lz77( fp, b, sz, sizeof( a));
	fclose( fp);

	for( i = 0; i < sizeof( a); i++){
		if( a[ i] != b[i]){
			printf( "found not match(%d)\n", i);
		}
	}

#endif
#if 1
	FILE   *fp;
	u_char *buf, *cbuf;
	struct stat sb;
	int	   sz;

	stat( "xxx", &sb);
	buf  = (u_char*)malloc( sizeof( u_char) * sb.st_size);
	cbuf = (u_char*)malloc( sizeof( u_char) * sb.st_size);

	fp = fopen( "xxx", "r");
	fread( buf, sb.st_size, 1, fp);

	fclose( fp);
	sz = compress_lz77( cbuf, buf, sb.st_size);

	fp = fopen( "cfo", "w");
	fwrite( buf, sb.st_size, 1, fp);
	fclose( fp);

	printf( "%d -> %d\n", sz);

	fp = fopen( "afo", "w");
	fwrite( cbuf, sz, 1, fp);
	fclose( fp);

	memset( buf, 0, sb.st_size);
	fp = fopen( "afo", "r");
	decode_lz77( fp, buf, sz, sb.st_size);
	fclose( fp);

	fp = fopen( "bfo", "w");
	fwrite( buf, sb.st_size, 1, fp);
	fclose( fp);
	
#endif

}
#endif
