/*
 * vi:ts=4:sw=4
 * $Id: datconv.c,v 1.5 2002/01/30 09:37:49 kgt Exp $
 */
/*
 * datconv - Angel Ladder image converter for "Angel Ladder for MGL2"
 * Copyright (c) 2001 Hiroshi Kuwagata <kgt@topaz.ocn.ne.jp>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by Hiroshi Kuwagata.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <setjmp.h>
#include <sys/types.h>

#include "hsb_color.h"
#include "aladder.h"
#include "archive_ex.h"

#define IMAGE_DEPTH_16BPP		0x00000010
#define IMAGE_DEPTH_4BPP		0x00000004
#define IMAGE_DEPTH_2BPP		0x00000002
#define IMAGE_DEPTH_MASK		0x000000ff
#define IMAGE_ENDIAN_LITTLE		0x00000100
#define IMAGE_ENDIAN_BIG		0x00000000

/*
 *	-o output file( default STDOUT
 *	-w set width (only percent specify)
 *	-h set hight (only percent specify)
 *	-b set brightness( 0-15)
 *	-d set dither type( JN, FS, none)
 *	-D set DEPTH(16bpp,4bpp,2bpp)
 *	-m with mask graphic
 */
COLOR_RGB* decode_image( Archive* arc, u_char* name, int* width, int* height);
void write_file( FILE *fp, int w, int h, int f, void* dat, int osz, int csz);
int is_archive( Archive *arc, u_char* name);

#undef DEBUG

#ifndef DEBUG
int main(int argc, char *argv[])
{
	int			i, j; 
	int			iw, ih, ow, oh, w = 50, h = 50;
	int			cnv_flg = WIDTH_PERCENT | HEIGHT_PERCENT | WITH_ALPHA, br = 15;
	char		output[100] = "\0";
	Archive		*in;
	ArchiveEx	*out;

	int			org_sz, comp_sz;
	COLOR_RGB	*rgb_image;
	int			*hsb_image;
	u_int8_t	*packed_image, *lz_image;
	u_char*		text;

	/*
	 * ޥɥ饤󥪥ץɾ
	 */
	for( i = 1; i < argc; i++){
		char* opt = argv[ i];
		
		if( *opt == '-'){
			if( !strcmp( "-o", opt) || !strcmp( "--output", opt)){
				if( i < argc){
					char* par = argv[ ++i];

					strcpy( output, par);
				} else {
					fprintf( stderr, "Not specified output file(%s).\n",opt);
					exit( 1);
				}

			} else if( !strcmp( "-w", opt) || !strcmp( "--width", opt)){
				if( i < argc){
					char* par = argv[ ++i];

					for( j = 0, w = 0; isdigit( par[j]); j++){
						w *= 10;
						w += par[j] - '0';
					}

				} else {
					fprintf( stderr, "Not specifed output width(%s).\n",opt);
					exit( 1);
				}

			} else if( !strcmp( "-h", opt) || !strcmp( "--height", opt)){
				if( i < argc){
					char* par = argv[ ++i];

					for( j = 0, h = 0; isdigit( par[j]); j++){
						h *= 10;
						h += par[j] - '0';
					}

				} else {
					fprintf( stderr, "Not specified output height(%s).\n",opt);
					exit( 1);
				}
			} else if( !strcmp( "-b", opt) || !strcmp( "--brightness", opt)){
				if( i < argc){
					br = strtol( argv[ ++i], NULL, 0);
				} else {
					fprintf( stderr, "Not specified brightness(%s).\n",opt);
					exit( 1);
				}

			} else if( !strcmp( "-d", opt) || !strcmp( "--dither", opt)){
				if( i < argc){
					char* par = argv[ ++i];

					cnv_flg &= ~DITHER_MASK;

					if( !strcmp( "FS", par)){
						cnv_flg |= ( DITHER_FS| DITHER_ALL);
					} else if( !strcmp( "JN", par)){
						cnv_flg |= ( DITHER_JN| DITHER_ALL);
					} else if( !strcmp( "none", par)){
						/* Nothing */
					} else {
						fprintf( stderr,
						    "Unknown dither type %s(%s).\n",par, opt);
						exit( 1);
					}
				} else {
					fprintf( stderr, "Not specified dither type(%s).\n",opt);
					exit( 1);
				}

			} else if( !strcmp( "-D", opt) || !strcmp( "--depth", opt)){
				if( i < argc){
					char* par = argv[ ++i];

					cnv_flg &= ~DEPTH_MASK;

					if( !strcmp( "16bpp", par)){
						/* NOTHING */
					} else if( !strcmp( "4bpp", par)){
						cnv_flg |= CHANGE_GRAYSCALE;
					} else if( !strcmp( "2bpp", par)){
						cnv_flg |= (CHANGE_GRAYSCALE | DEPTH_2BPP);
					} else {
						fprintf( stderr,
						    "Unknown depth %s(%s).\n",par, opt);
						exit( 1);
					}
				} else {
					fprintf( stderr, "Not specified depth type(%s).\n",opt);
					exit( 1);
				}

			} else {
				fprintf( stderr, "Unknown option %s\n", opt);
				exit( 1);
			}
		} else {
			break;
		}
	}

	/*
	 * ϥ֤Υץ
	 */
	if( i < argc){
		if( ( in = open_archive( argv[i])) == NULL){
			fprintf( stderr, "Can\'t open archive( File \"%s\").\n", argv[i]);
			exit( 1);
		}

		if( in->type != ARC_BMP){
			fprintf( stderr, "%s is Not ARC_BMP archive.\n", argv[i]);
			exit( 1);
		}
	} else {
		/* ̵Х顼 */
		fprintf( stderr, "Not specified input file.\n");
		exit( 1);
	}

	/*
	 * ϥ֤κ
	 */
	if( output[ 0] == '\0'){
		strcpy( output, argv[i]);
		strcat( output, ".xm2");
	} 
	out = make_archive( in->type, output);

	for( i = 0; i < in->num; i++){
		printf( "%s", in->item[i].name);

		if( in->item[i].name[0] == 'm'){
			if( is_archive( in, in->item[i].name + 1)){
				printf( "	skip\n");
				continue;
			}
		}

		/*
		 * PPMRGB᡼Ѵ
		 */
		rgb_image = decode_image( in, in->item[i].name, &iw, &ih);
		if( !rgb_image){
			fprintf( stderr, "Can\'t decode TENHASHI image.\n");
			exit(1);
		}

		/*
		 * RGB᡼MGL᡼Ѵ(Ƚ̴ޤ)
		 */
		ow = w;
		oh = h;
		printf( "	%dx%d => ", iw, ih);
		hsb_image = mk_mgl_image( rgb_image, iw, ih, cnv_flg, br, &ow, &oh);
		free( rgb_image);

		if( !hsb_image){
			fprintf( stderr, "Can\'t convert RGB to HSB image.\n");
			exit(1);
		}
		printf( "%dx%d\n", ow, oh);

		/*
		 * MGL᡼켡(ץӥåȴ)
		 */
		packed_image = (u_int8_t*)malloc(
							sizeof( u_int8_t) * ow * oh * sizeof( int) + 8);
		if( !packed_image){
			free( hsb_image);
			fprintf( stderr, "Can\'t malloc packed image buffer.\n");
			exit(1);
		}

		org_sz = pack_mgl_image(
					hsb_image, packed_image + 8, cnv_flg, ow * oh);
		free( hsb_image);

		packed_image[ 0] = ((ow >> 8) & 0xff);
		packed_image[ 1] = (ow & 0xff);

		packed_image[ 2] = ((oh >> 8) & 0xff);
		packed_image[ 3] = (oh & 0xff);

		packed_image[ 4] = ((cnv_flg >> 24) & 0xff);
		packed_image[ 5] = ((cnv_flg >> 16) & 0xff);
		packed_image[ 6] = ((cnv_flg >> 8) & 0xff);
		packed_image[ 7] = (cnv_flg & 0xff);

		if( !org_sz){
			free( packed_image);
			fprintf( stderr, "Can\'t hsb_image pack.\n");
			exit(1);
		}

		/*
		 * ֤ؤν
		 */
		append_item( out, in->item[i].name, packed_image, org_sz + 8, iw, ih);
		free( packed_image);
	}

	/* 饹Ȥɲä륢ƥ
		ӥåȥޥåץ֤ξˤϲѴѥ᡼ */
	/* Ѵѥ᡼եޥå
	 *  0 - 31	"converted for MGL2" ĤNULLʸ
	 * 32 - 33	X rate(big endian)
	 * 34 - 35	Y rate(big endian)
	 * 36 - 36	convert flag(big endian)
	 */
	{
		u_char status[ 100];

		memset( status, 0, 100);

		strncpy( status, "converted for MGL2", 18);

		status[ 32] = (w >> 8) & 0xff;
		status[ 33] = w & 0xff;

		status[ 34] = (h >> 8) & 0xff;
		status[ 35] = h & 0xff;

		switch( cnv_flg & DEPTH_MASK){
		case 0x0000:	// 16bpp
			status[ 36] = 16;
			break;

		case 0x0100:	// 4bpp
			status[ 36] = 4;
			break;

		case 0x0300:	// 2bpp
			status[ 36] = 2;
			break;

		default:
			fprintf( stderr, "Really? convert flag is illeagal value.\n");
			exit( 1);
		}
		append_item( out, "STATUS", status, 37, 0, 0);
	}

	finish_archive( out);
	close_archive( in);
}

int
is_archive( Archive *arc, u_char* name)
{
	int	i;

	for( i = 0; i < arc->num; i++){
		if( strcasecmp( arc->item[i].name, name) == 0){
			return 1;
		}
	}

	return 0;
}

COLOR_RGB*
decode_image( Archive* arc, u_char* name, int *width, int *height)
{
	int		  i, x, y, mask;
	Image	  img, msk_img;
	u_char	  mname[ FNAME_LEN+2];
	COLOR_RGB *ret;

	sprintf( mname, "m%s", name);

	get_image( arc, name, &img);
	if( is_archive( arc, mname)){
		mask = 1;
		get_image( arc, mname, &msk_img);
	} else {
		mask = 0;
	}

	ret = ( COLOR_RGB*)malloc( sizeof( COLOR_RGB) * img.width * img.height);

	for( i = 0, y = 0; y < img.height; y++){
		for( x = 0; x < img.width; x++, i++){
			ret[i].r = ((u_char*)img.bitmap)[ (i * 3) + 0];
			ret[i].g = ((u_char*)img.bitmap)[ (i * 3) + 1];
			ret[i].b = ((u_char*)img.bitmap)[ (i * 3) + 2];
			if( mask){
				ret[i].t = (((u_char*)msk_img.bitmap)[( i * 3) + 0] < 32)? 0: 1;
				ret[i].a = ( 255 - ((u_char*)msk_img.bitmap)[ ( i * 3) + 0]);
			} else {
				ret[i].t = 0;
				ret[i].a = 255;
			}
		}
	}

	*width  = img.width;
	*height = img.height;
	free( img.bitmap);
	if( mask){
		free( msk_img.bitmap);
	}

	return ret;
}


static void
put1b( FILE *fp, u_int8_t dat)
{
	fputc( dat & 0xff, fp);
}

static void
put2b( FILE *fp, u_int16_t dat)
{
	fputc( (dat >> 8) & 0xff, fp);
	fputc( dat & 0xff, fp);
}

static void
put4b( FILE *fp, u_int32_t dat)
{
	fputc( (dat >> 24) & 0xff, fp);
	fputc( (dat >> 16) & 0xff, fp);
	fputc( (dat >> 8) & 0xff, fp);
	fputc( dat & 0xff, fp);
}

/*
 * ե񤭽Фؿ
 */
void
write_file( FILE *fp, int w, int h, int f, void* dat, int osz, int csz)
{
	int	flag = 0;
	/*
	 * إåν񤭽Ф
	 */
	put2b( fp, w);	// WIDTH
	put2b( fp, h);	// HEIGHT

	// Flag
	switch( f & DEPTH_MASK){
	case 0x00000000: // 16bpp
		flag |= 0x00000010;
		break;

	case 0x00000100: // 4bpp
		flag |= 0x00000004;
		break;

	case 0x00000300: // 2bpp
		flag |= 0x00000002;
		break;
	}

	if( f & WITH_ALPHA){
		flag |= 0x00000100;
	}

	put4b( fp, flag);	// flag

	put4b( fp, osz);	// Orig size
	put4b( fp, csz);	// Compressed size

	fwrite( dat, csz, 1, fp);
}
#else /* DEBUG */
void
main()
{
	int			i;
	Archive		*in;

	in = open_archive( "BData_01.dat");

	for( i = 0; i < in->num; i++){
		printf( "found: %s\n", in->item[i].name);
	}
}
#endif /* DEBUG */
