/*
 * vi:ts=4:sw=4:
 * $Id: hsb_color.c,v 1.1 2001/11/29 13:06:26 kgt Exp $
 */
/*
 * hsb_color.c
 * Copyright (C) 2001
 *      Hiroshi Kuwagata (kgt@topaz.ocn.ne.jp)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY HIROSHI KUWAGATA ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE TERRENCE R. LAMBERT BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
#include "hsb_color.h"
#include <sys/types.h>

const static long alpha_table[] = {
    0x00000000, /* 0x00 */
    0x00001111, /* 0x01 */
    0x00002222, /* 0x02 */
    0x00003333, /* 0x03 */
    0x00004444, /* 0x04 */
    0x00005555, /* 0x05 */
    0x00006666, /* 0x06 */
    0x00007777, /* 0x07 */
    0x00008888, /* 0x08 */
    0x00009999, /* 0x09 */
    0x0000aaaa, /* 0x0a */
    0x0000bbbb, /* 0x0b */
    0x0000cccc, /* 0x0c */
    0x0000dddd, /* 0x0d */
    0x0000eeee, /* 0x0e */
    0x00010000, /* 0x0f */
};
   
#if 1
void RGBtoHSB( COLOR_RGB* rgb, COLOR_HSB* hsb)
{
	int max, min, diff;
	int hue, sat, bri;

	max = ( rgb->r > rgb->g)? rgb->r: rgb->g;
	if( rgb->b > max) {
		max = rgb->b;
	}

	min = ( rgb->r < rgb->g)? rgb->r: rgb->g;
	if( rgb->b < min) {
		min = rgb->b;
	}

	diff = max - min;

	// ٤RGBǤκǹ⵱Ǥ
	bri = max;

	// ٻ(âλ0ǳˤʤΤԤ)
	if( max == 0){
		sat = 0;
	} else {
		sat = (255 * diff)/max;
	}

	// 껻(0ξǤ⻻)
	if( diff != 0){
		if( rgb->r == max){
			hue = ((60 * (rgb->g - rgb->b))/ diff);
		} else if( rgb->g == max) {
			hue = 120 + ((60 * (rgb->b - rgb->r))/ diff);
		} else {
			hue = 240 + ((60 * (rgb->r - rgb->g))/ diff);
		}
	} else {
		hue = 0;
	}

	if( hue < 0){
		hue += 360;
	}
	if( hue > 360){
		hue -= 360;
	}

	hsb->h = hue;
	hsb->s = sat;
	hsb->b = bri;
	hsb->a = rgb->a;
	hsb->t = rgb->t;
}
#else
void RGBtoHSB( COLOR_RGB *rgb, COLOR_HSB* hsb)
{
	double fr, fg, fb;
	double max, min, diff;
	double hue, sat, bri;

	fr = ((double)rgb->r)/255.0;
	fg = ((double)rgb->g)/255.0;
	fb = ((double)rgb->b)/255.0;

	max = ( fr > fg)? fr: fg;
	if( fb > max) {
		max = fb;
	}

	min = ( fr < fg)? fr: fg;
	if( fb < min) {
		min = fb;
	}

	diff = max - min;

	// ٤RGBǤκǹ⵱Ǥ
	bri = max;

	// ٻ(âλ0ǳˤʤΤԤ)
	if( max == 0){
		sat = 0.0;
	} else {
		sat = diff/max;
	}

	// 껻(0ξǤ⻻)
	if( diff != 0.0){
		if( fr == max){
			hue = ((fg - fb)/ diff);
		} else if( fg == max) {
			hue = 2.0 + ((fb - fr) / diff);
		} else {
			hue = 4.0 + ((fr - fg) / diff);
		}
	} else {
		hue = 0.0;
	}
	hue /= 6.0;
	if( hue < 0.0){
		hue += 1.0;
	}

	hsb->h = (int)(hue * 360.0);
	hsb->s = (int)(sat * 255.0);
	hsb->b = (int)(bri * 255.0);
	hsb->a = rgb->a;
	hsb->t = rgb->t;
}
#endif

void RGBtoHSB_gs( COLOR_RGB* rgb, COLOR_HSB* hsb)
{
	hsb->h = 0;
	hsb->s = 0;
#if 0
	hsb->b = ((rgb->r * 77)+ (rgb->g * 28) + (rgb->b * 151)) >> 8;
#else
	hsb->b = (rgb->r + rgb->g + rgb->b) / 3;
#endif
	hsb->a = rgb->a;
	hsb->t = rgb->t;
}

#if 0
#define HUE_CONV12( c, d) {\
	if( (c)->h < 0){\
		(c)->h += 360;\
	} else if( (c)->h >= 360){\
		(c)->h -= 360;\
	}\
	if( 15 > (c)->h){\
		(d) = 0 - (c)->h;\
		(c)->h = 0;\
	} else if( 45 > (c)->h){\
		(d) = 30 - (c)->h;\
		(c)->h = 1;\
	} else if( 75 > (c)->h){\
		(d) = 60 - (c)->h;\
		(c)->h = 2;\
	} else if( 105 > (c)->h){\
		(d) = 90 - (c)->h;\
		(c)->h = 3;\
	} else if( 135 > (c)->h){\
		(d) = 120 - (c)->h; \
		(c)->h = 4;\
	} else if( 165 > (c)->h){\
		(d) = 150 - (c)->h;\
		(c)->h = 5;\
	} else if( 195 > (c)->h){\
		(d) = 180 - (c)->h;\
		(c)->h = 6;\
	} else if( 225 > (c)->h){\
		(d) = 210 - (c)->h;\
		(c)->h = 7;\
	} else if( 255 > (c)->h){\
		(d) = 240 - (c)->h;\
		(c)->h = 8;\
	} else if( 285 > (c)->h){\
		(d) = 270 - (c)->h;\
		(c)->h = 9;\
	} else if( 315 > (c)->h){\
		(d) = 300 - (c)->h;\
		(c)->h = 10;\
	} else if( 345 > (c)->h){\
		(d) = 330 - (c)->h;\
		(c)->h = 11;\
	} else if( 360 > (c)->h){\
		(d) = 360 - (c)->h;\
		(c)->h = 0;\
	}\
}
#else
#define HUE_CONV12( c, d) {\
	if( (c)->h < 0){\
		(c)->h += 360;\
	} else if( (c)->h >= 360){\
		(c)->h -= 360;\
	}\
	if( 20 > (c)->h){\
		(d) = 5 - (c)->h;\
		(c)->h = 0;\
	} else if( 50 > (c)->h){\
		(d) = 35 - (c)->h;\
		(c)->h = 1;\
	} else if( 80 > (c)->h){\
		(d) = 65 - (c)->h;\
		(c)->h = 2;\
	} else if( 110 > (c)->h){\
		(d) = 95 - (c)->h;\
		(c)->h = 3;\
	} else if( 140 > (c)->h){\
		(d) = 125 - (c)->h; \
		(c)->h = 4;\
	} else if( 170 > (c)->h){\
		(d) = 155 - (c)->h;\
		(c)->h = 5;\
	} else if( 200 > (c)->h){\
		(d) = 185 - (c)->h;\
		(c)->h = 6;\
	} else if( 230 > (c)->h){\
		(d) = 215 - (c)->h;\
		(c)->h = 7;\
	} else if( 260 > (c)->h){\
		(d) = 245 - (c)->h;\
		(c)->h = 8;\
	} else if( 290 > (c)->h){\
		(d) = 275 - (c)->h;\
		(c)->h = 9;\
	} else if( 320 > (c)->h){\
		(d) = 305 - (c)->h;\
		(c)->h = 10;\
	} else if( 350 > (c)->h){\
		(d) = 335 - (c)->h;\
		(c)->h = 11;\
	} else if( 360 > (c)->h){\
		(d) = 365 - (c)->h;\
		(c)->h = 0;\
	}\
}
#endif

#define SAT_CONV12(c, d, bri) {\
	if( (c)->s < 0){\
		(c)->s = 0;\
	}\
	if( (c)->s > 255){\
		(c)->s = 255;\
	}\
\
	(c)->s = ((c)->s * (bri)) >> 16;\
\
	if( (c)->s < 16){\
		(d) = 8 - (c)->s;\
		(c)->s = 0;\
	} else if( (c)->s < 32){\
		(d) = 24 - (c)->s;\
		(c)->s = 1;\
	} else if( (c)->s < 48){\
		(d) = 40 - (c)->s;\
		(c)->s = 2;\
	} else if( (c)->s < 64){\
		(d) = 56 - (c)->s;\
		(c)->s = 3;\
	} else if( (c)->s < 80){\
		(d) = 72 - (c)->s;\
		(c)->s = 4;\
	} else if( (c)->s < 96){\
		(d) = 88 - (c)->s;\
		(c)->s = 5;\
	} else if( (c)->s < 112){\
		(d) = 104 - (c)->s;\
		(c)->s = 6;\
	} else if( (c)->s < 128){\
		(d) = 120 - (c)->s;\
		(c)->s = 7;\
	} else if( (c)->s < 144){\
		(d) = 136 - (c)->s;\
		(c)->s = 8;\
	} else if( 160 - (c)->s < 160){\
		(d) = (c)->s - 152;\
		(c)->s = 9;\
	} else if( (c)->s < 176){\
		(d) = 168 - (c)->s;\
		(c)->s = 10;\
	} else if( (c)->s < 192){\
		(d) = 184 - (c)->s;\
		(c)->s = 11;\
	} else if( (c)->s < 208){\
		(d) = 200 - (c)->s;\
		(c)->s = 12;\
	} else if( (c)->s < 224){\
		(d) = 216 - (c)->s;\
		(c)->s = 13;\
	} else if( (c)->s < 240){\
		(d) = 232 - (c)->s;\
		(c)->s = 14;\
	} else {\
		(d) = 248 - (c)->s;\
		(c)->s = 15;\
	}\
}

#define BRI_CONV12(c, d, bri) {\
	if( (c)->b < 0){\
		(c)->b = 0;\
	}\
	if( (c)->b > 255){\
		(c)->b = 255;\
	}\
\
	(c)->b = ((c)->b * (bri)) >> 16;\
\
	if( (c)->b < 16){\
		(d) =  8 - (c)->b;\
		(c)->b = 0;\
	} else if( (c)->b < 32){\
		(d) = 24 - (c)->b;\
		(c)->b = 1;\
	} else if( (c)->b < 48){\
		(d) = 40 - (c)->b;\
		(c)->b = 2;\
	} else if( (c)->b < 64){\
		(d) = 56 - (c)->b;\
		(c)->b = 3;\
	} else if( (c)->b < 80){\
		(d) = 72 - (c)->b;\
		(c)->b = 4;\
	} else if( (c)->b < 96){\
		(d) = 88 - (c)->b;\
		(c)->b = 5;\
	} else if( (c)->b < 112){\
		(d) = 104 - (c)->b;\
		(c)->b = 6;\
	} else if( (c)->b < 128){\
		(d) = 120 - (c)->b;\
		(c)->b = 7;\
	} else if( (c)->b < 144){\
		(d) = 136 - (c)->b;\
		(c)->b = 8;\
	} else if( (c)->b < 160){\
		(d) = 152 - (c)->b;\
		(c)->b = 9;\
	} else if( (c)->b < 176){\
		(d) = 168 - (c)->b;\
		(c)->b = 10;\
	} else if( (c)->b < 192){\
		(d) = 184 - (c)->b;\
		(c)->b = 11;\
	} else if( (c)->b < 208){\
		(d) = 200 - (c)->b;\
		(c)->b = 12;\
	} else if( (c)->b < 224){\
		(d) = 216 - (c)->b;\
		(c)->b = 13;\
	} else if( (c)->b < 240){\
		(d) = 232 - (c)->b;\
		(c)->b = 14;\
	} else {\
		(d) = 248 - (c)->b;\
		(c)->b = 15;\
	}\
}

#define ALP_CONV12( c) {\
	(c)->a >>= 4;\
}


static void cut_off12( COLOR_HSB *b, int w, int h, int f, int bri)
{
	int	x, y, hdiff, sdiff, bdiff;
	COLOR_HSB* c;

	bri = alpha_table[ bri];

	if( f & DITHER_JN){
		for( y = 0, b += 2; y < h; y++, b += (w + 4)){
			for( x = 0; x < w; x++){
				c = b + x;

				HUE_CONV12( c, hdiff);
				SAT_CONV12( c, sdiff, bri);
				BRI_CONV12( c, bdiff, bri);
				ALP_CONV12( c);

				if( f & DITHER_HUE){
					COLOR_HSB *cx = c;

					// current line
					(cx + 1)->h -= (( hdiff * 7) / 48);
					(cx + 2)->h -= (( hdiff * 5) / 48);

					// Next line
					cx += (w + 4);
					//(cx -2)->h -= (( hdiff * 3) / 48);
					(cx -1)->h -= (( hdiff * 5) / 48);
					(cx +0)->h -= (( hdiff * 7) / 48);
					(cx +1)->h -= (( hdiff * 5) / 48);
					//(cx +2)->h -= (( hdiff * 3) / 48);

					// Next-Next line
					cx += (w + 4);
					//(cx -2)->h -= (( hdiff * 1) / 48);
					//(cx -1)->h -= (( hdiff * 3) / 48);
					(cx +0)->h -= (( hdiff * 5) / 48);
					//(cx +1)->h -= (( hdiff * 3) / 48);
					//(cx +2)->h -= (( hdiff * 1) / 48);
				}
				if( f & DITHER_SATUATION){
					COLOR_HSB *cx = c;

#if 0
					// current line
					(cx + 1)->s -= (( sdiff * 7) / 48);
					//(cx + 2)->s -= (( sdiff * 5) / 48);

					// Next line
					cx += (w + 4);
					//(cx -2)->s -= (( sdiff * 3) / 48);
					//(cx -1)->s -= (( sdiff * 5) / 48);
					(cx +0)->s -= (( sdiff * 7) / 48);
					//(cx +1)->s -= (( sdiff * 5) / 48);
					//(cx +2)->s -= (( sdiff * 3) / 48);

					// Next-Next line
					//cx += (w + 4);
					//(cx -2)->s -= (( sdiff * 1) / 48);
					//(cx -1)->s -= (( sdiff * 3) / 48);
					//(cx +0)->s -= (( sdiff * 5) / 48);
					//(cx +1)->s -= (( sdiff * 3) / 48);
					//(cx +2)->s -= (( sdiff * 1) / 48);
#else
					// current line
					(cx + 1)->s -= (( sdiff * 9) / 48);
					(cx + 2)->s -= (( sdiff * 7) / 48);

					// Next line
					cx += (w + 4);
					//(cx -2)->s -= (( sdiff * 5) / 48);
					(cx -1)->s -= (( sdiff * 7) / 48);
					(cx +0)->s -= (( sdiff * 9) / 48);
					(cx +1)->s -= (( sdiff * 7) / 48);
					//(cx +2)->s -= (( sdiff * 5) / 48);

					// Next-Next line
					//cx += (w + 4);
					//(cx -2)->s -= (( sdiff * 1) / 48);
					//(cx -1)->s -= (( sdiff * 3) / 48);
					(cx +0)->s -= (( sdiff * 7) / 48);
					//(cx +1)->s -= (( sdiff * 3) / 48);
					//(cx +2)->s -= (( sdiff * 1) / 48);
#endif
				}
				if( f & DITHER_BRIGHTNESS){
					COLOR_HSB *cx = c;

#if 0
					// current line
					(c + 1)->b -= (( bdiff * 7) / 48);
					//(c + 2)->b -= (( bdiff * 5) / 48);

					// Next line
					cx += (w + 4);
					//(cx -2)->b -= (( bdiff * 3) / 48);
					//(cx -1)->b -= (( bdiff * 5) / 48);
					(cx +0)->b -= (( bdiff * 7) / 48);
					//(cx +1)->b -= (( bdiff * 5) / 48);
					//(cx +2)->b -= (( bdiff * 3) / 48);

					// Next-Next line
					//cx += (w + 4);
					//(cx -2)->b -= (( bdiff * 1) / 48);
					//(cx -1)->b -= (( bdiff * 3) / 48);
					//(cx +0)->b -= (( bdiff * 5) / 48);
					//(cx +1)->b -= (( bdiff * 3) / 48);
					//(cx +2)->b -= (( bdiff * 1) / 48);
#else
					// current line
					(c + 1)->b -= (( bdiff * 9) / 48);
					(c + 2)->b -= (( bdiff * 7) / 48);

					// Next line
					cx += (w + 4);
					//(cx -2)->b -= (( bdiff * 5) / 48);
					(cx -1)->b -= (( bdiff * 7) / 48);
					(cx +0)->b -= (( bdiff * 9) / 48);
					(cx +1)->b -= (( bdiff * 7) / 48);
					//(cx +2)->b -= (( bdiff * 5) / 48);

					// Next-Next line
					//cx += (w + 4);
					//(cx -2)->b -= (( bdiff * 3) / 48);
					//(cx -1)->b -= (( bdiff * 5) / 48);
					(cx +0)->b -= (( bdiff * 7) / 48);
					//(cx +1)->b -= (( bdiff * 5) / 48);
					//(cx +2)->b -= (( bdiff * 3) / 48);
#endif
				}
			}
		}
	} else if( f & DITHER_FS){
		for( y = 0, b += 2; y < h; y++, b += (w + 4)){
			for( x = 0; x < w; x++){
				c = b + x;

				HUE_CONV12( c, hdiff);
				SAT_CONV12( c, sdiff, bri);
				BRI_CONV12( c, bdiff, bri);
				ALP_CONV12( c);

				if( f & DITHER_HUE){
					COLOR_HSB *cx = c;

					// current line
					(cx + 1)->h -= (( hdiff * 7) / 16);

					// Next line
					cx += (w + 4);
					(cx -1)->h -= (( hdiff * 3) / 16);
					(cx +0)->h -= (( hdiff * 5) / 16);
					//(cx +1)->h -= (( hdiff * 1) / 16);
				}
				if( f & DITHER_SATUATION){
					COLOR_HSB *cx = c;

					// current line
					(cx + 1)->s -= (( sdiff * 7) / 16);

					// Next line
					cx += (w + 4);
					(cx -1)->s -= (( sdiff * 3) / 16);
					(cx +0)->s -= (( sdiff * 5) / 16);
					//(cx +1)->s -= (( sdiff * 1) / 16);
				}
				if( f & DITHER_BRIGHTNESS){
					COLOR_HSB *cx = c;

					// current line
					(cx + 1)->b -= (( bdiff * 7) / 16);

					// Next line
					cx += (w + 4);
					(cx -1)->b -= (( bdiff * 3) / 16);
					(cx +0)->b -= (( bdiff * 5) / 16);
					//(cx +1)->b -= (( bdiff * 1) / 16);
				}
			}
		}
	} else {
		for( y = 0, b += 2; y < h; y++, b += (w + 4)){
			for( x = 0; x < w; x++){
				c = b + x;

				HUE_CONV12( c, hdiff);
				SAT_CONV12( c, sdiff, bri);
				BRI_CONV12( c, bdiff, bri);
				ALP_CONV12( c);
			}
		}
	}
}



#define BRI_CONV2(c, d, bri) {\
	if( (c)->b < 0){\
		(c)->b = 0;\
	}\
	if( (c)->b > 255){\
		(c)->b = 255;\
	}\
\
	(c)->b = ((c)->b * (bri)) >> 16;\
\
	if( (c)->b < 64){\
		(d) = 32 - (c)->b;\
		(c)->b = 0x00;\
	} else if( (c)->b < 128){\
		(d) = 96 - (c)->b;\
		(c)->b = 0x05;\
	} else if( (c)->b < 192){\
		(d) = 160 - (c)->b;\
		(c)->b = 0x0a;\
	} else {\
		(d) = 224 - (c)->b;\
		(c)->b = 0x0f;\
	}\
\
	(c)->s = 0;\
}

#define ALP_CONV2( c) {\
	(c)->a >>= 6;\
}

/*
 * 2bpp졼Ѥθ
 */
static void cut_off2( COLOR_HSB *b, int w, int h, int f, int bri)
{
	int	x, y, diff;
	COLOR_HSB* c;

	bri = alpha_table[ bri];

	if( f & DITHER_JN){
		for( y = 0, b += 2; y < h; y++, b += (w + 4)){
			for( x = 0; x < w; x++){
				c = b + x;

				BRI_CONV2( c, diff, bri);
				ALP_CONV2( c);

				// current line
				(c + 1)->b -= (( diff * 7) / 48);
				(c + 2)->b -= (( diff * 5) / 48);

				// Next line
				c += (w + 4);
				(c - 2)->b -= (( diff * 3) / 48);
				(c - 1)->b -= (( diff * 5) / 48);
				(c + 0)->b -= (( diff * 7) / 48);
				(c + 1)->b -= (( diff * 5) / 48);
				(c + 2)->b -= (( diff * 3) / 48);

				// Next-Next line
				c += (w + 4);
				(c - 2)->b -= (( diff * 1) / 48);
				(c - 1)->b -= (( diff * 3) / 48);
				(c + 0)->b -= (( diff * 5) / 48);
				(c + 1)->b -= (( diff * 3) / 48);
				(c + 2)->b -= (( diff * 1) / 48);
			}
		}
	} else if( f & DITHER_FS){
		for( y = 0, b += 2; y < h; y++, b += (w + 4)){
			for( x = 0; x < w; x++){
				c = b + x;

				BRI_CONV2( c, diff, bri);
				ALP_CONV2( c);

				// current line
				(c + 1)->b -= (( diff * 7) / 16);

				// Next line
				c += (w + 4);
				(c - 1)->b -= (( diff * 3) / 16);
				(c + 0)->b -= (( diff * 5) / 16);
				(c + 1)->b -= (( diff * 1) / 16);
			}
		}
	} else {
		for( y = 0, b += 2; y < h; y++, b += (w + 4)){
			for( x = 0; x < w; x++){
				c = b + x;

				BRI_CONV2( c, diff, bri);
				ALP_CONV2( c);
			}
		}
	}
}

/*
 * RGB¤ΤӥåȥޥåפȤߤʤΥꥵԤؿ
 */
COLOR_RGB* resize_rgb_map( COLOR_RGB *bmp, int w, int h, int wpct, int hpct)
{
	int		x, y, nx, ny, xs, ys;
	int		dw, dh, rw, rh;
	COLOR_RGB *ret, *d;

	dw = (w * wpct) / 100;
	dh = (h * hpct) / 100;

	rw = (wpct - 1) / 100 + 1;
	rh = (hpct - 1) / 100 + 1;

	if( ( ret = (COLOR_RGB*)malloc(
	  sizeof( COLOR_RGB) * (dw + 1) * (dh + 1))) == NULL){
		return NULL;
	}

	/* 0ꥢ */
	memset( ret, 0, sizeof( COLOR_RGB) * dw * dh);

	/*
	 * ʰץեѴ(ꥵΤ)
	 *    ι®Τ˾˱ƽʬ
	 */
	if( wpct <= 100 && hpct <= 100){
		/* Ĳ100%ʲξ */
		for( y = 0; y < h; y++, bmp += w){
			for( x = 0; x < w; x++){
				d = ret + (((y * hpct) / 100) * dw) + ((x * wpct) / 100);

				d->r += bmp[x].r;
				d->g += bmp[x].g;
				d->b += bmp[x].b;
				d->a += bmp[x].a;
				d->t += bmp[x].t;
				d->c++;
			}
		}
	} else if( hpct <= 100){
		/* 100%ۤ */
		for( y = 0; y < h; y++, bmp += w){
			for( x = 0; x < w; x++){
				nx = (x * wpct) / 100;
				ny = (y * hpct) / 100;

				d = ret + (ny * dw) + nx;

				for( xs = 0; (xs < rw) && (( nx + xs) < dw); xs++, d++){
					d->r += bmp[x].r;
					d->g += bmp[x].g;
					d->b += bmp[x].b;
					d->a += bmp[x].a;
					d->t += bmp[x].t;
					d->c++;
				}
			}
		}
	} else if( wpct <= 100){
		/* Ĥ100%ۤ */
		for( y = 0; y < h; y++, bmp += w){
			for( x = 0; x < w; x++){
				nx = (x * wpct) / 100;
				ny = (y * hpct) / 100;

				d = ret + (ny * dw) + nx;

				for( ys = 0; (ys < rh) && (( ny + ys) < dh); d += dw, ys++){
					d->r += bmp[x].r;
					d->g += bmp[x].g;
					d->b += bmp[x].b;
					d->a += bmp[x].a;
					d->t += bmp[x].t;
					d->c++;
				}
			}
		}
	} else {
		/* Ĳ100%ۤ(ǰ) */
		for( y = 0; y < h; y++, bmp += w){
			for( x = 0; x < w; x++){
				nx = (x * wpct) / 100;
				ny = (y * hpct) / 100;

				d = ret + (ny * dw) + nx;

				for( ys = 0; (ys < rh) && (( ny + ys) < dh); d += dw, ys++){
					for( xs = 0; (xs < rw) && (( nx + xs) < dw); xs++){
						d[xs].r += bmp[x].r;
						d[xs].g += bmp[x].g;
						d[xs].b += bmp[x].b;
						d[xs].a += bmp[x].a;
						d[xs].t += bmp[x].t;
						d[xs].c++;
					}
				}
			}
		}
	}

	for( d = ret, y = 0; y < dh; y++){
		for( x = 0; x < dw; x++, d++){
			if( d->c > 1){
				d->r /= d->c;
				d->g /= d->c;
				d->b /= d->c;
				d->a /= d->c;
				if( d->t > 0){
					d->t = 1;
				} else {
					d->t = 0;
				}
			}
        }
    }

	return ret;
}

/*
 * RGB¤Τ󤫤 mglput_pixstream_rectǻѤǤ
 * int롣
 *
 *	ϰʲ̤
 *		s		source bitmap (RGB color)
 *		w		width
 *		h		height
 *		f		flag
 *		wpct	width percentage(distination width)
 *		hpct	height percentage(distination height)
 */
int* mk_mgl_image( COLOR_RGB* s, int w, int h, int f, int b, int *dw, int *dh)
{
	int	y, x, wpct, hpct;
	int *r, *r2;
	COLOR_HSB *d, *c;
	COLOR_RGB *bup = NULL;

	if( dw == NULL){
		wpct = 100;
	} else if( f & WIDTH_DOT){ 
		// ɥåȿꤵƤѴ
		wpct = ( *dw * 100) / w;
    } else if( f & WIDTH_PERCENT) {
        wpct = *dw;
    } else {
        wpct = 100;
    }

	if( dh == NULL){
		hpct = 100;
	} else if( f & HEIGHT_DOT){ 
		// ɥåȿǹ⤵ꤵƤѴ
		hpct = ( *dh * 100) / h;
    } else if( f & HEIGHT_PERCENT){
		hpct = *dh;
    } else {
		hpct = 100;
    }

	if( wpct != 100 || hpct != 100){
		/* ĲΨ100%ʳλϥꥵԤ */
		if( ( s = resize_rgb_map( s, w, h, wpct, hpct)) == NULL){
			return NULL;
		}
		bup = s;
	}

	w = (w * wpct) / 100;
	h = (h * hpct) / 100;

	/*
	 * ѴѤHSBХåեγ
     * (ditherλΤ˲4Ĥ2;͵ΰ;͵äƤ
	 *  ΰ賰ȽԤɬפ̵ʤΤǸ塹Ψɤ)
	 */
	d = (COLOR_HSB*)malloc( sizeof( COLOR_HSB) * ( w + 4 ) * ( h + 2));
	if( d == NULL){
		if( bup != NULL){
			free( bup);
		}
		return NULL;
	}

	// RGB bitmap to HSB bitmap
    if( f & CHANGE_GRAYSCALE){
		for( y = 0, c = d + 2; y < h; y++, c += ( w + 4)){
			for( x = 0; x < w; x++){
				RGBtoHSB_gs( s++, c + x);
			}
		}
	} else {
		for( y = 0, c = d + 2; y < h; y++, c += ( w + 4)){
			for( x = 0; x < w; x++){
				RGBtoHSB( s++, c + x);
			}
		}
	}

 	// 
	if( f & DEPTH_2BPP){
		cut_off2( d, w, h, f, b);
	} else {
		cut_off12( d, w, h, f, b);
	}

    // intѴ
	r = (int*)malloc( sizeof( int) * w * h);
	if( r == NULL){
		free( d);
		if( bup != NULL){
			free( bup);
		}
		return NULL;
	}

	/*
	 * MGL imageǡѴ
     *	ե饰WITH_ALPHAꤵƤϦͤƬդ
	 *	(ξTRANSPARENTӥåȤϻȤʤ)
	 */
	if( f & WITH_ALPHA){
		for( y = 0, r2 = r, c = d + 2; y < h; y++, c += ( w + 4)){
			for( x = 0; x < w; x++){
				*r2++ = c[x].a << 12 | c[x].h << 8 | c[x].s << 4 | c[x].b;
			}
		}
	} else {
		for( y = 0, r2 = r, c = d + 2; y < h; y++, c += ( w + 4)){
			for( x = 0; x < w; x++){
				*r2++ = MGL_DITHER_BIT |
							 c[x].t << 13 | c[x].h << 8 | c[x].s << 4 | c[x].b;
			}
		}
	}

	free( d);
	if( bup != NULL){
		free( bup);
	}

	if( dw != NULL) *dw = w;
	if( dh != NULL) *dh = h;

	return r;
}

/*
 * mgl_image to packed mgl_image convert function
 *  ϰʲ̤
 *		s		mgl_image(int*)
 *		d		packed mgl_image(void *)
 *		f		mk_mgl_imageϤ	
 *		n		ǿ(width * height)
 */
int
pack_mgl_image( int* s, void* d, int f, int n)
{
	int			ret;	// packΥ
	int			i;
	u_int8_t	*dst = d;

	switch( f & DEPTH_MASK){
	case 0x00000000:	// 16BPP
		for( i = 0; i < n; i++){
			u_int16_t p = *s++;
			
			*dst++ =  p >> 8;
			*dst++ =  p & 0xff;
		}
		ret = n * 2;
		break;

	case 0x00000100: // 4BPP
		ret = n;
		for( i = 0; i < ret; i++){
			u_int8_t p;

			/*
			 * COLORեޥåȤOPեɤBեɤΤߤ
			 * ¸(1 == 8bit)
			 */
			p = ((*s & 0x0000f000) >> 8) | ( *s & 0x0000000f);
			s++;
			*dst++ = p;
		}
		break;

	case 0x00000300: // 2BPP
		ret = n / 2;
		if( f & WITH_ALPHA){
			for( i = 0; i < ret; i++){
				u_int8_t p1, p2;

				/*
				 * COLORեޥåȤOPե(TRANSPARENT)
				 * Bե(2bit)Τߤ¸(1 == 4bit)
				 */
				p1 = ((*s & 0x00003000) >> 10) |( *s & 0x00000003);
				s++;
				p2 = ((*s & 0x00003000) >> 10) |( *s & 0x00000003);
				s++;
				*dst++ = p1 << 4 | p2;
			}
		} else {
			for( i = 0; i < ret; i++){
				u_int8_t p1, p2;

				/*
				 * COLORեޥåȤOPե(TRANSPARENT)
				 * Bե(2bit)Τߤ¸(1 == 4bit)
				 */
				p1 = ((*s & 0x00006000) >> 11) |( *s & 0x00000003);
				s++;
				p2 = ((*s & 0x00006000) >> 11) |( *s & 0x00000003);
				s++;
				*dst++ = p1 << 4 | p2;
			}
		}
		break;

	default:
		ret = 0;
	}

	return ret;
}
