/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001 TAJIRI Yasuhiro <tajiri@venus.dti.ne.jp>
 * Original X11/XLCore.c
 *  Copyright (c) 2001, 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: SDLCore.c,v 1.13 2002/03/03 15:37:01 tf Exp $
 */

#include <stdio.h>

#include "../aladder.h"
#include "sdlladder.h"

static void
SDLMotion(SDL_Event *ev)
{
    SDL_MouseMotionEvent me = (SDL_MouseMotionEvent) ev->motion;
    Motion(me.x, me.y);
}

static void
SDLClickLeft(SDL_Event *ev)
{
    ClickButton1();
}

static void
SDLClickRight(SDL_Event *ev)
{
    ClickButton2();
}

static void
SDLCursorUp(SDL_Event *ev)
{
    CursorUp();
}

static void
SDLCursorDown(SDL_Event *ev)
{
    CursorDown();
}

static void
SDLKeyCG(SDL_Event *ev)
{
    KeyCG();
}

static void
SDLKeySkip(SDL_Event *ev)
{
    KeySkip();
}

/* 
 *  VRAM 
 */
static void
FlushWindow()
{
    SDLData *xp = (SDLData *) alCore.DependData;
    set_sdlimage(alCore.vram, xp->image, WIDTH, HEIGHT);
    SDLImagePut(xp->image);
    SDL_UpdateRect(xp->display, 0, 0, 0, 0);
}

static void
FlushWindowArea(int x, int y, int w, int h)
{
    SDLData *xp = (SDLData *) alCore.DependData;
    set_sdlimage_area(alCore.vram, xp->image, x, y, w, h);
    SDLImagePutArea(xp->image, x, y, w, h);
    SDL_UpdateRect(xp->display, x, y, w, h);
}

static void
ClearWindow()
{
    SDLData *xp = (SDLData *) alCore.DependData;
    RedisplayScene(alCore.work, SCENE);
    ALIB.imageDisplay(alCore.brightness);
    set_sdlimage(alCore.vram, xp->image, WIDTH, HEIGHT);
    SDLImagePut(xp->image);
    SDL_UpdateRect(xp->display, 0, 0, 0, 0);
}

static void
SetFullScreenMode(int mode)
{
    SDLData *xp = (SDLData *) alCore.DependData;

    if (mode) {
	if (CONFIG.window_size == CONFIG_WINDOW) {
	    SDL_WM_ToggleFullScreen(xp->display);
	    CONFIG.window_size = CONFIG_FULLSCREEN;
	}
    } else {
	if (CONFIG.window_size == CONFIG_FULLSCREEN) {
	    SDL_WM_ToggleFullScreen(xp->display);
	    CONFIG.window_size = CONFIG_WINDOW;
	}
    }
}

static void
Quit()
{
    ALIB.soundClose();
    DeleteSDL();
    exit(0);
}

/* 
 * ٥Ƚ
 */
#define SDL_BUTTON_WHEEL_UP 4
#define SDL_BUTTON_WHEEL_DOWN 5
static void
Flip()
{
    SDL_Event e;
    int i;
    while (SDL_PollEvent(&e)) {
      flipnext:
	switch (e.type) {
	case SDL_MOUSEMOTION:
	    SDLMotion(&e);
	    while (SDL_PollEvent(&e)) {
		if (e.type != SDL_MOUSEMOTION) {
		    SDLMotion(&e);
		    goto flipnext;
		}
	    }
	    SDLMotion(&e);
	    break;
	case SDL_MOUSEBUTTONDOWN:
	    switch (e.button.button) {
	    case SDL_BUTTON_LEFT:
		SDLClickLeft(&e);
		break;
	    case SDL_BUTTON_RIGHT:
		SDLClickRight(&e);
		break;
	    case SDL_BUTTON_WHEEL_UP:
		SDLCursorUp(&e);
		break;
	    case SDL_BUTTON_WHEEL_DOWN:
		SDLCursorDown(&e);
		break;
	    }
	    break;
	case SDL_KEYDOWN:
	    switch (e.key.keysym.sym) {
	    case SDLK_UP:
		SDLCursorUp(&e);
		break;
	    case SDLK_DOWN:
		SDLCursorDown(&e);
		break;
	    case SDLK_RETURN:
	    case SDLK_KP_ENTER:
		SDLClickLeft(&e);
		break;
	    case SDLK_SPACE:
		SDLKeyCG(&e);
		break;
	    case SDLK_ESCAPE:
		SDLClickRight(&e);
		break;
	    case SDLK_q:
		if (e.key.keysym.mod == KMOD_LALT
		    || e.key.keysym.mod == KMOD_RALT)
		    Quit();
		break;
	    case SDLK_s:
		if (e.key.keysym.mod == KMOD_LALT
		    || e.key.keysym.mod == KMOD_RALT)
		    SDLKeySkip(&e);
		break;
	    }
	    break;
	case SDL_QUIT:
	    Quit();
	default:
	    dprintf((stderr, "ev %x\n", e.type));
	    break;
	}
    }
}

void
SDLCoreBindRequestTable()
{
    REQTBL.flushWindow = FlushWindow;
    REQTBL.flushWindowArea = FlushWindowArea;
    REQTBL.clearWindow = ClearWindow;
    REQTBL.getFontGryph = get_sdlfont;
    REQTBL.flip = Flip;
    REQTBL.destroy = DeleteSDL;
    REQTBL.setFullScreenMode = SetFullScreenMode;
}
