/* 
 * Tenshi no Hashigo for X
 * Original X11/xfont.c
 *    Copyright (c) 2001 TF <tf@denpa.org>
 * Copyright (c) 2001 TAJIRI Yasuhiro <tajiri@venus.dti.ne.jp>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: sdlfont.c,v 1.9 2002/03/03 15:37:01 tf Exp $
 */
#include <SDL.h>
#include <SDL_ttf.h>
#include "../aladder.h"
#include "codeconv.h"

static SDL_Surface *disp, *font_buf;;
static TTF_Font *font;

static SDL_Color white = { 0xFF, 0xFF, 0xFF, 0 };

void
InitFont(SDL_Surface *d, char *fname, int ptsize)
{
    TTF_Font *f;
    Uint32 rmask, gmask, bmask, amask;
    /* Initialize the TTF library */
    if (TTF_Init() < 0) {
	fprintf(stderr, "Couldn't initialize TTF: %s\n", SDL_GetError());
	exit(2);
    }
    atexit(TTF_Quit);
    f = TTF_OpenFont(fname, ptsize);
    if (f == NULL) {
	fprintf(stderr, "Couldn't load %d pt font from %s: %s\n",
		ptsize, fname, SDL_GetError());
	exit(2);
    }
    disp = d;
    font = f;
    // finfo.width = TTF_FontHeight(font) + XPAD;
    // finfo.top = 0;
    // finfo.top = TTF_FontAscent(font);
    // finfo.pitch = TTF_FontAscent(font) + TTF_FontDescent(font) + YPAD;
    finfo.pitch = (HEIGHT - YOFFSET) / (TEXT_HEIGHT + 1);

    rmask = 0xff000000;
    gmask = 0x00ff0000;
    bmask = 0x0000ff00;
    amask = 0x000000ff;

    font_buf =
	SDL_CreateRGBSurface(SDL_SWSURFACE, FONT_WIDTH, FONT_HEIGHT, 32,
			     rmask, gmask, bmask, amask);
    if (font_buf == NULL) {
	fprintf(stderr, "CreateRGBSurface failed: %s\n", SDL_GetError());
	exit(1);
    }
}

Uint32
getpixel(SDL_Surface *surface, int x, int y)
{
    int bpp = surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to retrieve */
    Uint8 *p = (Uint8 *) surface->pixels + y * surface->pitch + x * bpp;

    switch (bpp) {
    case 1:
	return *p;

    case 2:
	return *(Uint16 *) p;

    case 3:
	if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
	    return p[0] << 16 | p[1] << 8 | p[2];
	else
	    return p[0] | p[1] << 8 | p[2] << 16;

    case 4:
	return *(Uint32 *) p;

    default:
	return 0;		/* shouldn't happen, but avoids warnings */
    }
}

/* 
 * SDL ΥեȾ(ӥåȥޥåפ)
 */
ALFont
get_sdlfont(int code)
{
    static Uint16 unicode_text[2];
    SDL_Surface *text;
    SDL_Rect dstrect;
    ALFont alfont;
    int i, j;
    u_char *p =
	(u_char *) calloc(FONT_WIDTH * FONT_HEIGHT, sizeof(u_char));

    if (code > 0xff) {
	unicode_text[0] = codeconv_euc_to_unicode(code & 0xffff);
    } else {
	unicode_text[0] = code;
    }
    text = TTF_RenderUNICODE_Solid(font, unicode_text, white);

    for (i = 0; i < text->h; i++) {
	u_char *q = p + i * text->w;
	for (j = 0; j < text->w; j++, q++) {
	    if (getpixel(text, j, i)) {
		*q = 0xff;
	    }
	}
    }

    SDL_FreeSurface(text);

    alfont.bitmap = (void *) p;
    alfont.width = text->w;
    return alfont;
}
