/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001 TAJIRI Yasuhiro <tajiri@venus.dti.ne.jp>
 * Original X11/sdlimage.c
 *  Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: sdlimage.c,v 1.5 2002/03/03 15:37:01 tf Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include "sdlimage.h"
#include "../aladder.h"

static int tcmap_idx = 0;

static int a_map[17][512];	/* alpha factor */
static int i_map[33][256];	/* intensity factor */


SDLImage *
SDLImageCreate(SDL_Surface *d, int width, int height)
{
    int major, minor;
    SDLImage *image;
    int depth;
    if ((image = (SDLImage *) malloc(sizeof(SDLImage))) == NULL) {
	perror("SDLImageCreate:malloc");
	exit(1);
    }

    image->display = d;
    image->xoffset = 0;
    image->yoffset = 0;
    image->width = width;
    image->height = height;

    image->offscreen =
	SDL_AllocSurface(SDL_ANYFORMAT, width, height, 32, 0, 0, 0, 0);
    if (image->offscreen == NULL) {
	perror("SDL_AllocSurface");
	exit(1);
    }
    return image;
}
void
SDLImageDestroy(SDLImage *image)
{
    if (image == NULL)
	return;
    SDL_FreeSurface(image->offscreen);
    free(image);
}

void
SDLImagePut(SDLImage *image)
{
    SDL_Rect r_src, r_dst;
    setRect(r_src, 0, 0, image->width, image->height);
    setRect(r_dst, 0, 0, image->width, image->height);
    SDL_BlitSurface(image->offscreen, &r_src, image->display, &r_dst);
}

void
SDLImagePutArea(SDLImage *image, int x, int y, int w, int h)
{
    SDL_Rect r_src, r_dst;
    setRect(r_src, x, y, w, h);
    setRect(r_dst, x, y, w, h);
    SDL_BlitSurface(image->offscreen, &r_src, image->display, &r_dst);
}

/* 
 * interface functions
 */
#define SXIMAGE_TPIXEL_SCREEN(s,r,g,b)\
(s->tcmap->red[r]|s->tcmap->green[g]|s->tcmap->blue[b])
#define RGB_PIX24(r,g,b) (Uint32)((((r) << 16) | ((g) << 8) | (b)      ))

void
set_sdlimage(Image *image, SDLImage *sdlimage, int width, int height)
{
    int x, y;
    Uint32 *p_dst;
    Uint8 *p_src, *bitmap;
    SDL_Surface *s;
    SDL_Rect r_src, r_dst;
    s = sdlimage->offscreen;
    bitmap = image->bitmap;
    SDL_LockSurface(s);
    p_dst = s->pixels;
    for (y = 0; y < HEIGHT; y++) {
	p_src = bitmap + (WIDTH * 3 * y);
	for (x = 0; x < WIDTH; x++) {
	    *(p_dst++) = RGB_PIX24(p_src[2], p_src[1], p_src[0]);
	    p_src += 3;
	}

    }
    SDL_UnlockSurface(s);
}

void
set_sdlimage_area(Image *image, SDLImage *sdlimage, int x0, int y0,
		  int width, int height)
{
    int x, y;
    Uint32 *p_dst;
    Uint8 *p_src, *bitmap;
    SDL_Surface *s;
    SDL_Rect r_src, r_dst;
    s = sdlimage->offscreen;
    bitmap = image->bitmap;
    SDL_LockSurface(s);
    for (y = 0; y < height; y++) {
	p_src = bitmap + (x0 + WIDTH * (y0 + y)) * 3;
	p_dst = (Uint32 *) s->pixels + x0 + WIDTH * (y0 + y);
	for (x = 0; x < width; x++) {
	    *p_dst++ = RGB_PIX24(p_src[2], p_src[1], p_src[0]);
	    p_src += 3;
	}
    }
    SDL_UnlockSurface(s);
}

void
InitXLImageProcTable()
{
    int i, j;

    /* alpha map */
    for (i = 0; i < 17; i++) {
	for (j = -255; j < 256; j++) {
	    a_map[i][j + 255] = i * j / 16;
	}
    }
    /* intensity map */
    for (i = 0; i < 33; i++) {
	for (j = 0; j < 256; j++) {
	    i_map[i][j] = i * j / 32;
	}
    }
}

#if 0
/* main procedure for test */
int
main(int argc, char *argv[])
{
    XLImage *image;
    Display *d;
    Window w;
    int width, height;
    int i;

    width = 640;
    height = 480;

    d = XOpenDisplay(NULL);
    w = XCreateSimpleWindow(d, RootWindow(d, 0), 400, 400, width, height,
			    2, 0, 0);
    XMapWindow(d, w);

    image = SDLImageCreate(d, w, width, height);

    for (i = 255; i > 0; i -= 4) {
	put_image(image, width, height, i);
	XLImagePut(image);
	XCopyArea(d, image->pixmap, w, image->gc, 0, 0, width, height, 0,
		  0);
	XSync(d, False);
    }

    XLImageDestroy(image);

    return 0;
}
#endif
