/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001, 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: XLCore.c,v 1.29 2002/03/03 15:37:01 tf Exp $
 */
#include <stdio.h>

#include <X11/Xlib.h>
#include <X11/StringDefs.h>

#include "XLCoreP.h"
#include "../aladder.h"
#include "xladder.h"

static void ClassInitialize(void);
static void Initialize(Widget, Widget, ArgList args, Cardinal *num);
static void Destroy(Widget);
static void Realize(Widget, XtValueMask *, XSetWindowAttributes *);
static void Redisplay(Widget widget, XEvent *ev, Region region);
static void Resize(Widget widget);

static void X11Motion(Widget w, XEvent *ev, String *p, Cardinal *n);

static void X11ClickLeft(Widget w, XEvent *ev, String *p, Cardinal *n);
static void X11ClickRight(Widget w, XEvent *ev, String *p, Cardinal *n);
static void X11KeySkip(Widget w, XEvent *ev, String *p, Cardinal *n);
static void X11KeyCG(Widget w, XEvent *ev, String *p, Cardinal *n);
static void X11CursorUp(Widget widget, XEvent *ev, String *p, Cardinal *n);
static void X11CursorDown(Widget widget, XEvent *ev, String *p,
			  Cardinal *n);

static void FlushWindow();
static void ClearWindow();
static void Flip();

ALFont get_xfont(int code);

#define offset(field) XtOffsetOf(XLCoreRec, field)
static XtResource resources[] = {
    {"sfont", "sFont", XtRFontStruct, sizeof(XFontStruct *),
     offset(xlcore.sfont), XtRString, XtDefaultFont},
    {"mfont", "mFont", XtRFontStruct, sizeof(XFontStruct *),
     offset(xlcore.mfont), XtRString, XtDefaultFont},
};

#define Win XtWindow(xp->core)
#define Disp XtDisplay(xp->core)

static XtActionsRec actions[] = {
    {"motion", X11Motion},
    {"click_left", X11ClickLeft},
    {"click_right", X11ClickRight},
    {"key_skip", X11KeySkip},
    {"key_cg", X11KeyCG},
    {"cursor_up", X11CursorUp},
    {"cursor_down", X11CursorDown},
};

static char translations[] =
    "<Motion>: motion()\n"
    "<Btn1Down>: click_left()\n"
    "<Btn3Down>: click_right()\n"
    "<Key>Return: click_left()\n"
    "<Key>KP_Enter: click_left()\n"
    "<Key>Escape: click_right()\n"
    "Alt<Key>s: key_skip()\n"
    "<Key>space: key_cg()\n"
    "<Btn4Down>: cursor_up()\n" "<Btn5Down>: cursor_down()\n"
    "<Key>Up: cursor_up()\n" "<Key>Down: cursor_down()";

#define superclass (&coreClassRec)
XLCoreClassRec xlCoreClassRec = {
    {
     /* superclass */ (WidgetClass) superclass,
     /* class_name */ "XLCore",
     /* size */ sizeof(XLCoreRec),
     /* class_initialize */ ClassInitialize,
     /* class_part_initialize */ NULL,
     /* Class init'ed */ FALSE,
     /* initialize */ Initialize,
     /* initialize_hook */ NULL,
     /* realize */ Realize,
     /* actions */ actions,
     /* num_actions */ XtNumber(actions),
     /* resources */ resources,
     /* resource_count */ XtNumber(resources),
     /* xrm_class */ NULLQUARK,
     /* compress_motion */ TRUE,
     /* compress_exposure */ TRUE,
     /* compress_enterleave */ TRUE,
     /* visible_interest */ FALSE,
     /* destroy */ Destroy,
     /* resize */ Resize,
     /* expose */ Redisplay,
     /* set_values */ NULL,
     /* set_values_hook */ NULL,
     /* set_values_almost */ XtInheritSetValuesAlmost,
     /* get_values_hook */ NULL,
     /* accept_focus */ NULL,
     /* intrinsics version */ XtVersion,
     /* callback offsets */ NULL,
     /* tm_table */ translations,
     /* query_geometry */ NULL,
     /* display_accelerator */ NULL,
     /* extension */ NULL
     }
};

WidgetClass xlCoreWidgetClass = (WidgetClass) & xlCoreClassRec;

/* ------------------------------------------------------------------- */
static void
ClassInitialize(void)
{
    InitXLImageProcTable();
}

static void
Initialize(Widget req, Widget new, ArgList args, Cardinal *num)
{
    XLCoreWidget cw = (XLCoreWidget) new;
    CW.image = NULL;

#if 0
    CW.sgc = DefaultGC(XtDisplay(cw), XScreenNumberOfScreen(XtScreen(cw)));
    CW.mgc = DefaultGC(XtDisplay(cw), XScreenNumberOfScreen(XtScreen(cw)));
    InitFont(XtDisplay(cw), CW.sgc, CW.sfont);
    InitFont(XtDisplay(cw), CW.mgc, CW.mfont);
#endif
    InitFont(XtDisplay(cw), CW.sfont, CW.mfont);
}

static void
Destroy(Widget widget)
{
    XLCoreWidget cw = (XLCoreWidget) widget;

    if (CW.image)
	XLImageDestroy(CW.image);
}

static void
Realize(Widget widget, XtValueMask *value_mask, XSetWindowAttributes *attr)
{
    XLCoreWidget cw = (XLCoreWidget) widget;
    Display *dpy = XtDisplay(widget);
    Window win;

    XtCreateWindow(widget, (u_int) InputOutput,
		   (Visual *) CopyFromParent, *value_mask, attr);
    win = XtWindow(widget);

    CW.image = XLImageCreate(dpy, win, CORE_WIDTH, CORE_HEIGHT);

    XtVaSetValues(widget, XtNbackground, 0, NULL);
}

static void
Redisplay(Widget widget, XEvent *ev, Region region)
{
    if (XtIsRealized(widget)) {
	// XLCoreWidget cw = (XLCoreWidget) widget;
	// ̤κ
	FlushWindow();
    }
}

static void
Resize(Widget widget)
{
    if (XtIsRealized(widget)) {
	// XLCoreWidget cw = (XLCoreWidget) widget;
	// եåȤѹ
    }
}

static void
X11Motion(Widget widget, XEvent *ev, String *p, Cardinal *n)
{
    XMotionEvent *me = (XMotionEvent *) ev;
    Motion(me->x, me->y);
}

static void
X11ClickLeft(Widget widget, XEvent *ev, String *p, Cardinal *n)
{
    ClickButton1();
}

static void
X11ClickRight(Widget widget, XEvent *ev, String *p, Cardinal *n)
{
    ClickButton2();
}

static void
X11KeySkip(Widget widget, XEvent *ev, String *p, Cardinal *n)
{
    KeySkip();
}

static void
X11KeyCG(Widget widget, XEvent *ev, String *p, Cardinal *n)
{
    KeyCG();
}

static void
X11CursorUp(Widget widget, XEvent *ev, String *p, Cardinal *n)
{
    CursorUp();
}

static void
X11CursorDown(Widget widget, XEvent *ev, String *p, Cardinal *n)
{
    CursorDown();
}

/* 
 *  VRAM ²̤
 */
static void
FlushWindow()
{
    X11Data *xp = (X11Data *) alCore.DependData;
    XLCoreWidget cw = (XLCoreWidget) xp->core;

    set_ximage(alCore.vram, CW.image, CORE_WIDTH, CORE_HEIGHT);
    XLImagePut(CW.image);
    XSync(Disp, False);
}

static void
FlushWindowArea(int x, int y, int width, int height)
{
    X11Data *xp = (X11Data *) alCore.DependData;
    XLCoreWidget cw = (XLCoreWidget) xp->core;

    set_ximage_area(alCore.vram, CW.image, x, y, width, height);
    XLImagePutArea(CW.image, x, y, width, height);
    XSync(Disp, False);
}

static void
ClearWindow()
{
    X11Data *xp = (X11Data *) alCore.DependData;
    XLCoreWidget cw = (XLCoreWidget) xp->core;

    RedisplayScene(alCore.work, SCENE);
    ALIB.imageDisplay(alCore.brightness);
    set_ximage(alCore.vram, CW.image, CORE_WIDTH, CORE_HEIGHT);
    XLImagePut(CW.image);
    XSync(Disp, False);
}

static void
SetFullScreenMode(int mode)
{
    if (mode) {
	if (CONFIG.window_size == CONFIG_WINDOW)
	    FullScreenMode();
    } else {
	if (CONFIG.window_size == CONFIG_FULLSCREEN)
	    WindowMode();
    }
}

void
SetFocus(Widget pw, XEvent *ev, String *p, Cardinal *n)
{
#ifdef XVIDMODE
    X11Data *xp = (X11Data *) alCore.DependData;

    if (CONFIG.window_size == CONFIG_FULLSCREEN) {
	XSetInputFocus(Disp, Win, RevertToNone, CurrentTime);
    }
#endif
}

/* 
 * ٥Ƚ
 */
static void
Flip()
{
    X11Data *xp = (X11Data *) alCore.DependData;
    XLCoreWidget cw = (XLCoreWidget) xp->core;

    ALIB.soundStatus();
    ALIB.musicStatus();
    XSync(XtDisplay((Widget) cw), False);

    while (!XtIsRealized((Widget) cw) || XtAppPending(xp->app)) {
	XEvent event;
	XtAppNextEvent(xp->app, &event);
	XtDispatchEvent(&event);
    }

    if (XtAppGetExitFlag(xp->app)) {
	LongJump(JUMP_END);
    }
}

void
XLCoreBindRequestTable()
{
    REQTBL.flushWindow = FlushWindow;
    REQTBL.flushWindowArea = FlushWindowArea;
    REQTBL.clearWindow = ClearWindow;
    REQTBL.getFontGryph = get_xfont;
    REQTBL.flip = Flip;
    REQTBL.destroy = DeleteX11;
    REQTBL.setFullScreenMode = SetFullScreenMode;
}
