/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: vidmode.c,v 1.2 2002/03/03 15:37:01 tf Exp $
 */
#include <stdio.h>
#include <stdlib.h>

#include "../aladder.h"
#include "xladder.h"

static void
RestoreMode()
{
    X11Data *xp = (X11Data *) alCore.DependData;
    if (CONFIG.window_size == CONFIG_FULLSCREEN) {
	XF86VidModeSwitchToMode(xp->vm.dpy_restore, xp->vm.scn_restore,
				xp->vm.modeinfos[0]);
	XSync(xp->vm.dpy_restore, False);
	CONFIG.window_size = CONFIG_WINDOW;
    }
}

static XF86VidModeModeInfo *
CheckModeLinesRes(int count, int w, int h)
{
    int i;
    X11Data *xp = (X11Data *) alCore.DependData;

    for (i = 0; i < count; i++) {
	if (xp->vm.modeinfos[i]->hdisplay == w
	    && xp->vm.modeinfos[i]->vdisplay == h)
	    return xp->vm.modeinfos[i];
    }
    return NULL;
}

static void
ToScreenMode(Widget top)
{
    X11Data *xp = (X11Data *) alCore.DependData;
    XF86VidModeModeInfo *sel;

    sel = CheckModeLinesRes(xp->vm.vid_count, WIDTH, HEIGHT);
    if (sel == NULL &&
	(sel = CheckModeLinesRes(xp->vm.vid_count, 800, 600)) == NULL) {
	fprintf(stderr, "WARNING: Good mode is not found...\n");
	xp->vm.enable_screen_mode = False;
    } else
	if (!XF86VidModeSwitchToMode
	    (XtDisplay(top), XScreenNumberOfScreen(XtScreen(top)), sel)) {
	fprintf(stderr, "WARNING: Can't set modeline.\n");
	xp->vm.enable_screen_mode = False;
    } else {
	XF86VidModeSetViewPort(XtDisplay(top),
			       XScreenNumberOfScreen(XtScreen(top)), 0, 0);
	XWarpPointer(XtDisplay(top),
		     None,
		     RootWindowOfScreen(XtScreen(top)), 0, 0, 0, 0, 0, 0);
	XSync(XtDisplay(top), False);
	XtVaSetValues(top, XtNoverrideRedirect, True, NULL);
	/* in fact, this function should not be used... */
	XtConfigureWidget(top, 0, 0, sel->hdisplay, sel->vdisplay, 0);
	XtAddEventHandler(top, EnterWindowMask, True,
			  (XtEventHandler) SetFocus, NULL);
	CONFIG.window_size = CONFIG_FULLSCREEN;
    }
}

XF86VidModeModeInfo *
SetupVidMode(Widget top)
{
    int major, minor;
    int eventBase, errorBase;
    XF86VidModeModeInfo *sel = NULL;
    X11Data *xp = (X11Data *) alCore.DependData;
    Display *d = XtDisplay(top);

    xp->vm.enable_screen_mode = False;

    if (!XF86VidModeQueryVersion(d, &major, &minor)) {
	xp->vm.enable_screen_mode = False;
	fprintf(stderr, "failed (Can't get the extension version).\n");
	return NULL;
    } else if (!XF86VidModeQueryExtension(d, &eventBase, &errorBase)) {
	xp->vm.enable_screen_mode = False;
	fprintf(stderr, "failed (Can't get the extension info.).\n");
	return NULL;
    } else if (major < MINMAJOR || (major == MINMAJOR && minor < MINMINOR)) {
	xp->vm.enable_screen_mode = False;
	fprintf(stderr,
		"failed (VidModeExtension (%d.%d) is older than required version (%d.%d)).\n",
		major, minor, MINMAJOR, MINMINOR);
	return NULL;
    } else {
	if (!XF86VidModeGetAllModeLines
	    (d, XScreenNumberOfScreen(XtScreen(top)), &(xp->vm.vid_count),
	     &(xp->vm.modeinfos))) {
	    fprintf(stderr, "Can't get modeinfos.\n");
	    xp->vm.enable_screen_mode = False;
	} else
	    if ((sel =
		 CheckModeLinesRes(xp->vm.vid_count, WIDTH,
				   HEIGHT)) == NULL
		&& (sel =
		    CheckModeLinesRes(xp->vm.vid_count, 800,
				      600)) == NULL) {
	    fprintf(stderr, "Good mode isn't found...\n");
	    xp->vm.enable_screen_mode = False;
	} else {
	    xp->vm.enable_screen_mode = True;
	    xp->vm.dpy_restore = d;
	    xp->vm.scn_restore = XScreenNumberOfScreen(XtScreen(top));
	    atexit(RestoreMode);
	}
    }

    if (xp->vm.enable_screen_mode && CONFIG.window_size == CONFIG_FULLSCREEN) {
	ToScreenMode(top);
    }

    return sel;
}

void
FullScreenMode()
{
    X11Data *xp = (X11Data *) alCore.DependData;
    if (!xp->vm.enable_screen_mode
	|| CONFIG.window_size == CONFIG_FULLSCREEN)
	return;

    XtUnmapWidget(xp->top);
    ToScreenMode(xp->top);
    CONFIG.window_size = CONFIG_FULLSCREEN;

    XtMapWidget(xp->top);
    XGrabPointer(XtDisplay(xp->top),
		 XtWindow(xp->top), True, 0,
		 GrabModeAsync, GrabModeAsync,
		 XtWindow(xp->top), None, CurrentTime);

    XMoveWindow(XtDisplay(xp->top), XtWindow(xp->top), 0, 0);
    XWarpPointer(XtDisplay(xp->top),
		 None,
		 RootWindowOfScreen(XtScreen(xp->top)), 0, 0, 0, 0, 0, 0);
    XSync(XtDisplay(xp->top), False);
}

void
WindowMode()
{
    int width = WIDTH;
    int height = HEIGHT;
    X11Data *xp = (X11Data *) alCore.DependData;

    if (!xp->vm.enable_screen_mode || CONFIG.window_size == CONFIG_WINDOW)
	return;

    XtUnmapWidget(xp->top);
    RestoreMode();
    CONFIG.window_size = CONFIG_WINDOW;
    XtRemoveEventHandler(xp->top, EnterWindowMask, True,
			 (XtEventHandler) SetFocus, NULL);
    XtVaSetValues(xp->top, XtNwidth, width, XtNheight, height,
		  XtNoverrideRedirect, False, NULL);
    XtMapWidget(xp->top);
}
