/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001, 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: xfont.c,v 1.10 2002/03/03 15:37:01 tf Exp $
 */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdlib.h>

#include "../aladder.h"

static Display *disp;
static Pixmap font_buf;
static XImage *xi;
static u_char *xi_data;
static XFontStruct *mfont, *sfont;
static GC mfont_gc, sfont_gc;
static long black, white;

/* 
 * X ΥեȾ(ӥåȥޥåפ)
 */
ALFont
get_xfont(int code)
{
    int i, j;
    ALFont font;

    u_char *p =
	(u_char *) calloc(FONT_WIDTH * FONT_HEIGHT, sizeof(u_char));

    if (code < 0x100) {
	u_char ch = code & 0xff;

	XSetForeground(disp, sfont_gc, black);
	XFillRectangle(disp, font_buf, sfont_gc, 0, 0, FONT_WIDTH,
		       FONT_HEIGHT);

	XSetForeground(disp, sfont_gc, white);

	XDrawString(disp, font_buf, sfont_gc, 0, sfont->max_bounds.ascent,
		    &ch, 1);
	XGetSubImage(disp, font_buf, 0, 0, FONT_WIDTH, FONT_HEIGHT, 1,
		     XYPixmap, xi, 0, 0);

	font.width = XTextWidth(sfont, &ch, 1);
    } else {
	XChar2b ch;

	ch.byte1 = (code >> 8) & 0x7f;
	ch.byte2 = code & 0x7f;

	XSetForeground(disp, mfont_gc, black);
	XFillRectangle(disp, font_buf, mfont_gc, 0, 0, FONT_WIDTH,
		       FONT_HEIGHT);

	XSetForeground(disp, mfont_gc, white);


	XDrawString16(disp, font_buf, mfont_gc, 0,
		      mfont->max_bounds.ascent, &ch, 1);
	XGetSubImage(disp, font_buf, 0, 0, FONT_WIDTH, FONT_HEIGHT, 1,
		     XYPixmap, xi, 0, 0);

	font.width = XTextWidth16(mfont, &ch, 1);
    }

    for (i = 0; i < FONT_HEIGHT; i++) {
	u_char *q = p + i * font.width;
	for (j = 0; j < font.width; j++, q++) {
	    if (XGetPixel(xi, j, i)) {
		*q = 0xff;
	    }
	}
    }

    font.bitmap = (void *) p;
    return font;
}

/* 
 * μ
 */
static long
GetPixelByName(Display *disp, const char *name)
{
    Colormap cmap;
    XColor c0, c1;

    cmap = DefaultColormap(disp, 0);
    XAllocNamedColor(disp, cmap, name, &c1, &c0);

    return c1.pixel;
}

void
InitFont(Display *d, XFontStruct *sf, XFontStruct *mf)
{
    disp = d;
    black = GetPixelByName(d, "black");
    white = GetPixelByName(d, "white");

    font_buf =
	XCreatePixmap(d, DefaultRootWindow(d), FONT_WIDTH, FONT_HEIGHT, 1);
    xi_data = (u_char *) malloc(((FONT_WIDTH - 1) / 8 + 1) * FONT_HEIGHT);
    xi = XCreateImage(d, DefaultVisual(d, 0), 1, XYBitmap, 0,
		      (char *) xi_data, FONT_WIDTH, FONT_HEIGHT, 8, 0);

    mfont_gc = XCreateGC(d, font_buf, 0, 0);
    sfont_gc = XCreateGC(d, font_buf, 0, 0);

    if (mf != NULL && mf->fid != None) {
	XSetFont(d, mfont_gc, mf->fid);
    } else {
	fprintf(stderr, "Can't get the font.\n");
	exit(1);
    }

    if (sf != NULL && sf->fid != None) {
	XSetFont(d, sfont_gc, sf->fid);
    } else {
	fprintf(stderr, "Can't get the font.\n");
	exit(1);
    }

    mfont = mf;
    sfont = sf;
    finfo.pitch = mf->max_bounds.ascent + mf->max_bounds.descent + YPAD;
}
