/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001, 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: action.c,v 1.27 2002/03/09 17:41:59 tf Exp $
 */

#include "aladder.h"
#include "event.h"

#define QUEUE_SIZE 32

static int queue[QUEUE_SIZE];
static int front;
static int rear;

#define NEXT(a) (((a) + 1) % QUEUE_SIZE)

/* 
 * ٥ȥ塼ν
 */
void
event_queue_init()
{
    front = rear = 0;
}

void
event_queue_put(int in_data)
{
    if (NEXT(rear) == front) {
	fprintf(stderr, "٥ȥ塼դޤ\n");
	return;
    }
    queue[rear] = in_data;
    rear = NEXT(rear);
}

int
event_queue_get()
{
    int out_data;
    if (front == rear) {
	fprintf(stderr, "塼դƤޤ\n");
	return -1;
    }
    out_data = queue[front];
    front = NEXT(front);
    return out_data;
}

int
event_queue_is_empty()
{
    return front == rear;
}

/* 
 * ΰؤνꥤ٥ (Motion ٥)
 * (ΰ꤬Ťʤäֹ椬㤤Τͥ)
 */
#define TRA_AREA_NUM 255
static int trap_num = 0;
static int trap_selected = -1;
static TrapArea trapArea[TRA_AREA_NUM];

void
trap_area_clear()
{
    trap_selected = -1;
    trap_num = 0;
}

void
trap_area_add(int x, int y, int w, int h)
{
    trapArea[trap_num].id = trap_num;
    trapArea[trap_num].status = OUTSIDE;
    trapArea[trap_num].x = x;
    trapArea[trap_num].y = y;
    trapArea[trap_num].w = w;
    trapArea[trap_num].h = h;
    trap_num++;
}

int
trap_area_selected()
{
    return trap_selected;
}

static int
trap_area_check(int id, int x, int y)
{
    int result, i;

    if (trap_selected > 0 && trap_selected < id) {
	return -1;
    }

    if (x > trapArea[id].x &&
	y > trapArea[id].y &&
	x < trapArea[id].x + trapArea[id].w &&
	y < trapArea[id].y + trapArea[id].h) {
	result = INSIDE;
    } else {
	result = OUTSIDE;
    }

    if (trapArea[id].status == result) {
	return -1;
    }
    for (i = 0; i < trap_num; i++)
	trapArea[i].status = OUTSIDE;
    if (result == INSIDE) {
	trapArea[id].status = result;
	trap_selected = id;
    } else {
	trap_selected = -1;
    }
    return result;
}

/* 
 * ƥ٥ȤФư
 * (AngelEngine Ƥ֤ΤǡľܸƤӽФʤ)
 */
void
action_click_button1()
{
    dprintf((stderr, "button1 tapped[%d]\n", CURRENT_STATE));

    if (ALIB.soundStatus()) {
	ALIB.soundStop();
    }
    clear_cursor();
    WAIT.skip = False;
}

void
action_click_button2()
{
    dprintf((stderr, "button2 tapped[%d]\n", CURRENT_STATE));
}

void
action_key_skip()
{
    dprintf((stderr, "ޤǥåפޤ.\n"));
    alCore.brightness = 10;
    WAIT.skip = True;
}

void
action_key_cg()
{
    dprintf((stderr, "toggle cg mode.\n"));
}

void
action_key_up()
{
    int sel;

    switch (CURRENT_STATE) {
    case STAT_BACKLOG:
	backlog_up();
	break;
    case STAT_WAIT:
	backlog_enter();
	break;
    case STAT_MENU:
    case STAT_ALTN:
	sel = (ATTR_ALL_MASK ^ ATTR_ALTN_MASK) & TEXT.highlight;

	if (sel == 0) {
	    sel = alCore.alternative_num;
	} else {
	    sel--;
	    if (sel == 0)
		sel = 1;
	}
	menu_set_selected(sel);
	break;
    }
}

void
action_key_down()
{
    int sel;

    switch (CURRENT_STATE) {
    case STAT_BACKLOG:
	if (!backlog_down())
	    backlog_leave();
	break;
    case STAT_MENU:
    case STAT_ALTN:
	sel = (ATTR_ALL_MASK ^ ATTR_ALTN_MASK) & TEXT.highlight;

	if (sel == 0) {
	    sel = 1;
	} else {
	    sel++;
	    if (sel > alCore.alternative_num)
		sel = alCore.alternative_num;
	}
	menu_set_selected(sel);
	break;
    }
}

#define is_clickable_text(x,y) (((TEXT.attr[x][y] & ATTR_DICT_MASK) | (TEXT.attr[x][y] & ATTR_ALTN_MASK)) && (x >= 0) && (x < TEXT_WIDTH) && (y >= 0) && (y < TEXT_HEIGHT))
/* 
 * ؤ饤٥ȤΤ뤿 public ؿ
 */
void
Motion(int ex, int ey)
{
    int x, y, i;
    ALIB.coordGraphicsToText(ex, ey, &x, &y);

    /* 褪ӥϥѡ */
    if (TEXT.attr[x][y] != TEXT.highlight && trap_area_selected() < 0) {
	if (is_clickable_text(x, y)) {
	    TEXT.highlight = TEXT.attr[x][y];
	} else {
	    TEXT.highlight = 0;
	}
	menu_draw_frame();
	redraw_text();
	REQTBL.flushWindow();
    }
    // fprintf(stderr, "%d,%d: <%d/%d>", x, y, TEXT.attr[x][y],
    // TEXT.highlight);

    /* ȥåΰ */
    for (i = 0; i < trap_num; i++) {
	switch (trap_area_check(i, ex, ey)) {
	case INSIDE:
	    event_queue_put(EVENT_ENTER_AREA);
	    break;
	case OUTSIDE:
	    event_queue_put(EVENT_LEAVE_AREA);
	    break;
	default:
	}
    }
}

void
ClickButton1()
{
    event_queue_put(EVENT_BUTTON1);
}

void
ClickButton2()
{
    event_queue_put(EVENT_BUTTON2);
}

void
KeySkip()
{
    event_queue_put(EVENT_KEY_SKIP);
}

void
KeyCG()
{
    event_queue_put(EVENT_KEY_CG);
}

void
CursorUp()
{
    event_queue_put(EVENT_KEY_UP);
}

void
CursorDown()
{
    event_queue_put(EVENT_KEY_DOWN);
}
