/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: animation.c,v 1.4 2002/03/03 15:37:00 tf Exp $
 */
#include <unistd.h>

#include "aladder.h"
#include "animation.h"
#include "event.h"

static int aptr;
static AnimationScript ascript;

int
animation_fadein(int arg)
{
    static int i = 0;

    if (i == 0) {
	ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, WHITE);
	ALIB.imageFill(alCore.sub, 0, 0, WIDTH, HEIGHT, BLACK);
    }

    if (!EffectAlphaBlend(&i))
	return False;

    i = 0;
    return True;
}

int
animation_fadeout(int arg)
{
    static int completed = True;
    static timer t;

    if (arg == 0)
	return True;

    if (completed) {
	completed = False;
	init_timer(&t);
	ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, BLACK);
	ALIB.imageCopy(alCore.sub, alCore.vram, 0, 0, 0, 0, WIDTH, HEIGHT);
    }

    get_timer(&t);
    ALIB.imageMerge(alCore.vram, alCore.work, alCore.sub,
		    0xff * t.msec / arg);
    REQTBL.flushWindow();

    if (t.msec > arg)
	completed = True;

    return completed;
}

int
animation_fadeout_pattern(int arg)
{
    static int i = 0;

    if (i == 0) {
	ALIB.imageCopy(alCore.sub, alCore.vram, 0, 0, 0, 0, WIDTH, HEIGHT);
	ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, BLACK);
    }

    if (!FadePattern(&i, True))
	return False;

    i = 0;
    return True;
}

int
animation_whiteout(int arg)
{
    static int i = 0;

    if (i == 0) {
	ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, WHITE);
	ALIB.imageCopy(alCore.sub, alCore.vram, 0, 0, 0, 0, WIDTH, HEIGHT);
    }

    if (!EffectAlphaBlend(&i))
	return False;

    i = 0;
    return True;
}

int
animation_wait(int arg)
{
    push_stack(STAT_ANIM_WAIT);
    return True;
}

int
animation_sleep(int arg)
{
    static int completed = True;
    static timer t;

    if (completed) {
	completed = False;
	init_timer(&t);
    }

    get_timer(&t);
    if (t.msec > arg)
	completed = True;

    return completed;
}

static int
proc_animation_script()
{
    if ((ascript.command[aptr].func) (ascript.command[aptr].arg))
	if (ascript.command[++aptr].func == NULL)
	    return True;

    return False;
}

void
run_animation_engine(AnimationScript as)
{
    timer t;
    int idle;

    aptr = 0;
    ascript = as;

    clear_stack();
    push_stack(STAT_ANIM);

    while (1) {
	init_timer(&t);
	if (!event_queue_is_empty()) {
	    int q = event_queue_get();
	    switch (q) {
	    case EVENT_BUTTON1:
		switch (CURRENT_STATE) {
		case STAT_ANIM_WAIT:
		    pop_stack();
		    break;
		default:
		    as.button_func();
		}
		break;
	    }
	}

	switch (CURRENT_STATE) {
	case STAT_EMPTY:
	case STAT_ANIM_WAIT:
	    break;
	case STAT_ANIM_QUIT:
	    if (animation_fadeout(0))
		goto end;
	    break;
	case STAT_ANIM:
	    if (proc_animation_script())
		goto end;
	    break;
	}
	REQTBL.flip();

	get_timer(&t);
	idle = WAIT_TIME - t.msec * 1000;
	if (WAIT.drop_frame) {
	    WAIT.drop_frame = False;
	    continue;
	}
	if (idle > 0)
	    usleep(idle);
    }
  end:
}
