/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: archive.c,v 1.9 2002/03/03 15:37:00 tf Exp $
 */
#include <stdio.h>
#include <stdlib.h>

#include "aladder.h"

#define BUFLEN 0x10000

u_char buff[BUFLEN];

static void
bit_swap(u_char *_v, int _shift_count)
{
    u_long data;

    data = (_v[3] << 24) | (_v[2] << 16) | (_v[1] << 8) | _v[0];

    data = (data << _shift_count) | (data >> (32 - _shift_count));

    _v[3] = (data >> 24) & 0xff;
    _v[2] = (data >> 16) & 0xff;
    _v[1] = (data >> 8) & 0xff;
    _v[0] = data & 0xff;
}

static void
byte_rotate(u_char *_v, int _len, int _offset)
{
    u_char b[256];
    memcpy(b, _v, _len);
    memcpy(_v, b + _len - _offset, _offset);
    memcpy(_v + _offset, b, _len - _offset);
}

static void
set_name(char *dst, const char *src, int len)
{
    int i, j;

    // for (i = 0, j = 0; i < FNAME_LEN - 1; i++) {
    for (i = 0, j = 0; i < len; i++) {
	if (src[i] == 0 || src[i] < 0)
	    break;
	dst[j++] = src[i];
    }
    dst[i] = '\0';
}

static int
check_type(const char *filename)
{
    char typechar = 0;
    int last = -1;
    int i;

    for (i = 0; i < strlen(filename); i++) {
	if (filename[i] == '\\' || filename[i] == '/')
	    last = i;
    }
    typechar = filename[last + 1];

    switch (typechar) {
    case 'B':
    case 'b':
	return ARC_BMP;
    case 'E':
    case 'e':
	return ARC_SCN;
    case 'V':
    case 'v':
	return ARC_WAV;
    default:
	return ARC_ERR;
    }
}

Archive *
open_archive(char *filename)
{
    Archive *arc;
    int bsize, fnlen;
    int i;

    arc = (Archive *) malloc(sizeof(Archive));
    if (arc == NULL) {
	perror("malloc");
	exit(1);
    }

    arc->fp = fopen(filename, "rb");
    if (arc->fp == NULL) {
	fprintf(stderr, "֤򳫤ޤ(%s)\n\n", filename);
	return NULL;
    }

    arc->type = check_type(filename);

    fseek(arc->fp, -4, SEEK_END);
    fread(buff, 4, 1, arc->fp);

    arc->num =
	(buff[3] << 24) | (buff[2] << 16) | (buff[1] << 8) | buff[0];

    arc->item = (ArchiveItem *) calloc(arc->num, sizeof(ArchiveItem));

    switch (arc->type) {
    case ARC_BMP:
	bsize = 60;
	break;
    case ARC_SCN:
	bsize = 44;
	break;
    case ARC_WAV:
	bsize = 32;
	break;
    default:
	fprintf(stderr, "̤бΥ֤Ǥ(%s)\n", filename);
	return NULL;
    }

    fseek(arc->fp, -bsize * arc->num - 4, SEEK_END);
    for (i = 0; i < arc->num; i++) {
	int j;
	u_char *p;
	memset(buff, 0, BUFLEN);
	if (fread(buff, bsize, 1, arc->fp) != 1) {
	    fprintf(stderr, "եɤ߹ߤ˼Ԥޤ\n");
	    return NULL;
	}
	if (arc->type != ARC_WAV) {
	    for (p = buff, j = 0; j < bsize / 4; j++) {
		bit_swap(p, 13);
		p += 4;
	    }
	    byte_rotate(buff, bsize, 7);

	    for (j = 0; j < bsize; j++) {
		buff[j] ^= 0xd6;
	    }
	} else {
	    for (j = 0; j < bsize; j++) {
		buff[j] ^= 0xde;
	    }
	}

	fnlen = buff[0] | buff[1] << 8 | buff[2] << 16 | buff[3] << 24;
	switch (arc->type) {
	case ARC_BMP:
	    bsize = 60;
	    set_name(arc->item[i].name, buff + 24, fnlen);

	    arc->item[i].osize =
		buff[4] | buff[5] << 8 | buff[6] << 16 | buff[7] << 24;
	    arc->item[i].esize =
		buff[8] | buff[9] << 8 | buff[10] << 16 | buff[11] << 24;
	    arc->item[i].pos =
		buff[12] | buff[13] << 8 | buff[14] << 16 | buff[15] << 24;
	    arc->item[i].width =
		buff[16] | buff[17] << 8 | buff[18] << 16 | buff[19] << 24;
	    arc->item[i].height =
		buff[20] | buff[21] << 8 | buff[22] << 16 | buff[23] << 24;
	    break;
	case ARC_SCN:
	    set_name(arc->item[i].name, buff + 28, fnlen);
	    arc->item[i].pos =
		buff[24] | buff[25] << 8 | buff[26] << 16 | buff[27] << 24;
	    break;
	case ARC_WAV:
	    set_name(arc->item[i].name, buff + 16, fnlen);
	    arc->item[i].osize =
		buff[4] | buff[5] << 8 | buff[6] << 16 | buff[7] << 24;
	    arc->item[i].esize =
		buff[8] | buff[9] << 8 | buff[10] << 16 | buff[11] << 24;
	    arc->item[i].pos =
		buff[12] | buff[13] << 8 | buff[14] << 16 | buff[15] << 24;
	    break;
	default:
	    fprintf(stderr, "̤бΥ֤Ǥ(%s)\n", filename);
	    return NULL;
	}
    }

    return arc;
}

void
print_archive(Archive *arc)
{
    int i;

    switch (arc->type) {
    case ARC_SCN:
	fprintf(stderr, "type: text\n");
	break;
    case ARC_BMP:
	fprintf(stderr, "type: bitmap\n");
	break;
    case ARC_WAV:
	fprintf(stderr, "type: wave\n");
	break;
    default:
	fprintf(stderr, "type: unknown\n");
    }

    for (i = 0; i < arc->num; i++) {
	fprintf(stderr, "%08x: %s", arc->item[i].pos, arc->item[i].name);
	if (arc->type == ARC_BMP) {
	    fprintf(stderr, " (%d, %d)", arc->item[i].width,
		    arc->item[i].height);
	}
	fprintf(stderr, "\n");
    }
}

void
close_archive(Archive *arc)
{
    fclose(arc->fp);
    free(arc);
}

#if 0
/* 
 * ֤Υƥ
 */
int
main(int argc, char *argv[])
{
    Archive *arc;
    int i;
    FILE *fp;
    u_char *res;

    arc = open_archive(argv[1]);
    if (arc == NULL) {
	exit(1);
    }
    print_archive(arc);
    for (i = 0; i < arc->num; i++) {
	switch (arc->type) {
	case ARC_SCN:
	    fprintf(stderr, "extract: %s\n", arc->item[i].name);
	    res = get_text(arc, arc->item[i].name);
	    fp = fopen(arc->item[i].name, "w");
	    {
		u_char *p = res;
		while (*p) {
		    putc(*p++, fp);
		}
	    }
	    fclose(fp);
	    free(res);
	    break;
	case ARC_BMP:
	    get_image(arc, arc->item[i].name);
	    break;
	case ARC_WAV:
	    res = get_sound(arc, arc->item[i].name);
	    fp = fopen(arc->item[i].name, "w");
	    fwrite(res, arc->item[i].esize, 1, fp);
	    fclose(fp);
	    free(res);
	    break;
	default:
	    break;
	}
    }
    close_archive(arc);

    return 0;
}
#endif
