/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: archive_text.c,v 1.6 2002/03/03 15:37:00 tf Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "aladder.h"

#define RBUFSZ    (0xfff + 1)
#define LEN       (0xf + 3)

static u_char *
decode_lz77(FILE *fp)
{
    char rbuf[RBUFSZ + LEN - 1];
    u_int ch, pos, c1, c2, offset, count, i;
    u_int bits = 0;
    u_char *text;
    int tpos = 0;
    int size = 4096;

    text = (u_char *) malloc(size * sizeof(u_char));
    if (text == NULL) {
	perror("malloc");
	exit(1);
    }

    memset(rbuf, 0, RBUFSZ - LEN);
    pos = RBUFSZ - LEN;

    while (!feof(fp)) {
	/* ե饰 (1B) ϲ̤鸫Ƥ */
	if (!((bits >>= 1) & 0x100)) {
	    ch = fgetc(fp);
	    bits = ch | 0xff00;
	}
	/* ӥåȤΩäƤ٥ 1B */
	if (bits & 1) {
	    ch = fgetc(fp);
	    /* 00 ʤ齪 */
	    if (!ch) {
		text[tpos] = 0;
		return text;
	    }
	    rbuf[pos = pos++ & (RBUFSZ - 1)] = ch;
	    text[tpos] = ch;
	    if (++tpos >= size) {
		size *= 2;
		text = realloc(text, size * sizeof(u_char));
	    }
	} else {
	    /* 2B (ab cd) ǰ cab Ĺ d + 3 */
	    c1 = fgetc(fp);
	    c2 = fgetc(fp);
	    offset = c1 | ((c2 & 0xf0) << 4);
	    count = (c2 & 0x0f) + 3;
	    for (i = 0; i < count; i++) {
		ch = rbuf[(offset + i) & (RBUFSZ - 1)];
		rbuf[pos = pos++ & (RBUFSZ - 1)] = ch;
		text[tpos] = ch;
		if (++tpos >= size) {
		    size *= 2;
		    text = realloc(text, size * sizeof(u_char));
		}
	    }
	}
    }

    return NULL;
}

u_char *
get_text(Archive *arc, const char *name)
{
    int i, len;
    u_char *sjis, *euc;

    for (i = 0; i < arc->num; i++) {
	if (!strcasecmp(arc->item[i].name, name))
	    break;
    }
    if (i == arc->num) {
	fprintf(stderr, "ץȤĤޤ(%s)\n", name);
	return NULL;
    }

    fseek(arc->fp, arc->item[i].pos, SEEK_SET);
    sjis = decode_lz77(arc->fp);

    if (sjis == NULL) {
	fprintf(stderr, "ƥȤŸ˼Ԥޤ(%s)\n", name);
	return NULL;
    }

    len = strlen(sjis) + 1;
    euc = (u_char *) malloc(len * sizeof(u_char));
    if (euc == NULL) {
	perror("malloc");
	exit(1);
    }
    sjis2euc(sjis, euc);
    free(sjis);

    return euc;
}
