/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: backlog.c,v 1.5 2002/03/03 15:37:00 tf Exp $
 */

#include "aladder.h"

static int history_pos;

void
backlog_init()
{
    alCore.text_history.y = 0;
    alCore.text_history.loop = False;
}

void
backlog_enter()
{
    if (!HISTORY.loop && HISTORY.y <= TEXT.y) {
	fprintf(stderr, "­\n");
	return;
    }

    dprintf((stderr, "enter backlog mode\n"));

    history_pos = 0;
    push_text();
    push_stack(STAT_BACKLOG);

    print_backlog(HISTORY.y - TEXT.y, TEXT.y);
}

void
backlog_leave()
{
    dprintf((stderr, "leave backlog mode\n"));
    pop_text();
    pop_stack();

    ALIB.imageDisplay(10);
    redraw_text();
    REQTBL.flushWindow();
}

void
backlog_up()
{
    int height;
    int from = HISTORY.y;

    if (TEXT.y + history_pos > BACKLOG_LEN)
	return;

    if (!HISTORY.loop && TEXT.y + history_pos >= HISTORY.y) {
	return;
    }

    history_pos++;

    height = TEXT.y + history_pos;
    height = (height > TEXT_HEIGHT) ? TEXT_HEIGHT : height;
    print_backlog(from - TEXT.y - history_pos, height);
}

int
backlog_down()
{
    int height;
    if (--history_pos < 0)
	return False;

    height = TEXT.y + history_pos;
    height = (height > TEXT_HEIGHT) ? TEXT_HEIGHT : height;

    print_backlog(HISTORY.y - TEXT.y - history_pos, height);

    return True;
}
