/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001, 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: command.c,v 1.53 2002/03/09 17:41:59 tf Exp $
 */
#include <stdlib.h>
#include <string.h>

#include "aladder.h"
#include "command.h"

#ifdef DEBUG
static void
cmd_test(char arg[][ARG_LEN], int num)
{
    int i;

    fprintf(stderr, "[%s]", arg[0]);
    for (i = 1; i < num; i++) {
	fprintf(stderr, "[%s]", arg[i]);
    }
    fprintf(stderr, "\n");
}
#else
#define cmd_test(x,y)
#endif

static int
eval(char arg[][ARG_LEN], int num)
{
    int i;

    for (i = 0; i < num; i++) {
	int result = False;
	FlagList *p = alCore.flag;
	while (p != NULL) {
	    if (!strcmp(p->value, arg[i])) {
		result = True;
		break;
	    }
	    p = p->next;
	}
	if (!result) {
	    dprintf((stderr, "cond:<%s>=False\n", arg[i]));
	    return False;
	}
	dprintf((stderr, "cond:<%s>=True\n", arg[i]));
    }
    return True;
}

static u_char command_buf[SCR_BUF_LEN];

void
set_command(const char *cmd)
{
    strcpy(command_buf, cmd);
}

int
get_command_arg(char arg[ARG_NUM][ARG_LEN])
{
    int num;

    num = script_parse_arg(command_buf + 1, arg);

    return num;
}

int
cmd_Bg(char arg[][ARG_LEN], int num, int interrupt)
{
    static int state = 0, cmd;
    static int lstate, dstate, abstate;

    switch (state) {
    case 0:
	lstate = dstate = abstate = 0;
	cmd_test(arg, num);
	cmd = strtol(arg[2], NULL, 0);

	WorkSceneBg(arg[1]);
	WorkSceneClearAllChara();

	if (cmd < 0)
	    goto bg_break;

	ReflectWorkScene();

	if (WAIT.skip || !CONFIG.use_effect) {
	    ALIB.imageLoad(alCore.work, arg[1]);
	    ALIB.imageDisplay(alCore.brightness);
	    goto bg_end;
	}
	state++;
    case 1:
	if (!Lighten(&lstate))
	    return False;
	state++;
    case 2:
	if (abstate == 0) {
	    switch (cmd) {
	    case 1:
	    case 7:
		ALIB.imageCopy(alCore.sub, alCore.vram, 0, 0, 0, 0, WIDTH,
			       HEIGHT);
		break;
	    case 2:
		ALIB.imageAdd(alCore.sub, alCore.work, 0, 0);
		break;
	    }
	    ALIB.imageLoad(alCore.work, arg[1]);
	}
	switch (cmd) {
	case 1:
	    if (!FadePattern(&abstate, True))
		return False;
	    break;
	case 2:
	    if (!EffectAlphaBlend(&abstate))
		return False;
	    break;
	case 3:
	    if (!WipeParallelVLtoR(&abstate))
		return False;
	    break;
	case 4:
	    if (!WipeParallelVRtoL(&abstate))
		return False;
	    break;
	case 6:
	    if (!WipeParallelHBtoT(&abstate))
		return False;
	    break;
	case 7:
	    if (!WipeLtoR(&abstate, True))
		return False;
	    break;
	case 8:
	    if (!WipeRtoL(&abstate, True))
		return False;
	    break;
	case 0:
	case 10:
	default:
	    dprintf((stderr, "Bg: ̤ <%d>\n", cmd));
	}
	state++;
    case 3:
	if (!Darken(&dstate))
	    return False;
	state++;
    default:
	break;
    }

  bg_end:
    if (!TEXT.clear) {
	redraw_text();
	REQTBL.flushWindow();
    }
  bg_break:
    state = 0;
    return True;
}

/* 
 * CG λȺѤߥե饰ꤹ
 */
int
cmd_CGLook(char arg[][ARG_LEN], int num, int interrupt)
{
    cmd_test(arg, num);

    add_flag(arg[1], &alCore.cg_flag);
    return True;
}

/* 
 * 饯Ωɽ
 */
int
cmd_Chara(char arg[][ARG_LEN], int num, int interrupt)
{
    static int state = 0, cmd;

    /*  */
    if (state == 0) {
	cmd_test(arg, num);
	cmd = strtol(arg[3], NULL, 0);
	if (strstr(arg[1], ".bmp") == NULL) {
	    char *p = arg[1];
	    while (*p != 0)
		p++;
	    strcpy(p, ".bmp");
	    dprintf((stderr, "̾: %s\n", arg[1]));
	}

	WorkSceneChara(arg[2], arg[1]);

	if (cmd < 0)
	    goto chara_break;

	ReflectWorkScene();

	if (WAIT.skip || !CONFIG.use_effect) {
	    RedisplayScene(alCore.work, SCENE);
	    goto chara_end;
	}

	switch (cmd) {
	case 1:
	    ALIB.imageCopy(alCore.sub, alCore.vram, 0, 0, 0, 0, WIDTH,
			   HEIGHT);
	    break;
	case 2:
	    ALIB.imageAdd(alCore.vram, alCore.work, 0, 0);
	    REQTBL.flushWindow();
	    ALIB.imageAdd(alCore.sub, alCore.work, 0, 0);
	    break;
	case 3:
	case 4:
	case 5:
	    break;
	case 7:
	case 8:
	    ALIB.imageCopy(alCore.sub, alCore.vram, 0, 0, 0, 0, WIDTH,
			   HEIGHT);
	    ALIB.imageAdd(alCore.vram, alCore.work, 0, 0);
	    REQTBL.flushWindow();
	    break;
	case 0:
	case 10:
	    dprintf((stderr, "̤(Chara[%d])\n", cmd));
	    break;
	}
	RedisplayScene(alCore.work, SCENE);
    }

    /*  */
    switch (cmd) {
    case 0:
	break;
    case 1:
	if (!FadePattern(&state, True))
	    return False;
	break;
    case 2:
	if (!EffectAlphaBlend(&state))
	    return False;
	break;
    case 3:
	if (!WipeParallelVLtoR(&state))
	    return False;
	break;
    case 4:
	if (!WipeParallelVRtoL(&state))
	    return False;
	break;
    case 5:
	if (!WipeParallelHTtoB(&state))
	    return False;
	break;
    case 7:
	if (!WipeLtoR(&state, True))
	    return False;
	break;
    case 8:
	if (!WipeRtoL(&state, True))
	    return False;
	break;
    case 10:
	break;
    }

  chara_end:
    ALIB.imageDisplay(alCore.brightness);
    redraw_text();
    REQTBL.flushWindow();
  chara_break:
    state = 0;
    return True;
}

int
cmd_Effect(char arg[][ARG_LEN], int num, int interrupt)
{
    static int state = 0;
    static int v1;

    if (WAIT.skip || !CONFIG.use_effect)
	goto effect_end;

    switch (strtol(arg[1], NULL, 0)) {
    case 0:
	if (state == 0) {
	    dprintf((stderr, "Ĥ\n"));
	    ALIB.imageAdd(alCore.sub, alCore.vram, 0, 0);
	}
	if (!RandomShake(&state, 1))
	    return False;
	break;
    case 1:
	if (state == 0) {
	    v1 = strtol(arg[2], NULL, 0);
	    dprintf((stderr, "椵椵[ %d]\n", v1));
	    ALIB.imageAdd(alCore.sub, alCore.vram, 0, 0);
	}
	if (!VerticalShake(&state, v1))
	    return False;
	break;
    case 0xffffff:
	if (state == 0) {
	    dprintf((stderr, "ɤ\n"));
	    ALIB.imageAdd(alCore.sub, alCore.vram, 0, 0);
	}
	if (!RandomShake(&state, 3))
	    return False;
	break;
    default:
	fprintf(stderr, "ʸ: ");
	cmd_test(arg, num);
	break;
    }

  effect_end:
    state = 0;
    return True;
}

/* 
 * åեȥڥʤΤǡȤä˼
 */
int
cmd_End(char arg[][ARG_LEN], int num, int interrupt)
{
    return True;
}

int
cmd_FadeOutMusic(char arg[][ARG_LEN], int num, int interrupt)
{
    cmd_test(arg, num);
    if (WAIT.skip) {
	ALIB.musicStop();
    } else {
	ALIB.musicFadeOut();
    }
    return True;
}

/* 
 * ʥꥪʬѤΥե饰ΩƤ
 */
int
cmd_Flag(char arg[][ARG_LEN], int num, int interrupt)
{
    int i;
    cmd_test(arg, num);

    for (i = 1; i < num; i++) {
	add_flag(arg[i], &alCore.flag);
    }
    return True;
}

int
cmd_IfJump(char arg[][ARG_LEN], int num, int interrupt)
{
    cmd_test(arg, num);

    if (num < 2) {
	fprintf(stderr, "郎ޤ\n");
	exit(1);
    }

    if (eval(arg + 2, num - 2)) {
	dprintf((stderr, "[<%s> إ]\n", arg[1]));
	cmd_Jump(arg, 2, False);
    }
    return True;
}

int
cmd_Jump(char arg[][ARG_LEN], int num, int interrupt)
{
    SceneList *p = SCRIPT.scene_list;

    while (p != NULL) {
	if (!strcmp(p->name, arg[1])) {
	    SCRIPT.cur_pos = p->offset;
	    dprintf((stderr, "٥[%s]: եå %d إ\n",
		     arg[1], p->offset));
	    strcpy(SCENE.name, arg[1]);
	    return True;
	}
	p = p->next;
    }
    fprintf(stderr, "٥[%s]ޤ\n", arg[1]);
    exit(1);
}

int
cmd_Light(char arg[][ARG_LEN], int num, int interrupt)
{
    cmd_test(arg, num);
    return True;
}

int
cmd_MessageType(char arg[][ARG_LEN], int num, int interrupt)
{
    int type;
    cmd_test(arg, num);

    sscanf(arg[1], "%d", &type);
    if (type == 0) {
	dprintf((stderr, "[]\n"));
	TEXT.fullscreen = True;
	TEXT.clear = True;
    } else if (type == 1) {
	dprintf((stderr, "[̲]\n"));
	TEXT.fullscreen = False;
	TEXT.clear = True;
    } else {
	fprintf(stderr, "ʥåɽǤ\n");
	exit(1);
    }
    return True;
}

int
cmd_Music(char arg[][ARG_LEN], int num, int interrupt)
{
    cmd_test(arg, num);
    if (ALIB.musicStatus())
	ALIB.musicStop();

    ALIB.musicLoad(arg[1]);
    ALIB.musicPlay();

    return True;
}

int
cmd_OutAllChara(char arg[][ARG_LEN], int num, int interrupt)
{
    static int state = 0, cmd;
    static int lstate, dstate, abstate;

    switch (state) {
    case 0:
	lstate = dstate = abstate = 0;
	cmd_test(arg, num);
	cmd = strtol(arg[1], NULL, 0);

	WorkSceneClearAllChara();

	if (cmd < 0)
	    goto outallchara_break;

	ReflectWorkScene();

	if (WAIT.skip || !CONFIG.use_effect) {
	    RedisplayScene(alCore.work, SCENE);
	    goto outallchara_end;
	}
	state++;
    case 1:
	if (!Lighten(&lstate))
	    return False;
	switch (cmd) {
	case 2:
	    ALIB.imageAdd(alCore.sub, alCore.work, 0, 0);
	    RedisplayScene(alCore.work, SCENE);
	    break;
	case 3:
	case 4:
	    break;
	case 7:
	case 8:
	    ALIB.imageAdd(alCore.vram, alCore.work, 0, 0);
	    ALIB.imageCopy(alCore.sub, alCore.vram, 0, 0, 0, 0, WIDTH,
			   HEIGHT);
	    break;
	case 0:
	case 9:
	    dprintf((stderr, "OutAllChara: ̤<%d>\n", cmd));
	    break;
	}
	state++;
    case 2:
	RedisplayScene(alCore.work, SCENE);
	switch (cmd) {
	case 2:
	    if (!EffectAlphaBlend(&abstate))
		return False;
	    break;
	case 3:
	    if (!WipeParallelVLtoR(&abstate))
		return False;
	    break;
	case 4:
	    if (!WipeParallelVRtoL(&abstate))
		return False;
	    break;
	case 7:
	    if (!WipeLtoR(&abstate, True))
		return False;
	    break;
	case 8:
	    if (!WipeRtoL(&abstate, True))
		return False;
	    break;
	}
	state++;
    case 3:
	if (!Darken(&dstate))
	    return False;
	state++;
    }

  outallchara_end:
    ALIB.imageDisplay(alCore.brightness);
    redraw_text();
    REQTBL.flushWindow();
  outallchara_break:
    state = 0;
    return True;
}

int
cmd_OutBg(char arg[][ARG_LEN], int num, int interrupt)
{
    static Color color;
    static int state = 0, cmd;
    static int lstate, dstate, abstate;

    switch (state) {
    case 0:
	lstate = dstate = abstate = 0;
	cmd_test(arg, num);
	color = string2color(arg[1]);

	WorkSceneOutBg(arg[1]);
	cmd = strtol(arg[2], NULL, 0);

	if (cmd < 0) {
	    ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, color);
	    goto outbg_break;
	}

	ReflectWorkScene();

	if (WAIT.skip || !CONFIG.use_effect) {
	    ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, color);
	    ALIB.imageDisplay(alCore.brightness);
	    goto outbg_end;
	}
	state++;
    case 1:
	if (!Lighten(&lstate))
	    return False;
	state++;
    case 2:
	if (abstate == 0) {
	    switch (cmd) {
	    case 1:
	    case 7:
		ALIB.imageCopy(alCore.sub, alCore.vram, 0, 0, 0, 0, WIDTH,
			       HEIGHT);
		break;
	    case 2:
		ALIB.imageAdd(alCore.sub, alCore.work, 0, 0);
		break;
	    }
	    ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, color);
	}
	switch (cmd) {
	case 1:
	    if (!FadePattern(&abstate, True))
		return False;
	    break;
	case 2:
	    if (!EffectAlphaBlend(&abstate))
		return False;
	    break;
	case 3:
	    if (!WipeParallelVLtoR(&abstate))
		return False;
	    break;
	case 6:
	    if (!WipeParallelHBtoT(&abstate))
		return False;
	    break;
	case 7:
	    if (!WipeLtoR(&abstate, True))
		return False;
	    break;
	default:
	    dprintf((stderr, "OutBg: ̤<%d>\n", cmd));
	}
	state++;
    case 3:
	if (!Darken(&dstate))
	    return False;
	state++;
    default:
	break;
    }

  outbg_end:
    if (!TEXT.clear) {
	redraw_text();
	REQTBL.flushWindow();
    }
  outbg_break:
    state = 0;
    return True;
}

int
cmd_OutChara(char arg[][ARG_LEN], int num, int interrupt)
{
    static int state = 0, cmd;

    if (state == 0) {
	cmd_test(arg, num);
	cmd = strtol(arg[2], NULL, 0);

	WorkSceneClearChara(arg[1]);

	if (cmd < 0)
	    goto outchara_break;

	ReflectWorkScene();

	if (WAIT.skip || !CONFIG.use_effect) {
	    RedisplayScene(alCore.work, SCENE);
	    goto outchara_end;
	}

	switch (cmd) {
	case 2:
	    ALIB.imageAdd(alCore.vram, alCore.work, 0, 0);
	    REQTBL.flushWindow();
	    ALIB.imageAdd(alCore.sub, alCore.work, 0, 0);
	    break;
	case 8:
	    ALIB.imageAdd(alCore.vram, alCore.work, 0, 0);
	    REQTBL.flushWindow();
	    break;
	case 0:
	    break;
	}
	RedisplayScene(alCore.work, SCENE);
    }

    switch (cmd) {
    case 2:
	if (!EffectAlphaBlend(&state))
	    return False;
	break;
    case 8:
	if (!WipeParallelVLtoR(&state))
	    return False;
	break;
    case 0:
	break;
    }

  outchara_end:
    ALIB.imageDisplay(alCore.brightness);
    redraw_text();
    REQTBL.flushWindow();
  outchara_break:
    state = 0;
    return True;
}

int
cmd_PREVLook(char arg[][ARG_LEN], int num, int interrupt)
{
    cmd_test(arg, num);

    add_flag(arg[1], &alCore.prev_flag);
    return True;
}

/* 
 * ڡ
 */
int
cmd_Page(char arg[][ARG_LEN], int num, int interrupt)
{
    dprintf((stderr, "[ڡ]\n"));

    if (TEXT.clear)
	return True;

    newpage();
    if (!WAIT.skip) {
	pop_stack();
	push_stack(STAT_WAIT);
    }

    return False;
}

int
cmd_Return(char arg[][ARG_LEN], int num, int interrupt)
{
    cmd_test(arg, num);
    return True;
}

int
cmd_SceneName(char arg[][ARG_LEN], int num, int interrupt)
{
    dprintf((stderr, "[: %s]\n", arg[1]));
    strncpy(SCENE.name, arg[1], ARG_LEN);
    alCore.alternative_num = 0;

    script_seen_proc();

    return True;
}

int
cmd_SelStart(char arg[][ARG_LEN], int num, int interrupt)
{
    int i, j;
    cmd_test(arg, num);

    if (!TEXT.fullscreen)
	newpage();

    menu_clear();
    clear_cursor();
    for (i = 0; i < alCore.alternative_num; i++) {
	for (j = 0; alCore.alternative[i].string[j] != 0; j++)
	    put_char(alCore.alternative[i].string[j],
		     (i + 1) | ATTR_ALTN_MASK, 0);
	newline();
    }

    pop_stack();
    push_stack(STAT_ALTN);
    push_stack(STAT_EMPTY);

    WAIT.skip = False;

    return True;
}

int
cmd_Select(u_char arg[][ARG_LEN], int num, int interrupt)
{
    int i, j;

    if (alCore.alternative_num >= ALT_NUM) {
	fprintf(stderr, "οĿ(%d)ĶƤޤ\n",
		ALT_NUM);
	exit(1);
    }

    strcpy(alCore.alternative[alCore.alternative_num].label, arg[1]);
    for (i = 0, j = 0; j < strlen(arg[2]);) {
	if (arg[2][j] < 0x80) {
	    alCore.alternative[alCore.alternative_num].string[i++] =
		arg[2][j];
	    j++;
	} else {
	    alCore.alternative[alCore.alternative_num].string[i++] =
		arg[2][j] << 8 | arg[2][j + 1];
	    j += 2;
	}
    }
    alCore.alternative[alCore.alternative_num].string[i] = 0;
    alCore.alternative_num++;

    return True;
}

int
cmd_Sound(char arg[][ARG_LEN], int num, int interrupt)
{
    cmd_test(arg, num);

    if (!WAIT.skip) {
	ALIB.soundLoad(arg[1]);
	ALIB.soundPlay();
    }

    return True;
}

int
cmd_Staffroll(char arg[][ARG_LEN], int num, int interrupt)
{
    int id;
    cmd_test(arg, num);

    save_persistent_info();
    id = strtol(arg[1], NULL, 0);
    ending_set_id(id);
    LongJump(JUMP_ENDING);

    return True;
}

int
cmd_StopMusic(char arg[][ARG_LEN], int num, int interrupt)
{
    cmd_test(arg, num);
    ALIB.musicStop();
    return True;
}

int
cmd_Time(char arg[][ARG_LEN], int num, int intrerupt)
{
    int t;
    cmd_test(arg, num);
    t = strtol(arg[1], NULL, 0);
    init_wait_timer(t);
    pop_stack();
    push_stack(STAT_WAIT_TIME);
    return False;
}

/* 
 * ȼɲåޥ
 */
int
cmd_EnterCGMode(char arg[][ARG_LEN], int num, int interrupt)
{
    static int state = 0;

    if (state < 10) {
	Lighten(&state);
	return False;
    }
    state = 0;
    return True;
}

int
cmd_LeaveCGMode(char arg[][ARG_LEN], int num, int interrupt)
{
    static int state = 0;

    if (state < 10) {
	Darken(&state);
	return False;
    }
    trap_area_clear();
    redraw_text();
    REQTBL.flushWindow();

    state = 0;
    return True;
}

int
cmd_EnterMenuMode(char arg[][ARG_LEN], int num, int interrupt)
{
    push_text();
    menu_set(arg[1]);

    return True;
}

int
cmd_LeaveMenuMode(char arg[][ARG_LEN], int num, int interrupt)
{
    menu_clear();
    RedisplayScene(alCore.work, SCENE);
    ALIB.imageDisplay(alCore.brightness);

    pop_text();

    redraw_text();
    REQTBL.flushWindow();

    return True;
}

int
cmd_EnterDictMode(char arg[][ARG_LEN], int num, int interrupt)
{
    BackupScene();
    ClearScene();
    strcpy(SCENE.bg, "yougo_bg.bmp");
    alCore.brightness = 20;

    TEXT.yoffset = DICT_YOFFSET;
    push_dict();

    return True;
}

int
cmd_LeaveDictMode(char arg[][ARG_LEN], int num, int interrupt)
{
    RestoreScene();
    RedisplayScene(alCore.work, SCENE);
    ALIB.imageDisplay(alCore.brightness);

    trap_area_clear();
    TEXT.yoffset = YOFFSET;
    pop_dict();

    return True;
}

typedef struct {
    char name[ARG_LEN];
    int (*func) (char[][ARG_LEN], int, int);
} Command;

Command command_list[] = {
    /* ɸॳޥ */
    {"Bg", cmd_Bg},
    {"CGLook", cmd_CGLook},
    {"Chara", cmd_Chara},
    {"Effect", cmd_Effect},
    {"End", cmd_End},
    {"FadeOutMusic", cmd_FadeOutMusic},
    {"Flag", cmd_Flag},
    {"IfJump", cmd_IfJump},
    {"Jump", cmd_Jump},
    {"Light", cmd_Light},
    {"MessageType", cmd_MessageType},
    {"Music", cmd_Music},
    {"OutAllChara", cmd_OutAllChara},
    {"OutBg", cmd_OutBg},
    {"OutChara", cmd_OutChara},
    {"PREVLook", cmd_PREVLook},
    {"Page", cmd_Page},
    {"Return", cmd_Return},
    {"SceneName", cmd_SceneName},
    {"SelStart", cmd_SelStart},
    {"Select", cmd_Select},
    {"Sound", cmd_Sound},
    {"Staffroll", cmd_Staffroll},
    {"StopMusic", cmd_StopMusic},
    {"Time", cmd_Time},
    /* ȼޥ */
    {"EnterCGMode", cmd_EnterCGMode},
    {"LeaveCGMode", cmd_LeaveCGMode},
    {"EnterMenuMode", cmd_EnterMenuMode},
    {"LeaveMenuMode", cmd_LeaveMenuMode},
    {"EnterDictMode", cmd_EnterDictMode},
    {"LeaveDictMode", cmd_LeaveDictMode},
};

int
proc_command(int interrupt)
{
    int i, num;
    char argv[ARG_NUM][ARG_LEN];

    num = script_parse_arg(command_buf + 1, argv);

    for (i = 0; i < sizeof(command_list) / sizeof(command_list[0]); i++) {
	if (!strcmp(command_list[i].name, argv[0])) {
	    return command_list[i].func(argv, num, interrupt);
	}
    }
    fprintf(stderr, "̤ΤΥޥ[%s]Ǥ\n", argv[0]);
    exit(1);
}
