/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: config_mode.c,v 1.8 2002/03/03 15:37:00 tf Exp $
 */

#include "aladder.h"

static Image *back_p, *back_r;
static Image *window_s[2], *window_h[2];
static Image *text_s[3], *text_h[3];
static Image *effect_s[2], *effect_h[2];

static void
init_config_mode_widget()
{
    int i;

    ALIB.imageLoad(alCore.sub, "set_c.bmp");

    back_p = ALIB.imageNew(200, 44);
    back_r = ALIB.imageNew(200, 44);
    ALIB.imageCopy(back_p, alCore.sub, 0, 0, 0, 192, 200, 44);

    for (i = 0; i < 2; i++) {
	window_s[i] = ALIB.imageNew(160, 48);
	window_h[i] = ALIB.imageNew(160, 48);
	ALIB.imageCopy(window_s[i], alCore.sub, 0, 0, i * 160, 0, 160, 48);
	ALIB.imageCopy(window_h[i], alCore.sub, 0, 0, i * 160, 48, 160,
		       48);
    }

    for (i = 0; i < 3; i++) {
	text_s[i] = ALIB.imageNew(100, 48);
	text_h[i] = ALIB.imageNew(100, 48);
	ALIB.imageCopy(text_s[i], alCore.sub, 0, 0, i * 100, 96, 100, 48);
	ALIB.imageCopy(text_h[i], alCore.sub, 0, 0, i * 100, 144, 100, 48);
    }

    for (i = 0; i < 2; i++) {
	effect_s[i] = ALIB.imageNew(160, 48);
	effect_h[i] = ALIB.imageNew(160, 48);
	ALIB.imageCopy(effect_s[i], alCore.sub, 0, 0, i * 160 + 320, 0,
		       160, 48);
	ALIB.imageCopy(effect_h[i], alCore.sub, 0, 0, i * 160 + 320, 48,
		       160, 48);
    }
}

void
enter_config_mode()
{
    int first = True;

    push_stack(STAT_TTL_CONFIG);
    ALIB.imageLoad(alCore.vram, "set_bg.bmp");

    if (first) {
	first = False;
	init_config_mode_widget();
	ALIB.imageCopy(back_r, alCore.vram, 0, 0, 226, 412, 200, 44);
    }

    clear_tvram();
    trap_area_clear();

    /* ץܥ */
    trap_area_add(226, 412, 200, 44);

    /* ̥ */
    trap_area_add(256, 60, 160, 48);
    trap_area_add(425, 60, 160, 48);

    /* ƥɽ® */
    trap_area_add(256, 114, 100, 48);
    trap_area_add(370, 114, 100, 48);
    trap_area_add(484, 114, 100, 48);

    /* ե */
    trap_area_add(256, 168, 160, 48);
    trap_area_add(424, 168, 160, 48);

    redraw_config_mode();
}

void
leave_config_mode()
{
    trap_area_clear();

    pop_stack();
    RedisplayScene(alCore.work, SCENE);
    ALIB.imageDisplay(alCore.brightness);
    pop_text();
    redraw_text();
    REQTBL.flushWindow();
}

int
proc_config_mode()
{
    switch (trap_area_selected()) {
    case 0:
	save_persistent_info();
	return True;
    case 1:
	REQTBL.setFullScreenMode(True);
	break;
    case 2:
	REQTBL.setFullScreenMode(False);
	break;
    case 3:
	CONFIG.text_speed = CONFIG_TEXT_SLOW;
	break;
    case 4:
	CONFIG.text_speed = CONFIG_TEXT_NORMAL;
	break;
    case 5:
	CONFIG.text_speed = CONFIG_TEXT_FAST;
	break;
    case 6:
	CONFIG.use_effect = True;
	break;
    case 7:
	CONFIG.use_effect = False;
	break;
    }
    redraw_config_mode();

    return False;
}

void
redraw_config_mode()
{
    ALIB.imageLoad(alCore.vram, "set_bg.bmp");

    switch (CONFIG.window_size) {
    case CONFIG_FULLSCREEN:
	ALIB.imageCopy(alCore.vram, window_s[0], 256, 60, 0, 0, 160, 48);
	break;
    case CONFIG_WINDOW:
	ALIB.imageCopy(alCore.vram, window_s[1], 424, 60, 0, 0, 160, 48);
	break;
    }

    switch (CONFIG.text_speed) {
    case CONFIG_TEXT_SLOW:
	ALIB.imageCopy(alCore.vram, text_s[0], 256, 114, 0, 0, 100, 48);
	break;
    case CONFIG_TEXT_NORMAL:
	ALIB.imageCopy(alCore.vram, text_s[1], 370, 114, 0, 0, 100, 48);
	break;
    case CONFIG_TEXT_FAST:
	ALIB.imageCopy(alCore.vram, text_s[2], 484, 114, 0, 0, 100, 48);
	break;
    }

    if (CONFIG.use_effect) {
	ALIB.imageCopy(alCore.vram, effect_s[1], 256, 168, 0, 0, 160, 48);
    } else {
	ALIB.imageCopy(alCore.vram, effect_s[0], 424, 168, 0, 0, 160, 48);
    }

    switch (trap_area_selected()) {
    case 0:			/* ץܥ */
	ALIB.imageCopy(alCore.vram, back_p, 226, 412, 0, 0, 200, 44);
	break;
    case 1:			/* ̥ */
	ALIB.imageCopy(alCore.vram, window_h[0], 256, 60, 0, 0, 160, 48);
	break;
    case 2:
	ALIB.imageCopy(alCore.vram, window_h[1], 424, 60, 0, 0, 160, 48);
	break;
    case 3:			/* ƥɽ® */
	ALIB.imageCopy(alCore.vram, text_h[0], 256, 114, 0, 0, 100, 48);
	break;
    case 4:
	ALIB.imageCopy(alCore.vram, text_h[1], 370, 114, 0, 0, 100, 48);
	break;
    case 5:
	ALIB.imageCopy(alCore.vram, text_h[2], 484, 114, 0, 0, 100, 48);
	break;
    case 6:			/* ̸̻ */
	ALIB.imageCopy(alCore.vram, effect_h[1], 256, 168, 0, 0, 160, 48);
	break;
    case 7:
	ALIB.imageCopy(alCore.vram, effect_h[0], 424, 168, 0, 0, 160, 48);
	break;
    }
    REQTBL.flushWindow();
}
