/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: cursor.c,v 1.4 2002/03/03 15:37:00 tf Exp $
 */

#include "aladder.h"

static Image *book[4], *book_m[4];
static Image *coin[4], *coin_m[4];
static Image *backup;
static int prev_x, prev_y;

void
cursor_init()
{
    static int initialized = False;
    int i;

    CURSOR.x = 0;
    CURSOR.y = 0;
    CURSOR.move = False;

    if (initialized)
	return;

    initialized = True;

    ALIB.imageLoad(alCore.sub, "set_c.bmp");
    backup = ALIB.imageNew(32, 32);

    for (i = 0; i < 4; i++) {
	book[i] = ALIB.imageNew(32, 32);
	ALIB.imageCopy(book[i], alCore.sub, 0, 0, 32 * i + 24, 448, 32,
		       32);
	book_m[i] = ALIB.imageCreateMask(book[i]);

	coin[i] = ALIB.imageNew(24, 24);
	ALIB.imageCopy(coin[i], alCore.sub, 0, 0, 24 * i + 152, 456, 24,
		       24);
	coin_m[i] = ALIB.imageCreateMask(coin[i]);
    }
}

void
draw_cursor()
{
    static int flip = 0, skip = 0;
    int x, y;

    if (++skip % 4 != 0)
	return;

    flip = ++flip % 4;

    ALIB.coordTextToGraphics(CURSOR.x, CURSOR.y, &x, &y);

    prev_x = x;
    prev_y = y;

    ALIB.imageCopy(backup, alCore.vram, 0, 0, x, y, 32, 32);
    if (TEXT.clear) {
	ALIB.imageAddWithMask(alCore.vram, book[flip], book_m[flip], x, y);
    } else {
	ALIB.imageAddWithMask(alCore.vram, coin[flip], coin_m[flip], x, y);
    }
    REQTBL.flushWindowArea(x, y, 32, 32);
    ALIB.imageCopy(alCore.vram, backup, x, y, 0, 0, 32, 32);
}

void
clear_cursor()
{
    if (CURSOR.move) {
	REQTBL.flushWindowArea(prev_x, prev_y, 32, 32);
	CURSOR.move = False;
    }
}
