/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001, 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: dict_lookup.c,v 1.10 2002/03/09 17:41:59 tf Exp $
 */

/* 
 * 񻲾Ȳ̤ν
 */
#include "aladder.h"

#define WORD_STACK_DEPTH 5
#define WORD_LEN 32

static Image *button_p, *button_r;
static u_int w_stack[WORD_STACK_DEPTH][WORD_LEN];
static int stack_id = -1;

static void
push_word(u_int *str)
{
    u_int *p;
    int w, x;

    stack_id++;
    if (stack_id >= WORD_STACK_DEPTH) {
	dprintf((stderr, "push_word: stack overflow\n"));
	stack_id = WORD_STACK_DEPTH;
	return;
    }
    p = w_stack[stack_id];

    while (*str) {
	*p++ = *str++;
    }
    *p = 0;

    w = ALIB.imageStringWidth(w_stack[stack_id]);
    x = (WIDTH - w) / 2;
    ALIB.drawImageString(w_stack[stack_id], x, DICT_SUBJECT_Y, PINK, 0xff);
}

static void
pop_word()
{
    int w, x;
    if (--stack_id < 0) {
	stack_id = -1;
	return;
    }

    w = ALIB.imageStringWidth(w_stack[stack_id]);
    x = (WIDTH - w) / 2;
    ALIB.drawImageString(w_stack[stack_id], x, DICT_SUBJECT_Y, PINK, 0xff);
}

void
dict_mode_leave()
{
    pop_stack();
    if (CURRENT_STATE == STAT_DICT) {
	pop_dict();
    } else {
	set_command("#LeaveDictMode");
	push_stack(STAT_COMMAND);
    }
}

static void
init_dict_widget()
{
    button_p = ALIB.imageNew(244, 44);
    button_r = ALIB.imageNew(244, 44);
    ALIB.imageLoad(alCore.sub, "set_c.bmp");
    ALIB.imageCopy(button_p, alCore.sub, 0, 0, 200, 236, 200, 44);
    ALIB.imageCopy(button_r, alCore.sub, 0, 0, 200, 192, 200, 44);
}

void
dict_button_push()
{
    ALIB.imageCopy(alCore.vram, button_p, 220, 436, 0, 0, 200, 44);
    REQTBL.flushWindowArea(220, 436, 200, 44);
}

void
dict_button_release()
{
    ALIB.imageCopy(alCore.vram, button_r, 220, 436, 0, 0, 200, 44);
    REQTBL.flushWindowArea(220, 436, 200, 44);
}

static void
set_dict_vram(u_int *word, u_int *desc, u_int *attr)
{
    int i, j, w, x = 2, y = 0;

    clear_tvram();

    for (i = 0; i < TEXT_HEIGHT; i++) {
	for (j = 0; j < 3; j++) {
	    w = ALIB.putChar(WCHAR_SPACE, j, i, ATTR_NORMAL);
	    TEXT.tvram[j][i] = WCHAR_SPACE;
	    TEXT.geom[j + 1][i] = TEXT.geom[j][i] + w;
	}
    }

    for (i = 0; i < text_len(desc); i++) {
	if (desc[i] == 0xd)
	    continue;

	if (desc[i] == 0xa) {
	    x = 2;
	    y++;
	    continue;
	}

	w = ALIB.putChar(desc[i], x, y, ATTR_NORMAL);
	TEXT.tvram[x][y] = desc[i];
	TEXT.attr[x][y] = attr[i];

	if (w > 0)
	    TEXT.geom[x + 1][y] = TEXT.geom[x][y] + w;

	x++;
	if (TEXT.geom[x][y] > WIDTH - DICT_ABS_LEFT_MARGIN) {
	    x = 2;
	    y++;
	} else if (TEXT.geom[x][y] > WIDTH - DICT_PRE_LEFT_MARGIN
		   && is_kinsoku_o(desc[i + 1])) {
	    x = 2;
	    y++;
	} else if (TEXT.geom[x][y] > WIDTH - DICT_LEFT_MARGIN
		   && !is_kinsoku_c(desc[i + 1])) {
	    x = 2;
	    y++;
	}
    }

    REQTBL.clearWindow();
    push_word(word);
    redraw_text();
    REQTBL.flushWindow();
}

void
set_dict(int id)
{
    u_int word[BUFSIZ], desc[BUFSIZ], attr[BUFSIZ];

    id &= ATTR_ALL_MASK ^ ATTR_DICT_MASK;

    dict_get_entry(id, word, desc);
    dict_lookup(desc, attr);

    set_dict_vram(word, desc, attr);
}

void
push_dict()
{
    static int first = True;
    int backup = TEXT.fullscreen;

    if (first) {
	init_dict_widget();
	first = False;
    }

    push_text();
    TEXT.fullscreen = True;

    set_dict(TEXT.highlight);
    trap_area_clear();
    trap_area_add(220, 436, 200, 44);

    TEXT.highlight = 0;
    TEXT.fullscreen = backup;
}

void
pop_dict()
{
    pop_text();

    REQTBL.clearWindow();
    pop_word();
    redraw_text();
    REQTBL.flushWindow();
}
