/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001, 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: effect.c,v 1.19 2002/03/03 15:37:00 tf Exp $
 */
#include <stdlib.h>

#include "aladder.h"

int
FadeIn()
{
    return True;
}

int
FadeOut()
{
    int state = 0;
    ALIB.imageCopy(alCore.sub, alCore.work, 0, 0, 0, 0, WIDTH, HEIGHT);
    ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, BLACK);
    clear_text(False);

    while (!FadePattern(&state, True));

    return True;
}

int
Lighten(int *state)
{
    static timer t;

    (*state)++;

    if (*state == 1) {
	init_timer(&t);
    } else if (*state != 10) {
	get_timer(&t);
	if (t.msec > (*state + 1) * 50) {
	    WAIT.drop_frame = True;
	    return False;
	}
    }

    if (*state == 10) {
	alCore.brightness = 20;
    } else {
	alCore.brightness = 10 + (*state);
    }
    ALIB.imageDisplay(alCore.brightness);
    REQTBL.flushWindow();

    if (*state == 10) {
	return True;
    } else {
	return False;
    }
}

int
Darken(int *state)
{
    static timer t;

    (*state)++;

    if (*state == 1) {
	init_timer(&t);
    } else if (*state != 10) {
	get_timer(&t);
	if (t.msec > (*state + 1) * 50) {
	    WAIT.drop_frame = True;
	    return False;
	}
    }

    if (*state == 10) {
	alCore.brightness = 10;
    } else {
	alCore.brightness = 20 - (*state);
    }

    ALIB.imageDisplay(alCore.brightness);
    REQTBL.flushWindow();

    if (*state == 10) {
	return True;
    } else {
	return False;
    }
}

/* 
 * 
 */
int
EffectAlphaBlend(int *state)
{
    int alpha;
    static timer t;

    (*state)++;

    if (*state == 1) {
	init_timer(&t);
    } else if (*state != 20) {
	get_timer(&t);
	if (t.msec > (*state + 1) * 50) {
	    WAIT.drop_frame = True;
	    return False;
	}
    }

    if (*state == 20) {
	alpha = 0xff;
    } else {
	alpha = 0xff * (*state) / 20;
    }
    ALIB.imageMerge(alCore.vram, alCore.work, alCore.sub, alpha);
    REQTBL.flushWindow();

    if (*state == 20) {
	return True;
    } else {
	return False;
    }
}

/* 
 * ļʥ磻()
 */
int
WipeParallelVLtoR(int *state)
{
    int i;

    for (i = 0; i < WIDTH; i += 16) {
	int x = i + *state;
	ALIB.imageCopy(alCore.vram, alCore.work, x, 0, x, 0, 1, HEIGHT);
	REQTBL.flushWindowArea(x, 0, 1, HEIGHT);
    }

    if (++*state < 16) {
	return False;
    } else {
	return True;
    }
}

/* 
 * ļʥ磻()
 */
int
WipeParallelVRtoL(int *state)
{
    int i;

    for (i = 0; i < WIDTH; i += 16) {
	int x = i + 15 - *state;
	ALIB.imageCopy(alCore.vram, alCore.work, x, 0, x, 0, 1, HEIGHT);
	REQTBL.flushWindowArea(x, 0, 1, HEIGHT);
    }

    if (++*state < 16) {
	return False;
    } else {
	return True;
    }
}

/* 
 * ʥ磻(墪)
 */
int
WipeParallelHTtoB(int *state)
{
    int i;

    for (i = 0; i < HEIGHT; i += 16) {
	int y = i + *state;
	ALIB.imageCopy(alCore.vram, alCore.work, 0, y, 0, y, WIDTH, 1);
	REQTBL.flushWindowArea(0, y, WIDTH, 1);
    }

    if (++*state < 16) {
	return False;
    } else {
	return True;
    }
}

/* 
 * ʥ磻()
 */
int
WipeParallelHBtoT(int *state)
{
    int i;

    for (i = 0; i < HEIGHT; i += 16) {
	int y = i + 15 - *state;
	ALIB.imageCopy(alCore.vram, alCore.work, 0, y, 0, y, WIDTH, 1);
	REQTBL.flushWindowArea(0, y, WIDTH, 1);
    }

    if (++*state < 16) {
	return False;
    } else {
	return True;
    }
}

/* 
 * ļʻֺ磻()
 */
int
WipeLtoR(int *state, int flush)
{
    int i;

    for (i = 0; i < 16; i++) {
	int x = (*state - 15 + i) * 16 + i;
	if (x > 0 && x < WIDTH) {
	    ALIB.imageCopy(alCore.sub, alCore.work, x, 0, x, 0, 1, HEIGHT);
	    ALIB.imageCopy(alCore.vram, alCore.work, x, 0, x, 0, 1,
			   HEIGHT);
	    if (flush)
		REQTBL.flushWindowArea(x, 0, 1, HEIGHT);
	}
    }

    if (++*state < 40 + 16) {
	return False;
    } else {
	return True;
    }
}

#ifdef FAST_EFFECT
int
WipeLtoR2(int *state, int flush)
{
    int i;

    for (i = 0; i < 8; i++) {
	int x = (*state - i + 1) * 32 - (i + 1) * 4;
	if (x >= 0 && x < WIDTH) {
	    ALIB.imageCopy(alCore.sub, alCore.work, x, 0, x, 0, 4, HEIGHT);
	    ALIB.imageCopy(alCore.vram, alCore.work, x, 0, x, 0, 4,
			   HEIGHT);
	    if (flush)
		REQTBL.flushWindowArea(x, 0, 4, HEIGHT);
	}
    }

    if (++*state < 20 + 7) {
	return False;
    } else {
	return True;
    }
}
#endif


/* 
 * ļʻֺ磻()
 */
int
WipeRtoL(int *state, int flush)
{
    int i;

    for (i = 0; i < 16; i++) {
	int x = (*state - 15 + i) * 16 + i;
	if (x > 0 && x < WIDTH) {
	    x = WIDTH - x;
	    ALIB.imageCopy(alCore.sub, alCore.work, x, 0, x, 0, 1, HEIGHT);
	    ALIB.imageCopy(alCore.vram, alCore.work, x, 0, x, 0, 1,
			   HEIGHT);
	    if (flush)
		REQTBL.flushWindowArea(x, 0, 1, HEIGHT);
	}
    }

    if (++*state < 40 + 16) {
	return False;
    } else {
	return True;
    }
}

/* 
 * ɤ
 */
int
VerticalShake(int *state, int amp)
{
    static int osc[] = { 8, 0, 4, 0, 2, 0, 1, 0, 0, 0 };

    if (*state > sizeof(osc)) {
	dprintf((stderr, "state ϰϥС.\n"));
	return True;
    }

    ALIB.imageFill(alCore.vram, 0, 0, WIDTH, HEIGHT, BLACK);
    ALIB.imageAdd(alCore.vram, alCore.sub, 0, osc[*state] * amp);
    REQTBL.flushWindow();

    if (++*state < 10) {
	return False;
    } else {
	return True;
    }
}

#define VIB_SIZE 32
int
RandomShake(int *state, int count)
{
    ALIB.imageFill(alCore.vram, 0, 0, WIDTH, HEIGHT, BLACK);
    if (++*state % 2) {		/* 餹 */
	ALIB.imageAdd(alCore.vram, alCore.sub,
		      (random() % (VIB_SIZE * 2 + 1)) - VIB_SIZE,
		      (random() % (VIB_SIZE * 2 + 1)) - VIB_SIZE);
    } else {			/* ɤ */
	ALIB.imageAdd(alCore.vram, alCore.sub, 0, 0);
    }
    REQTBL.flushWindow();

    if (*state < count * 2) {
	return False;
    } else {
	return True;
    }
}
#undef VIB_SIZE

/* 
 * ѥޥȤä
 * (work: ؤβ, sub: ؤβ)
 */
int
FadePattern(int *state, int flush)
{
    int i, j;

    for (i = 0; i < HEIGHT; i += 16) {
	for (j = 0; j < WIDTH; j += 16) {
	    ALIB.imageBlendPattern(alCore.sub, alCore.work, j, i, *state);
	}
    }
    ALIB.imageCopy(alCore.vram, alCore.sub, 0, 0, 0, 0, WIDTH, HEIGHT);
    if (flush)
	REQTBL.flushWindow();

    if (++*state == 16) {
	return True;
    } else {
	return False;
    }
}

#ifdef FAST_EFFECT
int
FadePattern8(int *state, int flush)
{
    int i, j;

    for (i = 0; i < HEIGHT; i += 16) {
	for (j = 0; j < WIDTH; j += 16) {
	    ImageBlendPattern8(alCore.sub, alCore.work, j, i, *state);
	}
    }
    ALIB.imageCopy(alCore.vram, alCore.sub, 0, 0, 0, 0, WIDTH, HEIGHT);
    if (flush)
	REQTBL.flushWindow();

    if (++*state == 8) {
	return True;
    } else {
	return False;
    }
}
#endif
