/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: ending.c,v 1.8 2002/03/05 17:17:14 tf Exp $
 */

#include "aladder.h"
#include "animation.h"

static int staffroll_id;

static Image *logo;
static Image *ch_label[5];
static Image *sale, *sale_s;
static Image *devel, *devel_s;
static Image *half_vram, *half_img, *half_black;

static char *kasumi[] = {
    "BG022_01.bmp", "KCG027.bmp", "BG009_01.bmp", "KCG026_01.bmp",
    "BG003_01.bmp", "KCG035.bmp", "BG017.bmp", "KCG060.bmp",
    "BG023_01.bmp", "KCG001_01.bmp", NULL
};

static char *misaki[] = {
    "BG012.bmp", "KCG051_01.bmp", "BG005.bmp", "KCG019.bmp",
    "BG015_01.bmp", "KCG023.bmp", "BG043_01.bmp", "MCG013.bmp",
    "BG035_01.bmp",
    "MCG003.bmp", NULL
};

static char *rindis[] = {
    "BG010_01.bmp", "RCG002.bmp", "RCG012_02.bmp", "BG001_01.bmp",
    "RCG014.bmp", "BG026.bmp", "RCG026_05.bmp", "BG002_01.bmp",
    "RCG019_01.bmp", NULL
};

static char *mia[] = {
    "BG019.bmp", "TCG001.bmp", "BG014_01.bmp", "TCG010_02.bmp",
    "BG013.bmp", "TCG011_01.bmp", "BG024.bmp", "TCG028_02.bmp",
    "TCG023.bmp", NULL
};

static char *sayako[] = {
    "BG020.bmp", "KCG020.bmp", "SCG002.bmp", "BG021.bmp", "SCG003_01.bmp",
    "SCG008.bmp", "SCG001.bmp", "BG011_01.bmp", "BG006.bmp", NULL
};

static char **ed_file[5] = {
    kasumi,
    misaki,
    rindis,
    mia,
    sayako
};

void
ending_set_id(int id)
{
    staffroll_id = id;
}

static int
ending_init(int arg)
{
    static int initialized = False;

    if (initialized)
	return True;

    clear_text(False);

    ALIB.imageLoad(alCore.sub, "end_c.bmp");

    logo = ALIB.imageNew(310, 40);
    ALIB.imageCopy(logo, alCore.sub, 0, 0, 0, 430, 310, 40);

    ch_label[0] = ALIB.imageNew(120, 40);
    ch_label[1] = ALIB.imageNew(120, 40);
    ch_label[2] = ALIB.imageNew(200, 40);
    ch_label[3] = ALIB.imageNew(120, 40);
    ch_label[4] = ALIB.imageNew(120, 40);
    ALIB.imageCopy(ch_label[0], alCore.sub, 0, 0, 400, 300, 120, 40);
    ALIB.imageCopy(ch_label[1], alCore.sub, 0, 0, 400, 260, 120, 40);
    ALIB.imageCopy(ch_label[2], alCore.sub, 0, 0, 440, 340, 200, 40);
    ALIB.imageCopy(ch_label[3], alCore.sub, 0, 0, 520, 260, 120, 40);
    ALIB.imageCopy(ch_label[4], alCore.sub, 0, 0, 520, 300, 120, 40);

    sale = ALIB.imageNew(320, 40);
    sale_s = ALIB.imageNew(76, 28);
    ALIB.imageCopy(sale, alCore.sub, 0, 0, 0, 390, 320, 40);
    ALIB.imageCopy(sale_s, alCore.sub, 0, 0, 239, 360, 76, 28);

    devel = ALIB.imageNew(320, 84);
    devel_s = ALIB.imageNew(128, 28);
    ALIB.imageCopy(devel, alCore.sub, 0, 0, 320, 390, 320, 84);
    ALIB.imageCopy(devel_s, alCore.sub, 0, 0, 314, 360, 128, 28);

    half_vram = ALIB.imageNew(WIDTH / 2, HEIGHT / 2);
    half_img = ALIB.imageNew(WIDTH / 2, HEIGHT / 2);
    half_black = ALIB.imageNew(WIDTH / 2, HEIGHT / 2);
    ALIB.imageFill(half_black, 0, 0, WIDTH / 2, HEIGHT / 2, BLACK);

    return True;
}

static int
ending_status(int arg)
{
    static int state = 0, es = 0;
    int ret;
    switch (state) {
    case 0:
	ALIB.imageFill(alCore.sub, 0, 0, WIDTH, HEIGHT, BLACK);
	ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, BLACK);
	if (staffroll_id < 6) {
	    int w, h, x, y;
	    w = ch_label[staffroll_id - 1]->width;
	    h = ch_label[staffroll_id - 1]->height;
	    x = (WIDTH - w) / 2;
	    y = 250;
	    ALIB.imageCopy(alCore.work, ch_label[staffroll_id - 1],
			   x, y, 0, 0, w, h);
	}
	ALIB.imageCopy(alCore.work, logo, 165, 200, 0, 0, 310, 40);

	ALIB.musicLoad("10tenhashi.mp3");
	ALIB.musicPlay();

	state++;
    case 1:
	ret = EffectAlphaBlend(&es);
	if (ret) {
	    es = 0;
	    state++;
	}
	return False;
    }
    state = 0;

    return True;
}

static int
ending_cg_fadein(char *filename)
{
    static int i = 0;

    ALIB.imageLoad(alCore.sub, filename);
    ALIB.imageHalfSize(half_img, alCore.sub);
    ALIB.imageMerge(half_vram, half_img, half_black, 255 * i / 16);

    ALIB.imageCopy(alCore.vram, half_vram, 32, 32, 0, 0, WIDTH / 2,
		   HEIGHT / 2);
    REQTBL.flushWindowArea(32, 32, WIDTH / 2, HEIGHT / 2);

    if (i++ < 16)
	return False;

    i = 0;
    return True;
}

static int
ending_cg_fadeout()
{
    static int i = 0;

    ALIB.imageMerge(half_vram, half_img, half_black, 255 * (16 - i) / 16);

    ALIB.imageCopy(alCore.vram, half_vram, 32, 32, 0, 0, WIDTH / 2,
		   HEIGHT / 2);
    REQTBL.flushWindowArea(32, 32, WIDTH / 2, HEIGHT / 2);

    if (i++ < 16)
	return False;

    i = 0;
    return True;
}

static char *telop[] = {
    "ؤƤ󤷤ΤϤ",
    "",
    "ץǥ塼",
    " ",
    "",
    "衦ã",
    "ŷ͵",
    "",
    "衦",
    "",
    "",
    "ʥꥪ",
    "",
    "",
    "ץ",
    "",
    "",
    "",
    "ؤƤ󤷤ΤϤ",
    "륷롦",
    "       ɡ",
    "           ǥ",
    "£ǣ͡cocone",
    "ʿ",
    "ӣšޤʤ",
    "Խͭ¼椿",
    "",
    "ΡAngel Ladder",
    "Ρ",
    "졧¼ǵ",
    "ʡcocone",
    "Rec,Mix&Mastering",
    "ƣꤸ",
    "",
    "òֺס",
    "ͣӣ",
    "塡",
    "",
    "ڥ륵󥯥",
    "ϥ顡դ",
    "Ϥޤ",
    "",
    "丵",
    "ʳ˥֥󥨥",
    "",
    "ƤϤ for X",
    "TF",
    "",
    "ƤϤ for SDL",
    "TAJIRI Yasuhiro",
    "",
    "ƤϤ for MGL2",
    "郎",
    "",
    "ϡȥե",
    "ϲ̡꡼",
    "ؤƤ󤷤ΤϤ",
    "ǯ",
    "",
    "StudioAir",
    /* "ӣ", */
    "",
};

static void
ending_telop(int msec)
{
    int i, txt[32];

#define TELOP_OFFSET 20
#define TELOP_PITCH 32
#define TELOP_SPEED 30
#define TELOP_DELAY 100

    ALIB.imageFill(alCore.sub, 32 + WIDTH / 2, 0, WIDTH / 2 - 32, HEIGHT,
		   BLACK);
    ALIB.imageFill(alCore.vram, 32 + WIDTH / 2, 0, WIDTH / 2 - 32, HEIGHT,
		   BLACK);

    for (i = 0; i < sizeof(telop) / sizeof(telop[0]); i++) {
	int x = WIDTH / 2 + 32 + TELOP_OFFSET;
	int y =
	    finfo.pitch * i - msec / TELOP_SPEED + HEIGHT + TELOP_DELAY;
	int alpha = 0xff;

	if (y < 0 || y > HEIGHT)
	    continue;

	if (y < TELOP_PITCH)
	    alpha = 0xff * y / TELOP_PITCH;

	if (y > HEIGHT - FONT_HEIGHT - TELOP_PITCH) {
	    alpha = 0xff * (HEIGHT - FONT_HEIGHT - y) / TELOP_PITCH;
	    alpha = (alpha < 0) ? 0 : alpha;
	}
	euc_to_text(telop[i], txt);
	ALIB.drawImageString(txt, x, y, WHITE, alpha);
    }
    REQTBL.flushWindowArea(32 + WIDTH / 2, 0, WIDTH / 2 - 32, HEIGHT);
}

static void
calc_duration(int ntime[])
{
    int i, interval;
    char **p = ed_file[staffroll_id - 1];
#define TOP_OFFSET 5
#define END_LENGTH 70

    for (i = 0; p[i] != NULL; i++);

    interval = END_LENGTH / i;

    for (i = 0; p[i] != NULL; i++) {
	ntime[i] = (TOP_OFFSET + i * interval) * 1000;
	SET_CACHE(alCore.sub, ed_file[staffroll_id - 1][i]);
    }
}

static int
ending_cg(int arg)
{
    static int state = 0, t = 0, cg = 0, first = True;
    static int ntime[10];
    static timer tm;

    if (staffroll_id == 6)
	return True;

    if (first) {
	first = False;
	init_timer(&tm);
	calc_duration(ntime);
    }

    get_timer(&tm);
    ending_telop(tm.msec);

    switch (state) {
    case 0:
	if (!ending_cg_fadein(ed_file[staffroll_id - 1][cg]))
	    return False;
	state++;
    case 1:
	if (tm.msec < ntime[cg])
	    return False;
	t = 0;
	state++;
    case 2:
	if (!ending_cg_fadeout())
	    return False;
	if (ed_file[staffroll_id - 1][++cg] != NULL) {
	    state = 0;
	    t = 0;
	    return False;
	}
    case 3:
    }

    first = True;
    t = cg = state = 0;
    return True;
}

static int
ending_sale(int arg)
{
    static int i = 0;

    if (staffroll_id == 6)
	return True;

    if (i == 0) {
	int x, y;
	ALIB.imageCopy(alCore.sub, alCore.vram, 0, 0, 0, 0, WIDTH, HEIGHT);
	ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, BLACK);
	x = (WIDTH - sale_s->width) / 2;
	y = 175;
	ALIB.imageCopy(alCore.work, sale_s, x, y, 0, 0, sale_s->width,
		       sale_s->height);
	x = (WIDTH - sale->width) / 2;
	y = 210;
	ALIB.imageCopy(alCore.work, sale, x, y, 0, 0, sale->width,
		       sale->height);
    }

    if (!EffectAlphaBlend(&i))
	return False;

    i = 0;
    return True;
}

static int
ending_devel(int arg)
{
    static int i = 0;

    if (staffroll_id == 6)
	return True;

    if (i == 0) {
	int x, y;
	ALIB.imageCopy(alCore.sub, alCore.vram, 0, 0, 0, 0, WIDTH, HEIGHT);
	ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, BLACK);
	x = (WIDTH - devel_s->width) / 2;
	y = 175;
	ALIB.imageCopy(alCore.work, devel_s, x, y, 0, 0, devel_s->width,
		       devel_s->height);
	x = (WIDTH - devel->width) / 2;
	y = 200;
	ALIB.imageCopy(alCore.work, devel, x, y, 0, 0, devel->width,
		       devel->height);
    }

    if (!EffectAlphaBlend(&i))
	return False;

    i = 0;
    return True;
}

static int
ending_end(int arg)
{
    static int state = 0, es = 0;
    int ret;
    switch (state) {
    case 0:
	ALIB.imageLoad(alCore.sub, "end_c.bmp");
	ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, BLACK);
	ALIB.imageCopy(alCore.work, alCore.sub, 0, 100, 0, 0, WIDTH, 259);
	ALIB.imageCopy(alCore.work, alCore.sub, 120, 168, 0, 260, 400,
		       100);
	if (staffroll_id != 6)
	    ALIB.imageCopy(alCore.work, alCore.sub, 200, 301, 0, 360, 238,
			   27);
	ALIB.imageCopy(alCore.sub, alCore.vram, 0, 0, 0, 0, WIDTH, HEIGHT);
	state++;
    case 1:
	ret = EffectAlphaBlend(&es);
	if (ret) {
	    es = 0;
	    state++;
	}
	return False;
    }
    state = 0;

    return True;
}

static int
ending_quit(int arg)
{
    ALIB.musicFadeOut();
    return True;
}

static int
ending_none(int arg)
{
    return True;
}

static AnimationScriptCommand endingScriptCommand[] = {
    {ending_init, 0},
    {ending_status, 0},
    {animation_sleep, 3000},
    {animation_fadeout, 2000},
    {ending_cg, 0},
    {ending_sale, 0},
    {animation_sleep, 2000},
    {ending_devel, 0},
    {animation_sleep, 2000},
    {ending_end, 0},
    {animation_wait, 0},
    {animation_fadeout, 2000},
    {ending_quit, 0},
    {NULL, 0},
};

static AnimationScript endingScript = {
    endingScriptCommand,
    ending_none
};

void
run_ending_engine()
{
    run_animation_engine(endingScript);
    LongJump(JUMP_TITLE);
}
