/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001, 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: file.c,v 1.23 2002/03/09 17:41:59 tf Exp $
 */

#include <sys/types.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <string.h>
#include <time.h>

#include "aladder.h"
#include "command.h"

void
setup_directory()
{
    char tmppath[FILENAME_MAX];
    struct stat sb;

    snprintf(tmppath, sizeof tmppath, "%s/%s", getenv("HOME"),
	     ".angelladder");
    if (stat(tmppath, &sb) < 0) {
	perror(tmppath);
	dprintf((stderr, "%s: making...\n", tmppath));
	if (mkdir(tmppath, 0700) < 0) {
	    perror(tmppath);
	    exit(1);
	}
    } else if (!(sb.st_mode & S_IFDIR)) {
	dprintf((stderr, "%s: not directory!\n", tmppath));
	exit(1);
    }

    DIR.save_dir = strdup(tmppath);

    if (!(DIR.app_dir = getenv("AL_APP_DIR")))
	DIR.app_dir = AL_APP_DIR;

    if (!(DIR.music_dir = getenv("AL_MUSIC_DIR")))
	DIR.music_dir = AL_MUSIC_DIR;
}

static void
write_config_value(FILE *fp)
{
    switch (CONFIG.window_size) {
    case CONFIG_WINDOW:
	fprintf(fp, "screen=window\n");
	break;
    case CONFIG_FULLSCREEN:
	fprintf(fp, "screen=fullscreen\n");
	break;
    }

    switch (CONFIG.text_speed) {
    case CONFIG_TEXT_FAST:
	fprintf(fp, "text=fast\n");
	break;
    case CONFIG_TEXT_NORMAL:
	fprintf(fp, "text=normal\n");
	break;
    case CONFIG_TEXT_SLOW:
	fprintf(fp, "text=slow\n");
	break;
    }

    if (CONFIG.use_effect) {
	fprintf(fp, "effect=yes\n");
    } else {
	fprintf(fp, "effect=no\n");
    }
}

void
save_persistent_info()
{
    FILE *fp;
    FlagList *p;
    char path[FILENAME_MAX];

    snprintf(path, sizeof path, "%s/%s", DIR.save_dir, "state");
    if ((fp = fopen(path, "w")) == NULL) {
	perror("save_persistent_info");
	return;
    }

    fprintf(fp, "# config\n");
    write_config_value(fp);

    fprintf(fp, "# ending\n");
    p = alCore.flag;
    while (p != NULL) {
	char *c;
	c = index(p->value, '_');
	if (c != NULL) {
	    if (!strncmp(c, "_end", 4)) {
		fprintf(fp, "%s\n", p->value);
	    }
	}
	p = p->next;
    }

    fprintf(fp, "# seen cg\n");
    p = alCore.cg_flag;
    while (p != NULL) {
	fprintf(fp, "%s\n", p->value);
	p = p->next;
    }

    fprintf(fp, "# seen H scene\n");
    p = alCore.prev_flag;
    while (p != NULL) {
	fprintf(fp, "%s\n", p->value);
	p = p->next;
    }

    fclose(fp);

    /* ɾ̥ե */
    snprintf(path, sizeof path, "%s/%s", DIR.save_dir, "read");
    if ((fp = fopen(path, "w")) == NULL) {
	perror("save_persistent_info");
	return;
    }

    p = alCore.read_flag;
    while (p != NULL) {
	fprintf(fp, "%s\n", p->value);
	p = p->next;
    }

    fclose(fp);
}

#define INPUT_DEFAULT 0
#define INPUT_FLAG 1
#define INPUT_CG 2
#define INPUT_PREV 3
#define INPUT_CONFIG 4

static void
set_config_value(const char *buf)
{
    if (!strcmp(buf, "screen=window\n")) {
	dprintf((stderr, ": ɥ⡼\n"));
	CONFIG.window_size = CONFIG_WINDOW;
    }
    if (!strcmp(buf, "screen=fullscreen\n")) {
	dprintf((stderr, ": ̥⡼\n"));
	CONFIG.window_size = CONFIG_FULLSCREEN;
    }

    if (!strcmp(buf, "text=fast\n")) {
	dprintf((stderr, "ƥ: ®\n"));
	CONFIG.text_speed = CONFIG_TEXT_FAST;
    }
    if (!strcmp(buf, "text=normal\n")) {
	dprintf((stderr, "ƥ: \n"));
	CONFIG.text_speed = CONFIG_TEXT_NORMAL;
    }
    if (!strcmp(buf, "text=slow\n")) {
	dprintf((stderr, "ƥ: ٤\n"));
	CONFIG.text_speed = CONFIG_TEXT_SLOW;
    }

    if (!strcmp(buf, "effect=yes\n")) {
	dprintf((stderr, "̸: Ȥ\n"));
	CONFIG.use_effect = True;
    }
    if (!strcmp(buf, "effect=no\n")) {
	dprintf((stderr, "̸: Ȥʤ\n"));
	CONFIG.use_effect = False;
    }
}

void
load_persistent_info()
{
    FILE *fp;
    char buf[BUFSIZ];
    char flag[ARG_LEN];
    int mode = INPUT_DEFAULT;
    char path[FILENAME_MAX];

    snprintf(path, sizeof path, "%s/%s", DIR.save_dir, "state");

    if ((fp = fopen(path, "r")) == NULL) {
	perror("load_persistent_info");
	return;
    }

    while (fgets(buf, BUFSIZ, fp) != NULL) {
	if (buf[0] == '#') {
	    if (!strncmp(buf, "# ending", 8)) {
		mode = INPUT_FLAG;
	    } else if (!strncmp(buf, "# config", 8)) {
		mode = INPUT_CONFIG;
	    } else if (!strncmp(buf, "# seen cg", 8)) {
		mode = INPUT_CG;
	    } else if (!strncmp(buf, "# seen H scene", 8)) {
		mode = INPUT_PREV;
	    } else {
		mode = INPUT_DEFAULT;
	    }
	} else {
	    switch (mode) {
	    case INPUT_FLAG:
		sscanf(buf, "%s", flag);
		add_flag(flag, &alCore.flag);
		break;
	    case INPUT_CG:
		sscanf(buf, "%s", flag);
		add_flag(flag, &alCore.cg_flag);
		break;
	    case INPUT_PREV:
		sscanf(buf, "%s", flag);
		add_flag(flag, &alCore.prev_flag);
		break;
	    case INPUT_CONFIG:
		set_config_value(buf);
		break;
	    default:
		fprintf(stderr, "ǥ󥰡ɾǤ.");
		exit(1);
	    }
	}
    }
    fclose(fp);

    /* ɾ̥ե */
    snprintf(path, sizeof path, "%s/%s", DIR.save_dir, "read");

    if ((fp = fopen(path, "r")) == NULL) {
	perror("load_persistent_info");
	return;
    }

    while (fgets(buf, BUFSIZ, fp) != NULL) {
	sscanf(buf, "%s", flag);
	add_flag(flag, &alCore.read_flag);
    }
    fclose(fp);
}

void
SaveGame(const char *name)
{
    FILE *fp;
    char path[FILENAME_MAX];

    save_persistent_info();

    snprintf(path, sizeof path, "%s/%s", DIR.save_dir, name);
    if ((fp = fopen(path, "w")) == NULL) {
	perror("SaveGame");
	return;
    }

    dprintf((stderr, "Save the game status.\n"));

    fprintf(fp, "scene=%s\n", SCENE.name);
    fprintf(fp, "page_top=%d\n", SCENE.pos);
    fprintf(fp, "cur_pos=%d\n", SCRIPT.cur_pos);

    fprintf(fp, "cg_bg=%s\n", SCENE.bg);
    fprintf(fp, "cg_left=%s\n", SCENE.left);
    fprintf(fp, "cg_middle=%s\n", SCENE.middle);
    fprintf(fp, "cg_right=%s\n", SCENE.right);

    fprintf(fp, "text_mode=%d\n", TEXT.fullscreen);
    fprintf(fp, "bgm=%s\n", SCENE.bgm);

    fprintf(fp, "# flags\n");

    {
	FlagList *p;
	p = alCore.flag;
	while (p != NULL) {
	    char *c;
	    c = index(p->value, '_');
	    if (c != NULL) {
		if (strncmp(c, "_end", 4)) {	/* ꥢϥХ */
		    fprintf(fp, "%s\n", p->value);
		}
	    }
	    p = p->next;
	}
    }
    fclose(fp);
}

static void
restore_screen()
{
    char arg[ARG_NUM][ARG_LEN];

    clear_text(False);

    RedisplayScene(alCore.work, SCENE);

    if (SCENE.bgm[0]) {
	strcpy(arg[0], "Music");
	strcpy(arg[1], SCENE.bgm);
	cmd_Music(arg, 1, False);
    }

    {
	int state = 0;

	ALIB.imageFill(alCore.sub, 0, 0, WIDTH, HEIGHT, BLACK);
	while (!WipeLtoR(&state, True));
    }

    ALIB.imageDisplay(alCore.brightness);
    redraw_text();
    REQTBL.flushWindow();

    clear_stack();
    clear_text_stack();
}

static void
set_var(char *buf)
{
    char name[ARG_LEN], value[ARG_LEN];
    char *p, *q;

    p = buf;
    q = name;
    while (*p != '=')
	*q++ = *p++;
    *q = '\0';

    p++;

    q = value;
    while (*p != '\n')
	*q++ = *p++;
    *q = '\0';

    if (!strcmp(name, "scene")) {
	sprintf(SCENE.name, value);
	dprintf((stderr, "scene: <%s>\n", SCENE.name));
    } else if (!strcmp(name, "page_top")) {
	SCENE.pos = atoi(value);
	dprintf((stderr, "page_top <%d>\n", SCENE.pos));
    } else if (!strcmp(name, "cur_pos")) {
	SCRIPT.cur_pos = SCENE.pos;
	dprintf((stderr, "cur_pos <%d>\n", SCRIPT.cur_pos));
    } else if (!strcmp(name, "cg_bg")) {
	strcpy(SCENE.bg, value);
	dprintf((stderr, "cg_bg=%s\n", SCENE.bg));
    } else if (!strcmp(name, "cg_left")) {
	strcpy(SCENE.left, value);
	dprintf((stderr, "cg_left=%s\n", SCENE.left));
    } else if (!strcmp(name, "cg_middle")) {
	strcpy(SCENE.middle, value);
	dprintf((stderr, "cg_middle=%s\n", SCENE.middle));
    } else if (!strcmp(name, "cg_right")) {
	strcpy(SCENE.right, value);
	dprintf((stderr, "cg_right=%s\n", SCENE.right));
    } else if (!strcmp(name, "text_mode")) {
	TEXT.fullscreen = atoi(value);
	dprintf((stderr, "text_mode <%d>\n", TEXT.fullscreen));
    } else if (!strcmp(name, "bgm")) {
	strcpy(SCENE.bgm, value);
	dprintf((stderr, "bgm=%s\n", SCENE.bgm));
    } else {
	dprintf((stderr, "ѿ %s (%s) Ǥ.", name, value));
    }
}

#define LOAD_VAR 0
#define LOAD_FLAG 1

void
LoadGame(const char *name)
{
    FILE *fp;
    char buf[BUFSIZ];
    char flag[ARG_LEN];
    char path[FILENAME_MAX];
    int mode = LOAD_VAR;

    snprintf(path, sizeof path, "%s/%s", DIR.save_dir, name);

    if ((fp = fopen(path, "r")) == NULL) {
	perror("LoadGame");
	return;
    }

    backlog_init();
    ClearScene();

    while (fgets(buf, BUFSIZ, fp) != NULL) {
	if (buf[0] == '#') {
	    if (!strncmp(buf, "# flags", 7)) {
		mode = LOAD_FLAG;
	    } else {
		mode = LOAD_VAR;
	    }
	} else {
	    switch (mode) {
	    case LOAD_VAR:
		set_var(buf);
		break;
	    case LOAD_FLAG:
		if (buf[0] != '\n' && buf[0] != '\0') {
		    sscanf(buf, "%s", flag);
		    dprintf((stderr, "flags: %s\n", flag));
		    add_flag(flag, &alCore.flag);
		}
		break;
	    default:
		fprintf(stderr, "־Ǥ.");
		exit(1);
	    }
	}
    }

    restore_screen();
    CopyScene();
}

static void
set_number(u_char *p, int num)
{
    u_char digit[] = "";

    if (num / 10) {
	*p++ = digit[0];
	*p++ = digit[1] + num / 10;
    } else {
	p += 2;
    }
    *p++ = digit[0];
    *p++ = digit[1] + num % 10;
}

void
get_save_file_info(int num, u_char *date, u_char *desc)
{
    u_char empty[] = "֥եϤޤ";
    char path[FILENAME_MAX];
    struct stat sb;

    snprintf(path, sizeof path, "%s/%03d", DIR.save_dir, num);
    if (stat(path, &sb) < 0) {
	strcpy(date, empty);
	strcpy(desc, "");
	return;
    }

    {
	u_char ds[] = "ǯʬ";
	struct tm *tm = localtime(&sb.st_mtime);

	strcpy(date, ds);
	set_number(date, tm->tm_year % 100);
	set_number(date + 6, tm->tm_mon + 1);
	set_number(date + 12, tm->tm_mday);
	set_number(date + 20, tm->tm_hour);
	set_number(date + 26, tm->tm_min);
	set_number(date + 32, tm->tm_sec);
    }

    {
	FILE *fp;
	u_char buf[BUFSIZ];

	if ((fp = fopen(path, "r")) == NULL) {
	    perror("get_save_file_info");
	    return;
	}

	desc[0] = '\0';
	while (fgets(buf, BUFSIZ, fp) != NULL) {
	    if (!strncmp(buf, "scene=", 6)) {
		strcpy(desc, buf + 6);
		break;
	    }
	}
	fclose(fp);
    }
}
