/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001, 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: image_font.c,v 1.13 2002/03/05 17:17:14 tf Exp $
 */

#include <stdlib.h>

#include "aladder.h"

/* 
 * եȥåϢ(LRU)
 */
#define FONT_CACHE_SIZE 255

typedef struct cache {
    int code;
    int width;
    u_char *bitmap;

    struct cache *prev;
    struct cache *next;
} font_cache_t;

static font_cache_t cache[FONT_CACHE_SIZE];
static font_cache_t cache_list;

static void
font_cache_insert(font_cache_t *pos, font_cache_t *x)
{
    pos->prev->next = x;
    x->prev = pos->prev;
    x->next = pos;
    pos->prev = x;
}

static font_cache_t *
font_cache_remove(font_cache_t *x)
{
    x->prev->next = x->next;
    x->next->prev = x->prev;

    return x;
}

void
font_cache_new()
{
    int i;

    cache_list.next = cache_list.prev = &cache_list;
    for (i = 0; i < FONT_CACHE_SIZE; i++) {
	cache[i].bitmap = NULL;
	font_cache_insert(&cache_list, &cache[i]);
    }
}

#if 0
static void
font_cache_delete()
{
    int i;

    for (i = 0; i < FONT_CACHE_SIZE; i++)
	if (cache[i].bitmap != NULL)
	    free(cache[i].bitmap);
}
#endif

ALFont
get_font(int code)
{
    font_cache_t *p;
    ALFont font;

    for (p = cache_list.next; p != &cache_list; p = p->next) {
	if (code == p->code) {
	    font_cache_remove(p);
	    font_cache_insert(&cache_list, p);
	    font.bitmap = (void *) p->bitmap;
	    font.width = p->width;
	    return font;
	}
    }

    p = font_cache_remove(cache_list.next);
    font_cache_insert(&cache_list, p);

    if (p->bitmap != NULL) {
	free(p->bitmap);
	p->bitmap = NULL;
    }

    font = REQTBL.getFontGryph(code);
    p->bitmap = (u_char *) font.bitmap;
    p->width = font.width;
    p->code = code;

    return font;
}

static void
draw_image_char(int x, int y, ALFont font, Color col)
{
    int i, j;
    u_char *vram = (u_char *) alCore.vram->bitmap;

    for (i = 0; i < FONT_HEIGHT; i++) {
	u_char *p = vram + (x + (y + i) * WIDTH) * 3;
	u_char *q = font.bitmap + i * font.width;
	for (j = 0; j < font.width; j++, p += 3) {
	    if (*(q + j)) {
		*p = col.b;
		*(p + 1) = col.g;
		*(p + 2) = col.r;
	    }
	}
    }
}

static void
draw_image_char_alpha(int x, int y, ALFont font, Color col, int alpha)
{
    int i, j, s;
    int h = FONT_HEIGHT;
    u_char *vram = (u_char *) alCore.vram->bitmap;
    u_char *sub = (u_char *) alCore.sub->bitmap;

    s = (y < 0) ? -y : 0;

    if (y + FONT_HEIGHT > HEIGHT)
	h -= y + FONT_HEIGHT - HEIGHT;

    for (i = s; i < h; i++) {
	u_char *p = vram + (x + (y + i) * WIDTH) * 3;
	u_char *q = sub + (x + (y + i) * WIDTH) * 3;
	u_char *r = font.bitmap + i * font.width;
	for (j = 0; j < font.width; j++, p += 3, q += 3) {
	    if (*(r + j)) {
		*p = alpha_map[col.b][*q][alpha];
		*(p + 1) = alpha_map[col.g][*(q + 1)][alpha];
		*(p + 2) = alpha_map[col.r][*(q + 2)][alpha];
	    }
	}
    }
}

int
PutChar(int code, int x, int y, int attr)
{
    ALFont font;

    if (code == 0xa || code == 0xd)
	return -1;

    font = get_font(code);

    x = TEXT.geom[x][y];
    y = y * finfo.pitch + TEXT.yoffset;

    switch (attr) {
    case ATTR_NORMAL:
	draw_image_char(x - 1, y, font, BLACK);
	draw_image_char(x, y - 1, font, BLACK);

	draw_image_char(x + 1, y, font, BLACK);
	draw_image_char(x, y + 1, font, BLACK);

	draw_image_char(x + 1, y + 1, font, BLACK);
	draw_image_char(x + 2, y + 2, font, BLACK);
	break;
    case ATTR_BACKLOG:
    case ATTR_LINK:
    case ATTR_ALTN:
    case ATTR_LINK_HL:
    case ATTR_ALTN_HL:
	draw_image_char(x - 1, y, font, BLACK);
	draw_image_char(x, y - 1, font, BLACK);

	draw_image_char(x + 1, y, font, BLACK);
	draw_image_char(x, y + 1, font, BLACK);
	break;
    }

    switch (attr) {
    case ATTR_NORMAL:
    case ATTR_ALTN:
	draw_image_char(x, y, font, WHITE);
	break;
    case ATTR_BACKLOG:
    case ATTR_LINK:
	draw_image_char(x, y, font, GRAY);
	break;
    case ATTR_LINK_HL:
    case ATTR_ALTN_HL:
	draw_image_char(x, y, font, YELLOW);
	break;
    }

    return font.width;
}

void
text_coordinate(int gx, int gy, int *tx, int *ty)
{
    int i;

    *ty = (gy - TEXT.yoffset) / finfo.pitch;

    if (gx < XOFFSET || gy < TEXT.yoffset) {
	*tx = -1;
	*ty = -1;
	return;
    }

    for (i = 0; i < TEXT_WIDTH - 1; i++) {
	if (gx >= TEXT.geom[i][*ty] && gx <= TEXT.geom[i + 1][*ty]) {
	    *tx = i;
	    return;
	}
    }

    *tx = -1;
    *ty = -1;
}

void
graphics_coordinate(int tx, int ty, int *gx, int *gy)
{
    *gx = TEXT.geom[tx][ty];
    *gy = ty * finfo.pitch + TEXT.yoffset;
}

static int
PutImageChar(int code, int x, int y, Color col, int alpha)
{
    ALFont font;

    if (code == 0xa || code == 0xd)
	return -1;

    font = get_font(code);

    draw_image_char_alpha(x - 1, y, font, col, alpha);
    return font.width;
}

void
DrawImageString(u_int *str, int x, int y, Color color, int alpha)
{
    u_int *p = str;
    int w;

    while (*p) {
	PutImageChar(*p, x - 1, y - 1, BLACK, alpha);
	PutImageChar(*p, x + 1, y - 1, BLACK, alpha);
	PutImageChar(*p, x - 1, y + 1, BLACK, alpha);
	PutImageChar(*p, x + 1, y + 1, BLACK, alpha);

	w = PutImageChar(*p++, x, y, color, alpha);
	x += w;
	if (w >= WIDTH)
	    break;
    }
}

int
ImageStringWidth(u_int *str)
{
    u_int *p = str;
    int w = 0;
    ALFont font;

    while (*p) {
	font = get_font(*p++);
	w += font.width;
    }
    return w;
}
