/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: jingle.c,v 1.8 2002/03/03 15:37:00 tf Exp $
 */
#include "aladder.h"
#include "animation.h"

static Image *logo[3], *logo_m[3];

static int
jingle_init(int arg)
{
    Image *work = ALIB.imageNew(327, 174);
    ALIB.imageLoad(work, "logo_c.bmp");

    logo[0] = ALIB.imageNew(183, 46);
    logo_m[0] = ALIB.imageNew(183, 46);
    ALIB.imageCopy(logo[0], work, 0, 0, 144, 0, 183, 46);
    ALIB.imageCopy(logo_m[0], work, 0, 0, 144, 46, 183, 46);

    logo[1] = ALIB.imageNew(144, 87);
    logo_m[1] = ALIB.imageNew(144, 87);
    ALIB.imageCopy(logo[1], work, 0, 0, 0, 0, 144, 87);
    ALIB.imageCopy(logo_m[1], work, 0, 0, 0, 87, 144, 87);

    logo[2] = ALIB.imageNew(127, 16);
    logo_m[2] = ALIB.imageNew(127, 16);
    ALIB.imageCopy(logo[2], work, 0, 0, 144, 92, 127, 16);
    ALIB.imageCopy(logo_m[2], work, 0, 0, 144, 108, 127, 16);

    ALIB.imageFill(alCore.vram, 0, 0, WIDTH, HEIGHT, BLACK);
    ALIB.imageDelete(work);

    REQTBL.flushWindow();

    return True;
}

static int
jingle_anim(int arg)
{
    static int i = 0;

    ALIB.imageFill(alCore.vram, 0, 0, WIDTH, HEIGHT, WHITE);
    if (i < 10) {
	ALIB.imageAddWithMask(alCore.vram, logo[0], logo_m[0], i * 25 - 85,
			      210);
    } else if (i < 20) {
	ALIB.imageAddWithMask(alCore.vram, logo[0], logo_m[0], 155, 210);
	ALIB.imageAddWithMask(alCore.vram, logo[1], logo_m[1],
			      328 + 49 * (20 - i), 186);
    } else if (i <= 30) {
	ALIB.imageAddWithMask(alCore.vram, logo[0], logo_m[0], 155, 210);
	ALIB.imageAddWithMask(alCore.vram, logo[1], logo_m[1], 328, 186);
	ALIB.imageAddWithMask(alCore.vram, logo[2], logo_m[2], 196,
			      250 + 21 * (30 - i));
    }
    REQTBL.flushWindow();

    if (++i > 30)
	return True;

    return False;
}

static int
jingle_quit(int arg)
{
    int i;

    for (i = 0; i < 3; i++) {
	ALIB.imageDelete(logo[i]);
	ALIB.imageDelete(logo_m[i]);
    }
    return True;
}

static int
jingle_none(int arg)
{
    return True;
}

static AnimationScriptCommand jingleScriptCommand[] = {
    {jingle_init, 0},
    {animation_fadein, 0},
    {jingle_anim, 0},
    {animation_sleep, 2000},
    {animation_fadeout, 1000},
    {jingle_quit, 0},
    {NULL, 0},
};


static AnimationScript jingleScript = {
    jingleScriptCommand,
    jingle_none
};

void
run_jingle_engine()
{
    if (!CONFIG.skip_jingle)
	run_animation_engine(jingleScript);
}
