/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001, 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: main.c,v 1.43 2002/03/09 17:41:59 tf Exp $
 */
#define MAIN

#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>

#include "aladder.h"

#ifdef USE_X11
void InitX11(int argc, char *argv[]);
#endif

void DummyMusicLibInit();

#if defined(MUSIC_MPG123) || defined(MUSIC_MPG123ESD)
void Mpg123MusicLibInit();
#endif

void DummySoundLibInit();	/* sound-dummy.c */

#ifdef SOUND_OSS
void OSSSoundLibInit();		/* sound-oss.c */
#endif
#ifdef SOUND_ESD
void ESDSoundLibInit();		/* sound-esd.c */
#endif

void
init_variable()
{
    alCore.alternative_num = 0;

    WAIT.skip = False;
    WAIT.drop_frame = False;

    text_init();
    backlog_init();
    cursor_init();

    ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, BLACK);
    ALIB.imageFill(alCore.sub, 0, 0, WIDTH, HEIGHT, BLACK);
    ALIB.imageFill(alCore.vram, 0, 0, WIDTH, HEIGHT, BLACK);

    ClearScene();
    ClearWorkScene();

    load_script("tenhashi.txt");
}

static void
init_config()
{
    CONFIG.window_size = CONFIG_WINDOW;
    CONFIG.text_speed = CONFIG_TEXT_NORMAL;
    CONFIG.use_effect = True;
    CONFIG.skip_read = False;

    CONFIG.skip_jingle = False;
    CONFIG.skip_opening = False;
    CONFIG.force_skip = False;
}

static void
setup_angellib()
{
    ImageLibInit();

    if (alCore.sound_arc == NULL) {
	fprintf(stderr, "*\n");
	fprintf(stderr,
		"* ɥե뤬ޤ󡥥ɤκԤ鷺³Ԥޤ\n");
	fprintf(stderr, "*\n\n");
	DummySoundLibInit();
    } else {
#ifdef SOUND_OSS
	OSSSoundLibInit();
#endif
#ifdef SOUND_ESD
	ESDSoundLibInit();
#endif
#ifdef USE_SDL
	SDLSoundLibInit();
#endif
    }

#if defined(MUSIC_MPG123) || defined(MUSIC_MPG123ESD)
    Mpg123MusicLibInit();
#endif
#ifdef USE_SDL
    SDLMusicLibInit();
#endif

    if (ALIB.soundOpen() < 0) {
	DummySoundLibInit();
	ALIB.soundOpen();
    }

    if (ALIB.musicOpen() < 0) {
	DummyMusicLibInit();
	ALIB.musicOpen();
    }
}

static int
initialize(int argc, char *argv[])
{
    char path[FILENAME_MAX];
    int dep_argc;
    char **dep_argv;

    srandom((unsigned) getpid());

    SCRIPT.data = NULL;
    SCRIPT.scene_list = NULL;

    alCore.flag = NULL;
    alCore.cg_flag = NULL;
    alCore.prev_flag = NULL;
    alCore.read_flag = NULL;

    setup_directory();

    sprintf(path, "%s/VData_01.dat", DIR.app_dir);
    alCore.sound_arc = open_archive(path);

    setup_angellib();

    sprintf(path, "%s/EData_01.dat", DIR.app_dir);
    alCore.script_arc = open_archive(path);

    sprintf(path, "%s/BData_01.dat", DIR.app_dir);
    alCore.cg_arc = open_archive(path);

    sprintf(path, "%s/jiten.txt", DIR.app_dir);
    dict_init(path);

    ClearScene();

    alCore.brightness = 10;

    alCore.work = ALIB.imageNew(WIDTH, HEIGHT);
    alCore.sub = ALIB.imageNew(WIDTH, HEIGHT);
    alCore.vram = ALIB.imageNew(WIDTH, HEIGHT);

    init_config();
    load_persistent_info();
    parse_argument(argc, argv, &dep_argc, &dep_argv);
    /* 
     * ʬϥץåȥեΩǤʤ
     */
#ifdef USE_X11
    InitX11(dep_argc, dep_argv);
#endif
#ifdef USE_SDL
    InitSDL(dep_argc, dep_argv);
#endif
    return 0;
}

static void
destroy()
{
    save_persistent_info();
    free_script();
    close_archive(alCore.script_arc);

    ALIB.musicClose();
    ALIB.soundClose();
    REQTBL.destroy();
}

void
LongJump(JumpParameter param)
{
    longjmp(alCore.jump, (int) param);
    fprintf(stderr, "Fatal Error: ʥפǤ[%d]\n", param);
    exit(1);
}

int
main(int argc, char *argv[])
{
    initialize(argc, argv);

    switch ((JumpParameter) setjmp(alCore.jump)) {
    case JUMP_NONE:		/*  */
	break;
    case JUMP_RESTART:
	goto main_start;
    case JUMP_JINGLE:
	goto jingle;
    case JUMP_OPENING:
	goto opening;
    case JUMP_TITLE:
	goto title;
    case JUMP_ENDING:
	goto ending;
    case JUMP_END:
	goto end;
    }

  jingle:
    run_jingle_engine();
  opening:
    run_opening_engine();
  title:
    run_title_engine();
  main_start:
    run_engine(ENGINE_GAME);
  ending:
    run_ending_engine();
  end:
    destroy();

    return 0;
}
