/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001, 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: menu.c,v 1.26 2002/03/03 15:37:00 tf Exp $
 */

#include <string.h>

#include "aladder.h"

typedef struct {
    int x;
    int y;
    u_char *string;
    int attr;
} MenuItem;

typedef struct {
    u_char *name;
    MenuItem *menu_item;
    int (*func) (int selection);
} Menu;

int menu_action_main_menu(int selection);
int menu_action_title_confirm(int selection);
int menu_action_exit_confirm(int selection);
int menu_action_file_save(int selection);
int menu_action_file_load(int selection);

static int draw_frame = False;

/* 
 * ˥塼
 */
#define FILE_DIALOG -1

#define SKIP_READ_SCRIPT "ʸϤ򥹥å"
#define NOT_SKIP_READ_SCRIPT "ʸϤ򥹥å"

static MenuItem main_menu[] = {
    {8, 3, "", 1},
    {8, 4, "", 2},
    {8, 5, "ʸä", 3},
    {8, 6, NOT_SKIP_READ_SCRIPT, 4},
    {8, 7, "", 5},
    {8, 8, "ȥ", 6},
    {8, 9, "", 7},
    {8, 10, "ཪλ", 8},
    {0, 0, NULL, 0},
};

static MenuItem main_menu2[] = {
    {8, 3, "", 1},
    {8, 4, "", 2},
    {8, 5, "ʸä", 3},
    {7, 6, SKIP_READ_SCRIPT, 4},
    {8, 7, "", 5},
    {8, 8, "ȥ", 6},
    {8, 9, "", 7},
    {8, 10, "ཪλ", 8},
    {0, 0, NULL, 0},
};

static MenuItem title_confirm[] = {
    {2, 6, "ȥ̤ޤǤ", 0},
    {10, 8, "Ϥ", 1},
    {10, 9, "", 2},
    {0, 0, NULL, 0},
};

static MenuItem exit_confirm[] = {
    {3, 6, "λޤǤ", 0},
    {10, 8, "Ϥ", 1},
    {10, 9, "", 2},
    {0, 0, NULL, 0},
};

static MenuItem file_save[] = {
    {8, 0, "Υ", FILE_DIALOG},
    {0, 0, NULL, 0},
};

static MenuItem file_load[] = {
    {8, 0, "Υ", FILE_DIALOG},
    {0, 0, NULL, 0},
};


static Menu menu[] = {
    {"main_menu", main_menu, menu_action_main_menu},
    {"title_confirm", title_confirm, menu_action_title_confirm},
    {"exit_confirm", exit_confirm, menu_action_exit_confirm},
    {"file_save", file_save, menu_action_file_save},
    {"file_load", file_load, menu_action_file_load},
    {NULL, NULL, NULL},
};

static void
set_menu_line(MenuItem *mi)
{
    int i, j;

    for (i = 0; i < mi->x; i++)
	TEXT.tvram[i][mi->y] = WCHAR_SPACE;

    for (j = 0; j < strlen(mi->string);) {
	if (mi->string[j] < 0x80) {
	    TEXT.tvram[i][mi->y] = mi->string[j];
	    TEXT.attr[i][mi->y] =
		(mi->attr == 0) ? 0 : mi->attr | ATTR_ALTN_MASK;
	    i++;
	    j++;
	} else {
	    TEXT.tvram[i][mi->y] = mi->string[j] << 8 | mi->string[j + 1];
	    TEXT.attr[i][mi->y] =
		(mi->attr == 0) ? 0 : mi->attr | ATTR_ALTN_MASK;
	    i++;
	    j += 2;
	}
    }
}

static void
set_menu_file_selection(MenuItem *menu)
{
    int i;
    MenuItem mi = *menu;

    mi.attr = 0;
    TEXT.view_width = WIDTH - F_MAR - FONT_WIDTH;
    set_menu_line(&mi);

    trap_area_clear();

    for (i = 0; i < 5; i++) {
	u_char date_str[ARG_LEN], desc_str[ARG_LEN];
	MenuItem date, desc;

	date.x = desc.x = 3;
	date.y = i * 3 + 2;
	desc.y = i * 3 + 3;
	date.attr = desc.attr = i + 1;

	get_save_file_info(i, date_str, desc_str);

	date.string = date_str;
	desc.string = desc_str;
	set_menu_line(&date);
	set_menu_line(&desc);

	trap_area_add(F_MAR, (F_SPC + F_HEIGHT) * i + F_TOP,
		      WIDTH - F_MAR * 2, F_HEIGHT);
    }
}

static void
set_menu_vram(MenuItem *menu)
{
    int i;

    clear_tvram();

    if (menu->attr == FILE_DIALOG) {
	draw_frame = True;
	set_menu_file_selection(menu);
	alCore.alternative_num = 5;
    } else {
	draw_frame = False;
	i = 0;
	while (menu->string != NULL) {
	    set_menu_line(menu);
	    menu++;
	    i++;
	}
	alCore.alternative_num = i;
    }

    {
	int backup = TEXT.fullscreen;

	TEXT.fullscreen = True;
	RedisplayScene(alCore.work, SCENE);
	ALIB.imageDisplay(10);
	menu_draw_frame();
	redraw_text();
	REQTBL.flushWindow();

	TEXT.fullscreen = backup;
    }
}

void
menu_set(const char *name)
{
    Menu *p = menu;

    dprintf((stderr, "menu: %s\n", name));

    while (p->name != NULL) {
	if (!strcmp(name, p->name)) {
	    if (!strcmp(name, "main_menu") && CONFIG.skip_read) {
		set_menu_vram(main_menu2);
	    } else {
		set_menu_vram(p->menu_item);
	    }
	    return;
	}
	p++;
    }

    dprintf((stderr, "˥塼λ꤬Ǥ[%s]\n", name));
}

void
menu_clear()
{
    draw_frame = False;
    trap_area_clear();
}

int
menu_action(const char *name, int selection)
{
    Menu *p = menu;

    dprintf((stderr, "menu: %s\n", name));

    while (p->name != NULL) {
	if (!strcmp(name, p->name))
	    return p->func(selection);
	p++;
    }

    return 0;
}

int
menu_action_main_menu(int selection)
{
    dprintf((stderr, "main_menu [%d]\n", selection));

    switch (selection) {
    case 1:			/*  */
	set_command("#EnterMenuMode(file_save)");
	menu_set("file_save");
	break;
    case 2:			/*  */
	set_command("#EnterMenuMode(file_load)");
	menu_set("file_load");
	break;
    case 3:			/* ʸä */
	pop_text();
	pop_stack();
	push_stack(STAT_CG);
	set_command("#EnterCGMode");
	push_stack(STAT_COMMAND);
	break;
    case 4:			/* ʸϤ򥹥å */
	CONFIG.skip_read = !CONFIG.skip_read;
	set_command("#EnterMenuMode(main_menu)");
	menu_set("main_menu");
	break;
    case 5:			/*  */
	pop_stack();
	enter_config_mode();
	break;
    case 6:			/* ȥ */
	set_command("#EnterMenuMode(title_confirm)");
	menu_set("title_confirm");
	break;
    case 7:			/*  */
	return True;
    case 8:			/* ཪλ */
	set_command("#EnterMenuMode(exit_confirm)");
	menu_set("exit_confirm");
	break;
    default:
	dprintf((stderr, "Ǥ[%d]\n", selection));
    }

    return False;
}

int
menu_action_title_confirm(int selection)
{
    dprintf((stderr, "title_confirm [%d]\n", selection));

    switch (selection) {
    case 1:			/* Ϥ */
	FadeOut();
	ALIB.musicFadeOut();
	LongJump(JUMP_TITLE);
	break;
    case 2:			/*  */
	return True;
    default:
	dprintf((stderr, "Ǥ[%d]\n", selection));
    }

    return False;
}

int
menu_action_exit_confirm(int selection)
{
    dprintf((stderr, "exit_confirm [%d]\n", selection));

    switch (selection) {
    case 1:			/* Ϥ */
	FadeOut();
	ALIB.musicFadeOut();
	LongJump(JUMP_END);
	break;
    case 2:			/*  */
	return True;
    default:
	dprintf((stderr, "Ǥ[%d]\n", selection));
    }

    return False;
}

int
menu_action_file_save(int selection)
{
    char buf[4];

    sprintf(buf, "%03d", selection - 1);
    draw_frame = False;
    TEXT.view_width = WIDTH;
    SaveGame(buf);

    return True;
}


int
menu_action_file_load(int selection)
{
    char buf[4];
    int state = 0;

    ALIB.musicFadeOut();

    ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, BLACK);
    ALIB.imageCopy(alCore.sub, alCore.vram, 0, 0, 0, 0, WIDTH, HEIGHT);

    while (!FadePattern(&state, True));

    sprintf(buf, "%03d", selection - 1);
    draw_frame = False;
    TEXT.view_width = WIDTH;
    LoadGame(buf);

    push_stack(STAT_MENU);	/* åˤʤΤ佼 */

    return True;
}

void
menu_draw_frame()
{
    int res, i;

    if (!draw_frame)
	return;

    res = (ATTR_ALL_MASK ^ ATTR_ALTN_MASK) & TEXT.highlight;

    for (i = 0; i < 5; i++) {
	Color c = WHITE;
	if (res == i + 1)
	    c = YELLOW;
	ALIB.imageBox(alCore.vram, F_MAR, (F_SPC + F_HEIGHT) * i + F_TOP,
		      WIDTH - F_MAR * 2, F_HEIGHT, c);
    }
}

void
menu_set_selected(int sel)
{
    if (sel)
	TEXT.highlight = sel | ATTR_ALTN_MASK;
    menu_draw_frame();
    redraw_text();
    REQTBL.flushWindow();
}
