/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001, 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: music-mpg123.c,v 1.11 2002/03/04 18:11:06 tf Exp $
 */
#if defined(MUSIC_MPG123) || defined(MUSIC_MPG123ESD)

#include <stdio.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "aladder.h"
#include "music.h"

#define STAT_WAIT_PLAY 0
#define STAT_PLAY 1
#define STAT_WAIT_STOP 2
#define STAT_STOP 3

#ifdef MUSIC_MPG123ESD
#include <esd.h>
#define ESD_ENTRY "al_bgm"
#endif

char *const mpg123path[] = {
#ifdef MUSIC_MPG123ESD
    "esddsp",
    "--name=" ESD_ENTRY,
#endif
    "mpg123",
    "--remote",
    "/dev/null",
    NULL,
};

static MusicInfo minfo;
static int state;

#ifdef MUSIC_MPG123ESD
static struct {
    int esd;
    esd_info_t *esdctl;
    int base_volume;
} einfo;
#endif

int
mpg123_device_open(MusicInfo *minfo)
{
    int ifds[2], ofds[2];
    int pid;

    state = STAT_STOP;

    if (pipe(ifds) < 0) {
	perror("pipe");
	return -1;
    }

    if (pipe(ofds) < 0) {
	perror("pipe");
	return -1;
    }

    if ((pid = fork()) < 0) {
	perror("fork");
	return -1;
    }

    if (pid == 0) {
	dup2(ifds[0], 0);
	close(ifds[0]);
	close(ifds[1]);

	dup2(ofds[1], 1);
	close(ofds[1]);
	close(ofds[0]);

	execvp(mpg123path[0], mpg123path);
	perror("execvp");
	exit(1);
    }
    minfo->pid = pid;

    minfo->control_fd = ifds[1];
    close(ifds[0]);

    minfo->result_fd = ofds[0];
    close(ofds[1]);

    minfo->control_fp = fdopen(ifds[1], "w");

#ifdef MUSIC_MPG123ESD
    einfo.esd = esd_open_sound(NULL);
    if (einfo.esd < 0) {
	fprintf(stderr, "esd 򳫤ޤ.\n");
	return -1;
    }
#endif
    return 0;
}

void
mpg123_device_close(MusicInfo *minfo)
{
    fprintf(minfo->control_fp, "QUIT\n");
    fflush(minfo->control_fp);

    wait(&minfo->pid);

#ifdef MUSIC_MPG123ESD
    esd_close(einfo.esd);
#endif
}

static void
mpg123_play(MusicInfo *minfo)
{
    state = STAT_WAIT_PLAY;

    dprintf((stderr, "Play %s\n", minfo->name));
    fprintf(minfo->control_fp, "LOAD %s\n", minfo->name);
    fflush(minfo->control_fp);
}

static int
get_line(char *line, char *buf, int idx)
{
    int i, j;

    for (i = idx, j = 0; buf[i] != '\0'; i++, j++) {
	if (buf[i] == 0x0a) {
	    line[j] = '\0';
	    return j + 1;
	}
	line[j] = buf[i];
    }
    return 0;
}

static int
mpg123_status(MusicInfo *minfo)
{
#define LINE_LEN 512
    char buf[BUFSIZ], rbuf[BUFSIZ], line[LINE_LEN];
    static int idx = 0;
    int i, j;
    int maxfd, ret;
    fd_set watchset;
    struct timeval tv;

    tv.tv_sec = 0;
    tv.tv_usec = 0;

    FD_ZERO(&watchset);
    FD_SET(minfo->result_fd, &watchset);
    maxfd = minfo->result_fd;

    if (select(maxfd + 1, &watchset, NULL, NULL, &tv) > 0) {
	if (FD_ISSET(minfo->result_fd, &watchset)) {
	    ret = read(minfo->result_fd, rbuf, BUFSIZ);
	    rbuf[ret] = '\0';

	    for (i = 0, j = idx; i < ret; i++, j++) {
		buf[j] = rbuf[i];
	    }
	    buf[j] = '\0';

	    while ((ret = get_line(line, buf, idx)) > 0) {
		switch (line[1]) {
		case 'P':
		    if (line[3] == '0') {
			minfo->playing = False;
			if (state == STAT_PLAY) {
			    dprintf((stderr, "music play[%s] done.\n",
				     minfo->name));
			    mpg123_play(minfo);
			} else {
			    state = STAT_STOP;
			}
		    }
		    break;
		case 'F':
		    minfo->playing = True;
		    if (state == STAT_WAIT_PLAY)
			state = STAT_PLAY;
		    break;
		}
		idx += ret;
	    }

	    i = 0;
	    while (buf[idx] != '\0') {
		buf[i++] = buf[idx++];
	    }
	    buf[i] = '\0';
	    idx = i;
	}
    }
    return minfo->playing;
}

static void
mpg123_stop(MusicInfo *minfo)
{
    state = STAT_WAIT_STOP;
    dprintf((stderr, "MP3 stop.\n"));
    fprintf(minfo->control_fp, "STOP\n");
    fflush(minfo->control_fp);

    while (mpg123_status(minfo));
}

static int
MusicOpen()
{
    return mpg123_device_open(&minfo);
}

#ifdef MUSIC_MPG123ESD
static void
esd_set_volume(int v)
{
    static esd_player_info_t *p = NULL;
    static int first = True;
    int vol;

    einfo.esdctl = esd_get_all_info(einfo.esd);
    p = einfo.esdctl->player_list;
    while (p != NULL) {
	if (!strcmp(p->name, ESD_ENTRY))
	    break;
	p = p->next;
    }
    if (p == NULL)
	return;

    if (first) {
	first = False;
	einfo.base_volume = p->left_vol_scale;
    }

    vol = einfo.base_volume * v / 10;
    esd_set_stream_pan(einfo.esd, p->source_id, vol, vol);
}
#endif

static void
MusicClose()
{
    mpg123_device_close(&minfo);
}

static void
MusicLoad(const char *name)
{
    char path[FILENAME_MAX];
    struct stat sb;

    strcpy(SCENE.bgm, name);
    snprintf(path, sizeof path, "%s/%s", DIR.music_dir, name);
    if (stat(path, &sb) < 0) {
	fprintf(stderr, "mp3 ե[%s]ޤ\n", path);
	return;
    }
    strcpy(minfo.name, path);
}

static void
MusicPlay()
{
#ifdef MUSIC_MPG123ESD
    esd_set_volume(10);
#endif

    mpg123_play(&minfo);
}

static void
MusicStop()
{
    dprintf((stderr, "stop music\n"));
    mpg123_stop(&minfo);
}

static void
MusicFadeOut()
{
#ifdef MUSIC_MPG123ESD
    int i;

    for (i = 10; i >= 0; i--) {
	mpg123_status(&minfo);
	esd_set_volume(i);
	usleep(100 * 1000);
    }
#endif
    MusicStop();
}

static int
MusicStatus()
{
    return mpg123_status(&minfo);
}

void
Mpg123MusicLibInit()
{
    ALIB.musicOpen = MusicOpen;
    ALIB.musicClose = MusicClose;
    ALIB.musicLoad = MusicLoad;
    ALIB.musicPlay = MusicPlay;
    ALIB.musicStop = MusicStop;
    ALIB.musicFadeOut = MusicFadeOut;
    ALIB.musicStatus = MusicStatus;
}

#endif				/* MUSIC_MPG123 || MUSIC_MPG123ESD */
