/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: opening.c,v 1.10 2002/03/05 17:17:14 tf Exp $
 */
#include "aladder.h"
#include "animation.h"

#define CH_KASUMI 0
#define CH_MISAKI 1
#define CH_RYNDIS 2
#define CH_MIA 3
#define CH_SAYAKO 4

static Image *feather[4], *feather_m[4];
static Image *chara[5], *chara_m[5];
static Image *l_chara[2], *l_chara_m[2];
static int f_xoff[4] = { 0, 35, 70, 105 };
static int f_frame[] = { 0, 0, 1, 1, 2, 2, 3, 3, 3, 3, 2, 2, 1, 1, 0, 0 };
static timer t;

#ifdef FAST_EFFECT
#define FadePattern FadePattern8
#define WipeLtoR WipeLtoR2
#endif

static int
opening_init(int arg)
{
    int i;
    static char *chara_file[] = {
	"Opksm.bmp", "Opmis.bmp", "Oprin.bmp", "Opmia.bmp",
    };
    static char *mask_file[] = {
	"mOpksm.bmp", "mOpmis.bmp", "mOprin.bmp", "mOpmia.bmp",
    };
    static char *l_chara_file[] = {
	"Opkscr.bmp", "Opsycr.bmp",
    };
    static char *l_mask_file[] = {
	"mOpkscr.bmp", "mOPSYCR.BMP",
    };
    static int l_chara_height[] = { 520, 560 };
    Image *work = ALIB.imageNew(320, 160);

    /* ޡ */
    init_timer(&t);

    /*  */
    ALIB.imageLoad(work, "op_c.bmp");
    for (i = 0; i < 4; i++) {
	feather[i] = ALIB.imageNew(80, 80);
	feather_m[i] = ALIB.imageNew(80, 80);
	ALIB.imageCopy(feather[i], work, 0, 0, 80 * i, 0, 80, 80);
	ALIB.imageCopy(feather_m[i], work, 0, 0, 80 * i, 80, 80, 80);
    }
    ALIB.imageDelete(work);

    /* 饯Ω */
    for (i = 0; i < 4; i++) {
	chara[i] = ALIB.imageNew(240, 480);
	chara_m[i] = ALIB.imageNew(240, 480);
	ALIB.imageLoad(chara[i], chara_file[i]);
	ALIB.imageLoad(chara_m[i], mask_file[i]);
    }

    /*  &  */
    for (i = 0; i < 2; i++) {
	l_chara[i] = ALIB.imageNew(640, l_chara_height[i]);
	l_chara_m[i] = ALIB.imageNew(640, l_chara_height[i]);

	ALIB.imageLoad(alCore.work, l_chara_file[i]);
	ALIB.imageCopy(l_chara[i], alCore.work, 0, 0, 0, 0, 640, 480);

	ALIB.imageLoad(alCore.work, l_mask_file[i]);
	ALIB.imageCopy(l_chara_m[i], alCore.work, 0, 0, 0, 0, 640, 480);
    }

    work = ALIB.imageNew(640, 100);
    ALIB.imageLoad(work, "Opsycr2.bmp");
    ALIB.imageCopy(l_chara[0], work, 0, 480, 0, 60, 640, 40);
    ALIB.imageCopy(l_chara[1], work, 0, 480, 0, 0, 640, 60);

    ALIB.imageLoad(work, "mOPSYCR2.BMP");
    ALIB.imageCopy(l_chara_m[0], work, 0, 480, 0, 60, 640, 40);
    ALIB.imageCopy(l_chara_m[1], work, 0, 480, 0, 0, 640, 60);

    ALIB.imageDelete(work);

    /*  */
    ALIB.imageFill(alCore.vram, 0, 0, WIDTH, HEIGHT, BLACK);
    REQTBL.flushWindow();

    ALIB.imageLoad(alCore.work, "OpBg028.bmp");
    ALIB.imageFill(alCore.sub, 0, 0, WIDTH, HEIGHT, BLACK);

    return True;
}

#define FEATHER_XOFFSET 100
#define FEATHER_YOFFSET 50
#define FRAME_MERGE 40
static int
opening_feather(int arg)
{
    static int i, state = 0;
    int alpha, f, x, y;

    get_timer(&t);
    i = t.msec / 100;

    alpha = 255 * (i - FRAME_MERGE) * 2 / 100;
    f = f_frame[i % (sizeof(f_frame) / sizeof(f_frame[0]))];
    x = 55 * i / 10 + f_xoff[f] - FEATHER_XOFFSET;
    y = 35 * i / 10 - FEATHER_YOFFSET;

    switch (state) {
    case 0:
	ALIB.imageFill(alCore.vram, 0, 0, WIDTH, HEIGHT, BLACK);
	ALIB.imageAddWithMask(alCore.vram, feather[f], feather_m[f], x, y);
	REQTBL.flushWindow();
	if (i >= FRAME_MERGE) {
	    ALIB.musicLoad("01tenhashi.mp3");
	    ALIB.musicPlay();
	    state++;
	}
	return False;
    case 1:
	ALIB.imageMerge(alCore.vram, alCore.work, alCore.sub, alpha);
	ALIB.imageAddWithMask(alCore.vram, feather[f], feather_m[f], x, y);
	REQTBL.flushWindow();
	if (i < 140)
	    return False;
	state++;
    }

    ALIB.imageLoad(alCore.sub, "OpBg028.bmp");
    state = 0;
    return True;
}
#undef FEATHER_XOFFSET
#undef FEATHER_YOFFSET
#undef FRAME_MERGE

static int
opening_narrowing(int arg)
{
    static int i, state = 0;
    int h1, h2;

    get_timer(&t);
    i = t.msec / 100 - 140;

    ALIB.imageLoad(alCore.vram, "OpBg023_01.bmp");
    ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, BLACK);
    ALIB.imageCopy(alCore.work, alCore.vram, 0, 50, 0, i / 2, WIDTH,
		   HEIGHT - 50 - i / 2);
    h1 = i * 8 / 10 + HEIGHT / 8;
    h2 = HEIGHT - h1 * 2;
    ALIB.imageFill(alCore.work, 0, 0, WIDTH, h1, BLACK);
    ALIB.imageFill(alCore.work, 0, HEIGHT - h1 - 1, WIDTH, h1, BLACK);

    switch (state) {
    case 0:
	ALIB.imageMerge(alCore.vram, alCore.work, alCore.sub,
			255 * i / 16);
	REQTBL.flushWindow();
	if (i >= 16)
	    state++;
	return False;
    case 1:
	ALIB.imageCopy(alCore.vram, alCore.work, 0, 0, 0, 0, WIDTH,
		       HEIGHT);
	REQTBL.flushWindow();
	if (i < 80)
	    return False;
	state++;
    }

    state = 0;
    return True;
}

static void
show_characters(int ch1, int ch2, int cnt, int y)
{
    int x1 = cnt * 75 / 10 - 120;
    int x2 = WIDTH - cnt * 75 / 10 - 120;

    ALIB.imageAddWithMask(alCore.vram, chara[ch2], chara_m[ch2], x2, y);
    ALIB.imageAddWithMask(alCore.vram, chara[ch1], chara_m[ch1], x1, y);
    REQTBL.flushWindow();
}

static int
opening_kasumi_misaki(int arg)
{
    static int i, state = 0, es = 0;
    int ret, y;

    get_timer(&t);
    i = t.msec / 100 - 240;

    y = i;

    switch (state) {
    case 0:
	ALIB.imageCopy(alCore.sub, alCore.vram, 0, 0, 0, 0, WIDTH, HEIGHT);
	ALIB.imageLoad(alCore.work, "OpBg028.bmp");
	i = 0;
	state++;
    case 1:
	ret = FadePattern(&es, False);
	show_characters(CH_KASUMI, CH_MISAKI, i, y);
	if (ret) {
	    state++;
	    es = 0;
	}
	return False;
    case 2:
	ALIB.imageLoad(alCore.vram, "OpBg028.bmp");
	show_characters(CH_KASUMI, CH_MISAKI, i, y);
	if (i > 35) {
	    ALIB.imageLoad(alCore.work, "OpBg003_01.bmp");
	    state++;
	}
	return False;
    case 3:
	ret = FadePattern(&es, False);
	show_characters(CH_KASUMI, CH_MISAKI, i, y);
	if (ret) {
	    state++;
	    es = 0;
	}
	return False;
    case 4:
	ALIB.imageLoad(alCore.vram, "OpBg003_01.bmp");
	show_characters(CH_KASUMI, CH_MISAKI, i, y);

	if (i > 100)
	    state++;
	return False;
    }
    state = 0;
    return True;
}

static int
opening_ryndis_mia(int arg)
{
    static int i = 0, state = 0, es = 0;
    int ret, y;

    get_timer(&t);
    i = t.msec / 100 - 340;

    y = 35;

    switch (state) {
    case 0:
	ALIB.imageLoad(alCore.work, "OpRcg002.bmp");
	i = 0;
	state++;
    case 1:
	ret = FadePattern(&es, False);
	show_characters(CH_RYNDIS, CH_MIA, i, y);
	if (ret) {
	    state++;
	    es = 0;
	}
	return False;
    case 2:
	ALIB.imageLoad(alCore.vram, "OpRcg002.bmp");
	show_characters(CH_RYNDIS, CH_MIA, i, y);
	if (i > 35) {
	    ALIB.imageLoad(alCore.work, "OpTcg010_01.bmp");
	    state++;
	}
	return False;
    case 3:
	ret = FadePattern(&es, False);
	show_characters(CH_RYNDIS, CH_MIA, i, y);
	if (ret) {
	    state++;
	    es = 0;
	}
	return False;
    case 4:
	ALIB.imageLoad(alCore.vram, "OpTcg010_01.bmp");
	show_characters(CH_RYNDIS, CH_MIA, i, y);

	if (i > 100)
	    state++;
	return False;
    }
    state = 0;
    return True;
}

static void
show_large_characters(Image *image, int cnt)
{
    ALIB.imageAddWithMask(image, l_chara[0], l_chara_m[0],
			  400 - cnt * 2, 190 - cnt);
    ALIB.imageAddWithMask(image, l_chara[1], l_chara_m[1],
			  cnt * 2 - 350, cnt - 60);
    REQTBL.flushWindow();
}

static int
opening_sephiroth(int arg)
{
    static int i, state = 0, es = 0, offset = -1;
    int ret;

    get_timer(&t);
    i = t.msec / 100 - 440;

    switch (state) {
    case 0:
	ALIB.imageLoad(alCore.work, "Opsmcr.bmp");
	state++;
    case 1:
	ret = FadePattern(&es, True);
	if (ret) {
	    state++;
	    es = 0;
	    ALIB.imageCopy(alCore.vram, alCore.sub, 0, 0, 0, 0, WIDTH,
			   HEIGHT);
	    show_large_characters(alCore.work, 0);
	}
	return False;
    case 2:
	ret = FadePattern(&es, True);
	if (ret) {
	    state++;
	    es = 0;
	}
	return False;
    case 3:
	if (offset < 0)
	    offset = i;
	ALIB.imageLoad(alCore.vram, "Opsmcr.bmp");
	show_large_characters(alCore.vram, i - offset);
	REQTBL.flushWindow();
	if (i > 220)
	    state++;
	return False;
    }

    ALIB.imageCopy(alCore.sub, alCore.vram, 0, 0, 0, 0, WIDTH, HEIGHT);
    offset = -1;
    state = 0;
    return True;
}

static void
opening_string()
{
    int cnt, txt[32];
    static struct {
	char *string;
	int x;
	int y;
	int start_cnt;
    } str[] = {
	{
	"Ĥ ̴Ƥޤ", 50, 120, 0}, {
	"̿ ʤȿƤ", 110, 310, 30}, {
	"ͤä Υʤ褦", 180, 200, 60}, {
	"ζ ꤤ᤿", 20, 390, 90}, {
	"ŷȤ񤤹ߤꤿ ϵξ", 240 - 50, 75, 120}, {
	"ʤȤĤޤǤ ФäƤ", 10, 235, 150}, {
	"⤷äƤ ФˤƤ", 250 - 50, 420, 180}, {
	"ʤΤΰ Υʤ", 100, 325, 210}, {
	NULL, 0, 0, 0}
    }, *p;

    get_timer(&t);
    cnt = t.msec / 100 - 660;

    for (p = str; p->string != NULL; p++) {
	int lcnt, y, alpha;

	lcnt = cnt - p->start_cnt;
	if (lcnt < 0 || lcnt > 100)
	    continue;

	y = p->y - lcnt / 10;
	if (lcnt < 20) {
	    alpha = 0xff * lcnt / 20;
	} else if (lcnt > 80) {
	    alpha = 0xff * (100 - lcnt) / 20;
	} else {
	    alpha = 0xff;
	}

	euc_to_text(p->string, txt);
	ALIB.drawImageString(txt, p->x + 2, y + 2, BLACK, alpha);
	ALIB.drawImageString(txt, p->x, y, WHITE, alpha);
    }
}

static int
opening_visual(int arg)
{
    static int i, state = 0, es = 0;
    int ret;
    static char *visual[] =
	{ "OpKcg035.bmp", "OpMcg2_2.bmp", "OpRcg019_01.bmp",
	"OpTcg023.bmp", "OpScg002.bmp"
    };

    get_timer(&t);
    i = t.msec / 100 - arg * 50 - 660;

    switch (state) {
    case 0:
	ALIB.imageLoad(alCore.work, visual[arg]);
	state++;
    case 1:
	ret = WipeLtoR(&es, False);
	ALIB.imageCopy(alCore.vram, alCore.sub, 0, 0, 0, 0, WIDTH, HEIGHT);
	opening_string();
	REQTBL.flushWindow();
	if (ret) {
	    state++;
	    es = 0;
	}
	return False;
    case 2:
	ALIB.imageCopy(alCore.vram, alCore.work, 0, 0, 0, 0, WIDTH,
		       HEIGHT);
	opening_string();
	REQTBL.flushWindow();
	if (i < 50)
	    return False;
	if (arg == 4 && i < 80)
	    return False;
    }

    state = 0;
    return True;
}

static int
opening_wait(int arg)
{
    static int completed = True;

    if (completed)
	completed = False;

    get_timer(&t);

    if (t.msec > arg)
	completed = True;

    return completed;
}

static int
opening_sky(int arg)
{
    static int i = 0;

    if (i == 0) {
	ALIB.imageCopy(alCore.sub, alCore.vram, 0, 0, 0, 0, WIDTH, HEIGHT);
	ALIB.imageLoad(alCore.work, "Opkc29.bmp");
    }

    if (!EffectAlphaBlend(&i))
	return False;

    i = 0;
    return True;
}

static int
opening_sky2(int arg)
{
    static int i = 0;

    if (i == 0) {
	ALIB.imageLoad(alCore.sub, "Opkc29.bmp");
	ALIB.imageLoad(alCore.work, "OpKcg029.bmp");
    }

    if (!EffectAlphaBlend(&i))
	return False;

    i = 0;
    return True;
}

static int
opening_switch_sky2_feather2(int arg)
{
    static int i = 0;

    if (i == 0) {
	ALIB.imageLoad(alCore.sub, "OpKcg029.bmp");
	ALIB.imageLoad(alCore.work, "Opksscr2.bmp");
    }

    if (!EffectAlphaBlend(&i))
	return False;

    i = 0;
    return True;
}

static int
opening_feather2(int arg)
{
    static int i = 0;
    int f;

    if (i == 0)
	ALIB.imageLoad(alCore.work, "Opksscr2.bmp");

    ALIB.imageCopy(alCore.vram, alCore.work, 0, 0, 0, 0, WIDTH, HEIGHT);

    f = f_frame[i % (sizeof(f_frame) / sizeof(f_frame[0]))];
    ALIB.imageAddWithMask(alCore.vram, feather[f], feather_m[f],
			  3 * i + 20 + f_xoff[f], 4 * i);
    REQTBL.flushWindow();

    if (++i > 80)
	return True;

    return False;
}

static int
opening_scrolling(int arg)
{
    static int h = 0;

    if (h == 0) {
	ALIB.imageLoad(alCore.work, "Opksscr1.bmp");
	ALIB.imageLoad(alCore.sub, "Opksscr2.bmp");
	ALIB.imageAddWithMask(alCore.sub, feather[1], feather_m[1],
			      3 * 80 + 20 + f_xoff[1], 4 * 80);
    }

    ALIB.imageCopy(alCore.vram, alCore.work, 0, 0, 0, HEIGHT - h, WIDTH,
		   h);
    ALIB.imageCopy(alCore.vram, alCore.sub, 0, h, 0, 0, WIDTH, HEIGHT - h);

    REQTBL.flushWindow();

    h += 5;
    if (h > 480)
	return True;

    return False;
}

static int
opening_quit(int arg)
{
    int i;

    ALIB.musicFadeOut();

    for (i = 0; i < 4; i++) {
	ALIB.imageDelete(feather[i]);
	ALIB.imageDelete(feather_m[i]);
    }
    for (i = 0; i < 4; i++) {
	ALIB.imageDelete(chara[i]);
	ALIB.imageDelete(chara_m[i]);
    }
    for (i = 0; i < 2; i++) {
	ALIB.imageDelete(l_chara[i]);
	ALIB.imageDelete(l_chara_m[i]);
    }
    return True;
}

static int
opening_interrupt(int arg)
{
    static int state = 0;

    opening_quit(0);
    ALIB.imageCopy(alCore.sub, alCore.vram, 0, 0, 0, 0, WIDTH, HEIGHT);
    ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, BLACK);

    while (!FadePattern(&state, True));
    state = 0;

    push_stack(STAT_ANIM_QUIT);
    return True;
}

static AnimationScriptCommand openingScriptCommand[] = {
    {opening_init, 0},
    {opening_feather, 0},
    {opening_narrowing, 0},
    {animation_fadeout, 2000},
    {opening_kasumi_misaki, 0},
    {opening_ryndis_mia, 0},
    {opening_sephiroth, 0},
    {opening_visual, CH_KASUMI},
    {opening_visual, CH_MISAKI},
    {opening_visual, CH_RYNDIS},
    {opening_visual, CH_MIA},
    {opening_visual, CH_SAYAKO},
    {opening_wait, 94000},
    {opening_sky, 0},
    {opening_wait, 99000},
    {opening_sky2, 0},
    {opening_wait, 106000},
    {opening_switch_sky2_feather2, 0},
    {opening_feather2, 0},
    {opening_scrolling, 0},
    {animation_sleep, 2000},
    {animation_whiteout, 0},
    {animation_sleep, 2000},
    {animation_fadeout_pattern, 0},
    {animation_sleep, 2000},
    {opening_quit, 0},
    {NULL, 0},
};

static AnimationScript openingScript = {
    openingScriptCommand,
    opening_interrupt
};

void
run_opening_engine()
{
    if (!CONFIG.skip_opening)
	run_animation_engine(openingScript);
}
