/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: prev_mode.c,v 1.9 2002/03/06 15:53:14 tf Exp $
 */

#include "aladder.h"
#include "prev_list.h"

static Image *prev_r, *back_r, *next_r;
static Image *prev_p, *back_p, *next_p;
static Image *tn[16], *tn_d[16];

#define PAGE_NUM 2
static int page;
static int sel = -1;

/* 
 * H ɽ⡼
 */
static void
init_prev_mode_widget()
{
    int i;

    ALIB.imageLoad(alCore.sub, "title_c.bmp");

    prev_r = ALIB.imageNew(100, 44);
    prev_p = ALIB.imageNew(100, 44);
    ALIB.imageCopy(prev_r, alCore.sub, 0, 0, 100, 319, 100, 44);
    ALIB.imageCopy(prev_p, alCore.sub, 0, 0, 100, 363, 100, 44);

    back_r = ALIB.imageNew(200, 44);
    back_p = ALIB.imageNew(200, 44);
    ALIB.imageCopy(back_r, alCore.sub, 0, 0, 349, 88, 200, 44);
    ALIB.imageCopy(back_p, alCore.sub, 0, 0, 0, 407, 200, 44);

    next_r = ALIB.imageNew(100, 44);
    next_p = ALIB.imageNew(100, 44);
    ALIB.imageCopy(next_r, alCore.sub, 0, 0, 0, 319, 100, 44);
    ALIB.imageCopy(next_p, alCore.sub, 0, 0, 0, 363, 100, 44);

    for (i = 0; i < 16; i++) {
	tn[i] = ALIB.imageNew(128, 96);
	tn_d[i] = ALIB.imageNew(128, 96);
    }
}

static void
destroy_prev_mode_widget()
{
    int i;

    ALIB.imageDelete(prev_r);
    ALIB.imageDelete(prev_p);

    ALIB.imageDelete(back_r);
    ALIB.imageDelete(back_p);

    ALIB.imageDelete(next_r);
    ALIB.imageDelete(next_p);

    for (i = 0; i < 16; i++) {
	ALIB.imageDelete(tn[i]);
	ALIB.imageDelete(tn_d[i]);
    }
}

static void
setup_prev_mode()
{
    int i, j, x, y;

    trap_area_clear();
    trap_area_add(104, 436, 100, 44);
    trap_area_add(220, 436, 200, 44);
    trap_area_add(436, 436, 100, 44);

    for (i = 0, y = 48; i < 4; i++, y += 96) {
	for (j = 0, x = 64; j < 4; j++, x += 128) {
	    trap_area_add(x, y, 128, 96);
	}
    }

    redraw_prev_mode();
}

static void
set_thumbnail()
{
    int i;

    for (i = 0; i < 16; i++) {
	ALIB.imageLoad(tn[i], prev_index[page][i].name);
	ALIB.imageCopy(tn_d[i], tn[i], 0, 0, 0, 0, 128, 96);
	if (strcmp(prev_index[page][i].name, "t_no.bmp"))
	    ALIB.imageDarken(tn_d[i], 12);
    }
}

void
enter_prev_mode()
{
    page = 0;
    init_prev_mode_widget();
    push_stack(STAT_TTL_PREVMODE);
    set_thumbnail();
    setup_prev_mode();
}

void
leave_prev_mode()
{
    destroy_prev_mode_widget();
}

void
forward_prev_mode()
{
    page = (page + 1 + PAGE_NUM)
	% PAGE_NUM;
    dprintf((stderr, "forward <%d>\n", page));
    set_thumbnail();
    redraw_prev_mode();
}

void
backward_prev_mode()
{
    page = (page - 1 + PAGE_NUM)
	% PAGE_NUM;
    dprintf((stderr, "backward <%d>\n", page));
    set_thumbnail();
    redraw_prev_mode();
}

void
redraw_prev_mode()
{
    int i, j, x, y, idx;
    char str[6];
    int txt[6];

    ALIB.imageLoad(alCore.vram, "prev_bg.bmp");

    sprintf(str, "%d/2", page + 1);
    euc_to_text(str, txt);
    ALIB.drawImageString(txt, 575, 12, WHITE, 0xff);

    ALIB.imageCopy(alCore.vram, prev_r, 104, 436, 0, 0, 100, 44);
    ALIB.imageCopy(alCore.vram, back_r, 220, 436, 0, 0, 200, 44);
    ALIB.imageCopy(alCore.vram, next_r, 436, 436, 0, 0, 100, 44);
    switch (trap_area_selected()) {
    case 0:
	ALIB.imageCopy(alCore.vram, prev_p, 104, 436, 0, 0, 100, 44);
	break;
    case 1:
	ALIB.imageCopy(alCore.vram, back_p, 220, 436, 0, 0, 200, 44);
	break;
    case 2:
	ALIB.imageCopy(alCore.vram, next_p, 436, 436, 0, 0, 100, 44);
	break;
    }

    for (i = 0, idx = 0, y = 48; i < 4; i++, y += 96) {
	for (j = 0, x = 64; j < 4; j++, x += 128, idx++) {
	    char num[4];
	    sprintf(num, "%d", prev_index[page][idx].id);
	    if (!isset_flag(alCore.prev_flag, num)
		&& prev_index[page][idx].id >= 0)
		continue;
	    if (trap_area_selected() - 3 == idx) {
		ALIB.imageCopy(alCore.vram, tn[idx], x, y, 0, 0, 128, 96);
	    } else {
		ALIB.imageCopy(alCore.vram, tn_d[idx], x, y, 0, 0, 128,
			       96);
	    }
	}
    }
    REQTBL.flushWindow();
}

void
proc_prev_mode()
{
    char num[4], script[ARG_LEN];

    sel = trap_area_selected() - 3;

    if (sel < 0)
	return;

    if (prev_index[page][sel].id < 0)
	return;

    sprintf(num, "%d", prev_index[page][sel].id);
    if (!isset_flag(alCore.prev_flag, num))
	return;

    sprintf(script, "Prev%02d.txt", prev_index[page][sel].id + 1);

    ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, BLACK);
    ALIB.imageFill(alCore.vram, 0, 0, WIDTH, HEIGHT, BLACK);
    ALIB.imageFill(alCore.sub, 0, 0, WIDTH, HEIGHT, BLACK);
    REQTBL.flushWindow();

    load_script(script);

    run_engine(ENGINE_PREV);

    clear_stack();
    enter_prev_mode();
}
