/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001, 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: scene.c,v 1.12 2002/03/09 17:41:59 tf Exp $
 */
#include <stdio.h>

#include "aladder.h"

static Scene work_scene, backup_scene;
static int bri_backup;

void
ClearScene()
{
    SCENE.bg[0] = '\0';
    SCENE.left[0] = '\0';
    SCENE.middle[0] = '\0';
    SCENE.right[0] = '\0';
    SCENE.bgm[0] = '\0';
    SCENE.pos = 0;
}

void
ClearWorkScene()
{
    work_scene.bg[0] = '\0';
    work_scene.left[0] = '\0';
    work_scene.middle[0] = '\0';
    work_scene.right[0] = '\0';
}

void
WorkSceneBg(const char *bg)
{
    strcpy(work_scene.bg, bg);
}

void
WorkSceneOutBg(const char *bg)
{
    char s[ARG_LEN];
    sprintf(s, "#%s", bg);
    strcpy(work_scene.bg, s);
}

void
WorkSceneClearAllChara()
{
    work_scene.left[0] = '\0';
    work_scene.middle[0] = '\0';
    work_scene.right[0] = '\0';
}

void
WorkSceneClearChara(const char *name)
{
    if (!strcmp(work_scene.left, name))
	work_scene.left[0] = '\0';
    if (!strcmp(work_scene.middle, name))
	work_scene.middle[0] = '\0';
    if (!strcmp(work_scene.right, name))
	work_scene.right[0] = '\0';
}

void
WorkSceneChara(const char *pos, const char *cg)
{
    if (!strcmp(work_scene.left, cg))
	work_scene.left[0] = '\0';
    if (!strcmp(work_scene.middle, cg))
	work_scene.middle[0] = '\0';
    if (!strcmp(work_scene.right, cg))
	work_scene.right[0] = '\0';

    switch (pos[0]) {
    case 'L':
	strcpy(work_scene.left, cg);
	break;
    case 'M':
	strcpy(work_scene.middle, cg);
	break;
    case 'R':
	strcpy(work_scene.right, cg);
	break;
    default:
	strcpy(work_scene.right, cg);
	break;
    }
}

Scene
GetWorkScene()
{
    return work_scene;
}

void
ReflectWorkScene()
{
    strcpy(SCENE.bg, work_scene.bg);
    strcpy(SCENE.left, work_scene.left);
    strcpy(SCENE.middle, work_scene.middle);
    strcpy(SCENE.right, work_scene.right);
}

void
CopyScene()
{
    strcpy(work_scene.bg, SCENE.bg);
    strcpy(work_scene.left, SCENE.left);
    strcpy(work_scene.middle, SCENE.middle);
    strcpy(work_scene.right, SCENE.right);
}

void
BackupScene()
{
    backup_scene = SCENE;
    bri_backup = alCore.brightness;
}

void
RestoreScene()
{
    SCENE = backup_scene;
    alCore.brightness = bri_backup;
}

void
RedisplayScene(Image *image, Scene scene)
{
    char arg[ARG_NUM][ARG_LEN];

    if (scene.bg[0]) {
	if (scene.bg[0] == '#') {
	    Color color;
	    color = string2color(scene.bg + 1);
	    ALIB.imageFill(image, 0, 0, WIDTH, HEIGHT, color);
	} else {
	    ALIB.imageLoad(image, scene.bg);
	}
    }

    if (scene.left[0]) {
	strcpy(arg[0], "Chara");
	strcpy(arg[1], scene.left);
	strcpy(arg[2], "L");
	ALIB.imageCharacterLoad(image, arg, 2);
    }

    if (scene.middle[0]) {
	strcpy(arg[0], "Chara");
	strcpy(arg[1], scene.middle);
	strcpy(arg[2], "M");
	ALIB.imageCharacterLoad(image, arg, 2);
    }

    if (scene.right[0]) {
	strcpy(arg[0], "Chara");
	strcpy(arg[1], scene.right);
	strcpy(arg[2], "R");
	ALIB.imageCharacterLoad(image, arg, 2);
    }
}
