/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001, 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: script.c,v 1.32 2002/03/09 17:41:59 tf Exp $
 */
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "aladder.h"
#include "command.h"
#include "event.h"

#define SCR_CMD 0
#define SCR_TXT 1
#define SCR_EOF 2

static int get_next_script(u_char *buf, int running);

static void
scan_script()
{
    u_char buf[SCR_BUF_LEN];
    char argv[ARG_NUM][ARG_LEN];
    SceneList *scene;

    SCRIPT.cur_pos = 0;

    while (1) {
	int p = SCRIPT.cur_pos;
	switch (get_next_script(buf, False)) {
	case SCR_CMD:
	    script_parse_arg(buf + 1, argv);
	    if (!strcmp(argv[0], "SceneName")) {
		scene = (SceneList *) malloc(sizeof(SceneList));
		scene->next = SCRIPT.scene_list;
		strcpy(scene->name, argv[1]);
		scene->offset = p;

		SCRIPT.scene_list = scene;
	    }
	    break;
	case SCR_TXT:
	    break;
	case SCR_EOF:
	    goto end;
	}
    }
  end:
}

void
load_script(const char *name)
{
    free_script();
    SCRIPT.data = get_text(alCore.script_arc, name);
    SCRIPT.end_pos = strlen(SCRIPT.data);

    scan_script();
    SCRIPT.cur_pos = 0;

    ClearScene();
}

void
free_script()
{
    if (SCRIPT.data != NULL) {
	free(SCRIPT.data);
	SCRIPT.data = NULL;
    }

    if (SCRIPT.scene_list != NULL) {
	SceneList *p, *q;
	p = SCRIPT.scene_list;
	while (p != NULL) {
	    q = p->next;
	    free(p);
	    p = q;
	}
	SCRIPT.scene_list = NULL;
    }
}

static int
get_next_script(u_char *buf, int running)
{
    int i = 0;
    u_char ch;

    if (SCRIPT.cur_pos == SCRIPT.end_pos) {
	return SCR_EOF;
    }

    if (SCRIPT.data[SCRIPT.cur_pos] == '#') {
	while (1) {
	    ch = buf[i++] = SCRIPT.data[SCRIPT.cur_pos++];
	    if (SCRIPT.cur_pos == SCRIPT.end_pos) {
		break;
	    }
	    if (ch == 0xa) {
		break;
	    }
	    if (ch == ')') {
		if (SCRIPT.data[SCRIPT.cur_pos] == 0xa)
		    SCRIPT.cur_pos++;
		break;
	    }
	}
	buf[i] = '\0';

	if (running) {
	    set_command(buf);
	    push_stack(STAT_COMMAND);
	}
	return SCR_CMD;
    }

    /* /Ѥ˶ڤФƤ */
    SCENE.block_top = SCRIPT.cur_pos;

    while (1) {
	ch = buf[i++] = SCRIPT.data[SCRIPT.cur_pos++];
	if (SCRIPT.cur_pos == SCRIPT.end_pos) {
	    break;
	}
	if (ch == 0xa) {
	    break;
	}
	if (ch == '#') {
	    SCRIPT.cur_pos--;
	    buf[i - 1] = '\0';
	    break;
	}
    }
    buf[i] = '\0';

    if (running) {
	set_text(buf);
	push_stack(STAT_TEXT);
    }
    return SCR_TXT;
}

static int prev_read = 0;

void
script_seen_proc()
{
    char buf[6];

    sprintf(buf, "%x", SCENE.pos);
    if (isset_flag(alCore.read_flag, buf)) {
	dprintf((stderr, "<>\n"));
	if (CONFIG.skip_read)
	    WAIT.skip = True;
    } else {
	dprintf((stderr, "<̤:%s>\n", buf));
	if (!CONFIG.force_skip)
	    WAIT.skip = False;
    }

    sprintf(buf, "%x", prev_read);
    if (!isset_flag(alCore.read_flag, buf)) {
	add_flag(buf, &alCore.read_flag);
    }
    prev_read = SCENE.pos;
}

static int
script_current_is_seen()
{
    char buf[6];

    sprintf(buf, "%x", prev_read);
    return isset_flag(alCore.read_flag, buf);
}

static int
cancel_wait()
{
    int p = SCRIPT.cur_pos;

    if (SCRIPT.data[p] == 0xa || SCRIPT.data[p] == 0xd) {
	return True;
    }
    if (!strncmp(SCRIPT.data + p, "#Page", 5)) {
	return True;
    }
    if (!CURSOR.move) {
	return True;
    }
    return False;
}

int
script_parse_arg(const char *cmd, char argv[ARG_NUM][ARG_LEN])
{
    int i = 0, j = 0, p = 0;

    while (1) {
	argv[i][j++] = cmd[p++];
	if (cmd[p] == '(' || cmd[p] == ',') {
	    argv[i][j] = '\0';
	    i++;
	    p++;
	    j = 0;
	}
	if (cmd[p] == ')' || cmd[p] == 0xa) {
	    argv[i][j] = '\0';
	    i++;
	    break;
	}
    }
    return i;
}

void
clear_stack()
{
    alCore.state_stack[0] = STAT_EMPTY;
    alCore.state_stack_depth = 0;
}

void
push_stack(int val)
{
    if (alCore.state_stack_depth == STAT_STACK_LEN - 1) {
	fprintf(stderr, "stack overrun\n");
	exit(1);
    }
    alCore.state_stack[++alCore.state_stack_depth] = val;
}

int
pop_stack()
{
    if (alCore.state_stack_depth == 0) {
	fprintf(stderr, "stack underrun\n");
	exit(1);
    }
    return alCore.state_stack[alCore.state_stack_depth--];
}

int
proc_alternative()
{
    int res;
    char arg[ARG_NUM][ARG_LEN];

    if (!(TEXT.highlight & ATTR_ALTN_MASK))
	return 0;

    res = ((ATTR_ALL_MASK ^ ATTR_ALTN_MASK) & TEXT.highlight) - 1;

    newpage();
    TEXT.highlight = 0;
    alCore.alternative_num = 0;
    strcpy(arg[1], alCore.alternative[res].label);
    cmd_Jump(arg, 2, False);

    return 1;
}

int
proc_menu()
{
    int res;
    char arg[ARG_NUM][ARG_LEN];

    if (!(TEXT.highlight & ATTR_ALTN_MASK))
	return 0;

    get_command_arg(arg);

    res = (ATTR_ALL_MASK ^ ATTR_ALTN_MASK) & TEXT.highlight;
    TEXT.highlight = 0;
    alCore.alternative_num = 0;

    return menu_action(arg[1], res);
}

void
run_engine(int engine_mode)
{
    u_char buf[SCR_BUF_LEN];
    int idle;
    timer t;

    alCore.engine_mode = engine_mode;
    event_queue_init();
    clear_stack();
    trap_area_clear();

    while (1) {
	init_timer(&t);
      head:
	if (!event_queue_is_empty()) {
	    int q = event_queue_get();

	    switch (q) {
	    case EVENT_BUTTON1:
		action_click_button1();
		switch (CURRENT_STATE) {
		case STAT_WAIT:
		    if (TEXT.highlight) {
			push_stack(STAT_DICT);
			set_command("#EnterDictMode");
			push_stack(STAT_COMMAND);
		    } else {
			pop_stack();
		    }
		    break;
		case STAT_WAIT_TIME:
		    pop_stack();
		    break;
		case STAT_ALTN:
		    if (proc_alternative())
			pop_stack();
		    break;
		case STAT_CG:
		    pop_stack();
		    set_command("#LeaveCGMode");
		    push_stack(STAT_COMMAND);
		    break;
		case STAT_MENU:
		    if (proc_menu()) {
			pop_stack();
			set_command("#LeaveMenuMode");
			push_stack(STAT_COMMAND);
		    }
		    break;
		case STAT_DICT:
		    if (TEXT.highlight) {
			push_stack(STAT_DICT);
			push_dict();
		    }
		    if (trap_area_selected() == 0)
			dict_mode_leave();
		    break;
		case STAT_TTL_CONFIG:
		    if (proc_config_mode())
			leave_config_mode();
		    break;
		}
		break;
	    case EVENT_BUTTON2:
		action_click_button2();
		switch (CURRENT_STATE) {
		case STAT_WAIT:
		case STAT_ALTN:
		    push_stack(STAT_MENU);
		    set_command("#EnterMenuMode(main_menu)");
		    push_stack(STAT_COMMAND);
		    break;
		case STAT_CG:
		    pop_stack();
		    set_command("#LeaveCGMode");
		    push_stack(STAT_COMMAND);
		    break;
		case STAT_MENU:
		    pop_stack();
		    set_command("#LeaveMenuMode");
		    push_stack(STAT_COMMAND);
		    break;
		case STAT_DICT:
		    dict_mode_leave();
		    break;
		case STAT_BACKLOG:
		    backlog_leave();
		    break;
		}
		break;
	    case EVENT_KEY_SKIP:
		action_key_skip();
		switch (CURRENT_STATE) {
		case STAT_WAIT:
		    pop_stack();
		    break;
		}
	    case EVENT_KEY_CG:
		action_key_cg();
		switch (CURRENT_STATE) {
		case STAT_WAIT:
		    push_stack(STAT_CG);
		    set_command("#EnterCGMode");
		    push_stack(STAT_COMMAND);
		    break;
		case STAT_CG:
		    pop_stack();
		    set_command("#LeaveCGMode");
		    push_stack(STAT_COMMAND);
		    break;
		}
		break;
	    case EVENT_KEY_UP:
		action_key_up();
		break;
	    case EVENT_KEY_DOWN:
		action_key_down();
		break;
	    case EVENT_ENTER_AREA:
		switch (CURRENT_STATE) {
		case STAT_DICT:
		    dict_button_push();
		    break;
		case STAT_MENU:
		    menu_set_selected(trap_area_selected() + 1);
		    break;
		case STAT_TTL_CONFIG:
		    redraw_config_mode();
		    break;
		}
		break;
	    case EVENT_LEAVE_AREA:
		switch (CURRENT_STATE) {
		case STAT_DICT:
		    dict_button_release();
		    break;
		case STAT_MENU:
		    menu_set_selected(trap_area_selected() + 1);
		    break;
		case STAT_TTL_CONFIG:
		    redraw_config_mode();
		    break;
		}
		break;
	    default:
		dprintf((stderr, "̤ΤΥ٥[%d]Ǥ.", q));
	    }
	}
	switch (CURRENT_STATE) {
	case STAT_EMPTY:
	    if (get_next_script(buf, True) == SCR_EOF)
		goto end;
	    goto head;
	case STAT_COMMAND:
	    if (proc_command(False)) {
		pop_stack();
	    }
	    if (CONFIG.skip_read && CURRENT_STATE == STAT_WAIT) {
		if (script_current_is_seen) {
		    pop_stack();
		    WAIT.skip = True;
		}
	    }
	    break;
	case STAT_TEXT:
	    if (proc_text()) {
		if (CURRENT_STATE != STAT_TEXT)
		    break;
		pop_stack();
		if (CONFIG.text_speed != CONFIG_TEXT_FAST
		    && !WAIT.skip && !cancel_wait()) {
		    push_stack(STAT_WAIT);
		} else {
		    goto head;
		}
	    }
	    break;
	case STAT_WAIT:
	    draw_cursor();
	    break;
	case STAT_WAIT_TIME:
	    if (end_wait_timer())
		pop_stack();
	    break;
	case STAT_MENU:
	case STAT_DICT:
	case STAT_ALTN:
	case STAT_CG:
	}
	REQTBL.flip();

	if (WAIT.skip)
	    continue;

	if (WAIT.drop_frame) {
	    WAIT.drop_frame = False;
	    continue;
	}

	get_timer(&t);
	idle = WAIT_TIME - t.msec * 1000;
	if (idle > 0)
	    usleep(idle);
    }
  end:
    cmd_End(NULL, 0, False);
}
