/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001, 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: sound-base.c,v 1.5 2002/03/09 17:41:59 tf Exp $
 */
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <string.h>

#include "aladder.h"
#include "sound.h"

extern SoundInfo sinfo;

static void
device_write(const u_char *data, size_t size)
{
    size_t ret, s;
    char buf[100];
    int maxfd;
    struct timeval tv;
    fd_set watchset, inset;
    FILE *fp;

    tv.tv_sec = 0;
    tv.tv_usec = 0;

    FD_ZERO(&watchset);
    FD_SET(sinfo.control_fd, &watchset);
    maxfd = sinfo.control_fd;

    do {
	inset = watchset;
	select(maxfd + 1, &inset, NULL, NULL, &tv);
	if (FD_ISSET(sinfo.control_fd, &inset)) {
	    if ((fp = fdopen(sinfo.control_fd, "rw")) == NULL) {
		perror("fdopen");
		exit(1);
	    }
	    fgets(buf, sizeof buf, fp);
	    switch (buf[0]) {
	    case 'S':
		dprintf((stderr, ": \n"));
		sinfo.control.interrupt(&sinfo);
		return;
	    case 'L':
		{
		    char *p;
		    if ((p = strchr(buf, '\n'))) {
			*p = '\0';
		    }
		}
		dprintf((stderr, ": [%s]\n", buf + 1));
		sinfo.control.interrupt(&sinfo);
		sinfo.data =
		    get_sound(alCore.sound_arc, buf + 1, &(sinfo.length));
		break;
	    case 'P':
		dprintf((stderr, ": (ʤΤ̵)\n"));
		break;
	    default:
		fprintf(stderr, "̤ΤΥޥ[%s]\n", buf);
		break;
	    }
	}

	/* ǳ褦˺ڤˤƽ */
	s = (size < sinfo.play_byte) ? size : sinfo.play_byte;
	if ((ret = sinfo.control.write(&sinfo, data, s)) < 0) {
	    perror("device_write");
	    return;
	}
	size -= ret;
	data += ret;

	tv.tv_sec = 0;
	tv.tv_usec = sinfo.play_wait;
    } while (size > 3);

    /* մλԤ */
    sinfo.control.sync(&sinfo);
}

int
BaseSoundOpen()
{
    int fd[2];
    pid_t pid;

    sinfo.control_fd = -1;

    if (sinfo.control.open(&sinfo) < 0) {
	dprintf((stderr, ": ǥХ򳫤ޤǤ.\n"));
	return -1;
    }

    if (socketpair(AF_UNIX, SOCK_STREAM, AF_UNSPEC, fd) < 0) {
	perror("socketpair AF_UNIX/PF_UNSPEC");
	return -1;
    }

    if ((pid = fork()) < 0) {
	perror("fork");
	sinfo.control.close(&sinfo);
	close(fd[0]);
	close(fd[1]);
	return -1;
    } else if (pid) {
	sinfo.control.close(&sinfo);
	close(fd[1]);
	sinfo.pid = pid;
	sinfo.control_fd = fd[0];
	return 0;
    }

    sinfo.control_fd = fd[1];
    close(fd[0]);

    {
	char buf[100];
	FILE *fp;
	sinfo.data = NULL;

	if ((fp = fdopen(fd[1], "rw")) == NULL) {
	    perror("fdopen");
	    exit(1);
	}

	while (fgets(buf, sizeof buf, fp)) {
	    char *p;
	    if ((p = strchr(buf, '\n'))) {
		*p = '\0';
	    }
	    dprintf((stderr, ": [%s]\n", buf));

	    if (alCore.sound_arc == NULL)
		continue;

	    switch (buf[0]) {
	    case 'L':
		/*  */
		if (sinfo.data)
		    free(sinfo.data);
		sinfo.data =
		    get_sound(alCore.sound_arc, buf + 1, &sinfo.length);
		break;

	    case 'P':
		/*  */
		if (sinfo.data) {
		    /* TODO: WAV ΥإåΥåפȴ */
		    device_write(sinfo.data + 20, sinfo.length - 20);
		}
		dprintf((stderr, ": λ.\n"));
		write(fd[1], "E\n", 2);
		break;

	    case 'S':
		/* λ */
		if (sinfo.data)
		    free(sinfo.data);
		sinfo.data = NULL;
		break;
	    }
	}
    }
    exit(0);
}

void
BaseSoundClose()
{
    if (sinfo.control_fd >= 0) {
	close(sinfo.control_fd);	/* ҥץϤǽλȤʤ */
    }
    sinfo.control_fd = -1;
}

void
BaseSoundLoad(const char *name)
{
    char buf[100];
    if (sinfo.control_fd >= 0) {
	snprintf(buf, sizeof buf, "L%s\n", name);
	write(sinfo.control_fd, buf, strlen(buf));
    }
}

void
BaseSoundStop()
{
    if (!sinfo.playing) {
	return;
    }
    dprintf((stderr, ": .\n"));

    if (sinfo.control_fd >= 0) {
	write(sinfo.control_fd, "S\n", 2);
    }
    sinfo.playing = False;
}

int
BaseSoundStatus()
{
    char buf[100];
    int maxfd;
    fd_set watchset, inset;
    struct timeval tv;
    FILE *fp;

    tv.tv_sec = 0;
    tv.tv_usec = 0;

    FD_ZERO(&watchset);
    FD_SET(sinfo.control_fd, &watchset);
    maxfd = sinfo.control_fd;

    inset = watchset;
    select(maxfd + 1, &inset, NULL, NULL, &tv);

    if (FD_ISSET(sinfo.control_fd, &inset)) {
	if ((fp = fdopen(sinfo.control_fd, "r")) == NULL) {
	    perror("fdopen");
	    exit(1);
	}
	fgets(buf, sizeof buf, fp);
	if (buf[0] == 'E') {
	    sinfo.playing = False;
	}
    }

    return sinfo.playing;
}

void
BaseSoundPlay()
{
    if (BaseSoundStatus()) {
	BaseSoundStop();
    }

    if (sinfo.control_fd >= 0) {
	write(sinfo.control_fd, "P\n", 2);
    }
    dprintf((stderr, ": .\n"));
    sinfo.playing = True;
}
