/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: sound-esd.c,v 1.3 2002/03/03 15:37:00 tf Exp $
 */
#include <stdlib.h>
#include <unistd.h>

#ifdef SOUND_ESD

#include <esd.h>

#include "aladder.h"
#include "sound.h"

SoundInfo sinfo;

static int esd;
static int sock;

static int
esd_device_open(SoundInfo *sinfo)
{
    char *host;
    esd_format_t format = 0;
    int rate = 22050;

    host = getenv("ESD_HOST");

    if (!host)
	host = NULL;

    esd = esd_open_sound(host);

    format = ESD_BITS16 | ESD_MONO | ESD_STREAM | ESD_PLAY;

    sock = esd_play_stream_fallback(format, rate, host, "al_sound");
    if (sock < 0) {
	fprintf(stderr, "ESD: ³Ǥޤ[%s].\n", host);
	return -1;
    }
    return 0;
}

static void
esd_device_close(SoundInfo *sinfo)
{
    if (sock >= 0)
	close(sock);
    esd_close(esd);
}

static int
esd_device_write(SoundInfo *sinfo, const u_char *data, size_t size)
{
    return write(sock, data, size);
}

static void
esd_device_sync(SoundInfo *sinfo)
{

}

static void
esd_device_interrupt(SoundInfo *sinfo)
{

}

void
ESDSoundLibInit()
{
    sinfo.control.open = esd_device_open;
    sinfo.control.close = esd_device_close;
    sinfo.control.write = esd_device_write;
    sinfo.control.sync = esd_device_sync;
    sinfo.control.interrupt = esd_device_interrupt;

    /* 
     * 0.5 ñ̤ʬ䤷ƥǡ
     * (ESD ϻֳִ֤򳫤ƥǡȲӤ뤳Ȥ褦ʡ)
     */
    sinfo.play_byte = 22050;
    sinfo.play_wait = 500 * 1000;

    ALIB.soundOpen = BaseSoundOpen;
    ALIB.soundClose = BaseSoundClose;
    ALIB.soundLoad = BaseSoundLoad;
    ALIB.soundPlay = BaseSoundPlay;
    ALIB.soundStop = BaseSoundStop;
    ALIB.soundStatus = BaseSoundStatus;
}
#endif
