/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001, 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: sound-oss.c,v 1.7 2002/03/03 15:37:00 tf Exp $
 */
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>

#ifdef SOUND_OSS

#define AUDIO_DEVICE "/dev/dsp"

#if defined(__FreeBSD__)
#include <machine/soundcard.h>
#elif defined(__NetBSD__)
#include <soundcard.h>
#elif defined(__linux__)
#include <sys/soundcard.h>
#endif

#include "aladder.h"
#include "sound.h"

SoundInfo sinfo;

static int dev_fd;

static int
oss_device_open(SoundInfo *sinfo)
{
    char *device;
    int fd, t;

    device = getenv("AUDIO_DEVICE");

    if (!device)
	device = AUDIO_DEVICE;

    if ((fd = open(device, O_WRONLY, 0)) < 0) {
	perror(device);
	return -1;
    }

    t = 16;
    if (ioctl(fd, SNDCTL_DSP_SAMPLESIZE, &t) < 0) {
	perror(device);
	close(fd);
	return -1;
    }

    t = 0;
    if (ioctl(fd, SNDCTL_DSP_STEREO, &t) < 0) {
	perror(device);
	close(fd);
	return -1;
    }

    t = 22050;
    if (ioctl(fd, SNDCTL_DSP_SPEED, &t) < 0) {
	perror(device);
	close(fd);
	return -1;
    }

    dev_fd = fd;

    return 0;
}

static void
oss_device_close(SoundInfo *sinfo)
{
    if (dev_fd >= 0) {
	close(dev_fd);
    }
}

static int
oss_device_write(SoundInfo *sinfo, const u_char *data, size_t size)
{
    return write(dev_fd, data, size);
}

static void
oss_device_sync(SoundInfo *sinfo)
{
    ioctl(dev_fd, SNDCTL_DSP_SYNC, NULL);
}

static void
oss_device_interrupt(SoundInfo *sinfo)
{
    ioctl(dev_fd, SNDCTL_DSP_RESET, NULL);
}

void
OSSSoundLibInit()
{
    sinfo.control.open = oss_device_open;
    sinfo.control.close = oss_device_close;
    sinfo.control.write = oss_device_write;
    sinfo.control.sync = oss_device_sync;
    sinfo.control.interrupt = oss_device_interrupt;

    /* 0.5 ñ̤ʬ䤷ƥǡ */
    sinfo.play_byte = 22050;
    sinfo.play_wait = 500 * 1000;

    ALIB.soundOpen = BaseSoundOpen;
    ALIB.soundClose = BaseSoundClose;
    ALIB.soundLoad = BaseSoundLoad;
    ALIB.soundPlay = BaseSoundPlay;
    ALIB.soundStop = BaseSoundStop;
    ALIB.soundStatus = BaseSoundStatus;
}
#endif
