/* 
 * Tenshi no Hashigo for SDL
 * Copyright (c) 2001 TAJIRI Yasuhiro <tajiri@venus.dti.ne.jp>
 * Copyright (c) 2001, 2002 TF <tf@denpa.org>
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: sound-sdl.c,v 1.8 2002/03/03 15:37:00 tf Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <SDL.h>
#include <SDL_mixer.h>

#include "aladder.h"


#if 0
ALCore alCore;
#endif
static int audio_open = 0;
static Mix_Music *music = NULL;
static Mix_Chunk *sound = NULL;
static Mix_Chunk *voice = NULL;

static int
SoundOpen()
{
    if (Mix_OpenAudio(22050, AUDIO_S16, 2, 4096) < 0) {
	fprintf(stderr, "Couldn't open audio: %s\n", SDL_GetError());
	return -1;
    } else {
	audio_open = 1;
    }
    return 0;
}

static void
SoundClose()
{
    if (Mix_PlayingMusic()) {
	Mix_FadeOutMusic(150);
	SDL_Delay(150);
    }
    if (music) {
	Mix_FreeMusic(music);
	music = NULL;
    }
    if (voice) {
	Mix_FreeChunk(voice);
	voice = NULL;
    }
    if (sound) {
	Mix_FreeChunk(sound);
	sound = NULL;
    }
    if (audio_open) {
	Mix_CloseAudio();
	audio_open = 0;
    }
}

static void
SoundLoad(const char *name)
{
    int length;
    Uint8 *buf;
    if (!audio_open)
	return;
    buf = get_sound(alCore.sound_arc, name, &length);
    sound = Mix_LoadWAV_RW(SDL_RWFromMem(buf, length), 1);

}

static void
SoundPlay()
{
    Mix_PlayChannel(1, sound, 0);
}

static void
SoundStop()
{
    if (!audio_open || !Mix_Playing(1)) {
	return;
    }
    dprintf((stderr, "interrupted playing.\n"));
    Mix_HaltChannel(1);
    if (sound) {
	Mix_FreeChunk(sound);
	sound = NULL;
    }

}

static int
SoundStatus()
{
    if (!audio_open || !Mix_Playing(1)) {
	return 0;
    }
    return 1;
}

void
SDLSoundLibInit()
{
    ALIB.soundOpen = SoundOpen;
    ALIB.soundClose = SoundClose;
    ALIB.soundLoad = SoundLoad;
    ALIB.soundPlay = SoundPlay;
    ALIB.soundStop = SoundStop;
    ALIB.soundStatus = SoundStatus;
}

static int
MusicOpen()
{
    if (!audio_open) {
	SoundOpen();
	audio_open = 1;
    }

    return 0;
}

static void
MusicClose()
{

}

static int
MusicStatus()
{
    return True;
}

static void
MusicLoad(const char *name)
{
    int length;
    Uint8 *buf;
    char path[500];

    if (!audio_open)
	return;
    sprintf(path, "%s/%s", DIR.music_dir, name);
    music = Mix_LoadMUS(path);

}

static void
MusicPlay()
{
    Mix_PlayMusic(music, -1);
}

static void
MusicFadeOut()
{
    Mix_FadeOutMusic(100);
}

static void
MusicStop()
{
    if (!audio_open || !Mix_PlayingMusic()) {
	return;
    }
    dprintf((stderr, "interrupted playing.\n"));
    Mix_HaltMusic();
    if (music) {
	Mix_FreeMusic(music);
	music = NULL;
    }

}

void
SDLMusicLibInit()
{
    ALIB.musicOpen = MusicOpen;
    ALIB.musicClose = MusicClose;
    ALIB.musicLoad = MusicLoad;
    ALIB.musicPlay = MusicPlay;
    ALIB.musicStop = MusicStop;
    ALIB.musicFadeOut = MusicFadeOut;
    ALIB.musicStatus = MusicStatus;
}

#if 0
int
main(int argc, char *argv[])
{
    alCore.sound_arc = open_archive(argv[1]);
    if (alCore.sound_arc == NULL) {
	exit(1);
    }

    SoundOpen();

    SoundLoad(argv[2]);
    SoundPlay();

    while (1) {
	char ch;
	ch = getchar();
	SoundStatus();

	if (ch == 'p')
	    SoundPlay();
	if (ch == 's')
	    SoundStop();
	if (ch == 'x') {
	    SoundStop();
	    break;
	}
    }

    SoundClose();
}
#endif
