/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001, 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: text.c,v 1.29 2002/03/09 17:41:59 tf Exp $
 */
#include "aladder.h"

static int text_buf[SCR_BUF_LEN];
static int text_buf_pos;
static int text_buf_len;

static int attr_buf[SCR_BUF_LEN];

typedef struct {
    int tvram[TEXT_WIDTH][TEXT_HEIGHT];
    int attr[TEXT_WIDTH][TEXT_HEIGHT];
    int geom[TEXT_WIDTH][TEXT_HEIGHT];
} VramInfo;

#define TEXT_STACK_DEPTH 5
static int text_stack_depth = 0;
static VramInfo text_stack[TEXT_STACK_DEPTH];

void
text_init()
{
    TEXT.x = 0;
    TEXT.y = 0;
    TEXT.yoffset = YOFFSET;
    TEXT.highlight = 0;
    TEXT.fullscreen = True;
    TEXT.clear = False;
    TEXT.view_width = WIDTH;

    clear_tvram();
}

void
clear_text_stack()
{
    text_stack_depth = 0;
}

void
push_text()
{
    int i, j;
    if (text_stack_depth == TEXT_STACK_DEPTH) {
	dprintf((stderr, "text stack overflowed.\n"));
	return;
    }

    for (i = 0; i < TEXT_HEIGHT; i++) {
	for (j = 0; j < TEXT_WIDTH; j++) {
	    text_stack[text_stack_depth].tvram[j][i] = TEXT.tvram[j][i];
	    text_stack[text_stack_depth].attr[j][i] = TEXT.attr[j][i];
	    text_stack[text_stack_depth].geom[j][i] = TEXT.geom[j][i];
	}
    }
    text_stack_depth++;
}

void
pop_text()
{
    int i, j;
    if (text_stack_depth == 0) {
	dprintf((stderr, "text stack is empty.\n"));
	return;
    }

    text_stack_depth--;
    for (i = 0; i < TEXT_HEIGHT; i++) {
	for (j = 0; j < TEXT_WIDTH; j++) {
	    TEXT.tvram[j][i] = text_stack[text_stack_depth].tvram[j][i];
	    TEXT.attr[j][i] = text_stack[text_stack_depth].attr[j][i];
	    TEXT.geom[j][i] = text_stack[text_stack_depth].geom[j][i];
	}
    }
}

void
newpage()
{
    TEXT.clear = True;
}

int
newline()
{
    int ret = 0;
    TEXT.tvram[TEXT.x][TEXT.y] = 0;
    HISTORY.tvram[TEXT.x][HISTORY.y] = 0;

    TEXT.x = 0;

    if (++TEXT.y >= TEXT_HEIGHT) {
	newpage();
	if (!WAIT.skip)
	    push_stack(STAT_WAIT);
	ret = -1;
    }

    if (++HISTORY.y >= BACKLOG_LEN) {
	HISTORY.y = 0;
	HISTORY.loop = True;
    }
    return ret;
}

void
clear_tvram()
{
    int i, j;

    for (i = 0; i < TEXT_HEIGHT; i++) {
	for (j = 0; j < TEXT_WIDTH; j++) {
	    TEXT.tvram[j][i] = 0;
	    TEXT.attr[j][i] = ATTR_NORMAL;
	    TEXT.geom[j][i] = 0;
	}
	TEXT.geom[0][i] = XOFFSET;
    }
}

void
clear_text(int draw_window)
{
    clear_tvram();

    TEXT.x = TEXT.y = 0;
    TEXT.highlight = 0;
    if (!TEXT.fullscreen)
	TEXT.y = TEXT_WINDOW_POS;

    if (draw_window)
	REQTBL.clearWindow();
    TEXT.clear = False;
}

#define KINSOKU_TOUTEN 0xa1a2	/*  */
#define KINSOKU_KUTEN 0xa1a3	/*  */
#define KINSOKU_QMARK 0xa1a9	/*  */
#define KINSOKU_EMARK 0xa1aa	/*  */
#define KINSOKU_HYPHEN 0xa1bc	/*  */
#define KINSOKU_DOT 0xa1a6	/*  */

#define KINSOKU_OPAR 0xa1ca	/*  */
#define KINSOKU_OBRACE 0xa1d6	/*  */
#define KINSOKU_OBRACE2 0xa1d8	/*  */
#define KINSOKU_OBRACE3 0xa1e3	/*  */

#define KINSOKU_CPAR 0xa1cb	/*  */
#define KINSOKU_CBRACE 0xa1d7	/*  */
#define KINSOKU_CBRACE2 0xa1d9	/*  */
#define KINSOKU_CBRACE3 0xa1e4	/*  */

#define KINSOKU_XA 0xa4a1	/*  */
#define KINSOKU_XI 0xa4a3	/*  */
#define KINSOKU_XU 0xa4a5	/*  */
#define KINSOKU_XE 0xa4a7	/*  */
#define KINSOKU_XO 0xa4a9	/*  */
#define KINSOKU_XTU 0xa4c3	/*  */
#define KINSOKU_XYA 0xa4e3	/*  */
#define KINSOKU_XYU 0xa4e5	/*  */
#define KINSOKU_XYO 0xa4e7	/*  */

#define KINSOKU_KXA 0xa5a1	/*  */
#define KINSOKU_KXI 0xa5a3	/*  */
#define KINSOKU_KXU 0xa5a5	/*  */
#define KINSOKU_KXE 0xa5a7	/*  */
#define KINSOKU_KXO 0xa5a9	/*  */
#define KINSOKU_KXTU 0xa5c3	/*  */
#define KINSOKU_KXYA 0xa5e3	/*  */
#define KINSOKU_KXYU 0xa5e5	/*  */
#define KINSOKU_KXYO 0xa5e7	/*  */

int
is_kinsoku_o(int next)
{
    switch (next) {
    case KINSOKU_OPAR:
    case KINSOKU_OBRACE:
    case KINSOKU_OBRACE2:
    case KINSOKU_OBRACE3:
	return True;
    }
    return False;
}

int
is_kinsoku_c(int next)
{
    switch (next) {
    case KINSOKU_TOUTEN:
    case KINSOKU_KUTEN:
    case KINSOKU_QMARK:
    case KINSOKU_EMARK:
    case KINSOKU_HYPHEN:
    case KINSOKU_DOT:
    case KINSOKU_CPAR:
    case KINSOKU_CBRACE:
    case KINSOKU_CBRACE2:
    case KINSOKU_CBRACE3:
    case KINSOKU_XA:
    case KINSOKU_XI:
    case KINSOKU_XU:
    case KINSOKU_XE:
    case KINSOKU_XO:
    case KINSOKU_XTU:
    case KINSOKU_XYA:
    case KINSOKU_XYU:
    case KINSOKU_XYO:
    case KINSOKU_KXA:
    case KINSOKU_KXI:
    case KINSOKU_KXU:
    case KINSOKU_KXE:
    case KINSOKU_KXO:
    case KINSOKU_KXTU:
    case KINSOKU_KXYA:
    case KINSOKU_KXYU:
    case KINSOKU_KXYO:
	return True;
    }
    return False;
}

int
put_char(int c, int attr, int next)
{
    int type = ATTR_NORMAL;
    int ret = 0;

    if (TEXT.clear) {
	SCENE.pos = SCENE.block_top;
	SCENE.offset = text_buf_pos;
	dprintf((stderr, "<PAGE TOP: %d>\n", SCENE.pos));
	dprintf((stderr, "<OFFSET: %d>\n", SCENE.offset));

	script_seen_proc();

	clear_text(True);
    }

    if (c == 0xa) {
	if (TEXT.fullscreen) {
	    if (TEXT.x == 0 && TEXT.y == 0)
		return ret;
	} else {
	    if (TEXT.x == 0 && TEXT.y == TEXT_WINDOW_POS)
		return ret;
	}
	if (newline() < 0)
	    ret = -1;
	return ret;
    }

    if (attr & ATTR_DICT_MASK) {
	if (attr == TEXT.highlight) {
	    type = ATTR_LINK_HL;
	} else {
	    type = ATTR_LINK;
	}
    } else if (attr & ATTR_ALTN_MASK) {
	if (attr == TEXT.highlight) {
	    type = ATTR_ALTN_HL;
	} else {
	    type = ATTR_ALTN;
	}
    }

    {
	int x, y, w;
	w = ALIB.putChar(c, TEXT.x, TEXT.y, type);
	if (w < 0)
	    return -1;

	TEXT.geom[TEXT.x + 1][TEXT.y] = TEXT.geom[TEXT.x][TEXT.y] + w;

	TEXT.tvram[TEXT.x][TEXT.y] = c;
	TEXT.attr[TEXT.x][TEXT.y] = attr;
	HISTORY.tvram[TEXT.x][HISTORY.y] = c;

	CURSOR.move = True;
	CURSOR.x = TEXT.x + 1;
	CURSOR.y = TEXT.y;

	ALIB.coordTextToGraphics(TEXT.x, TEXT.y, &x, &y);
	REQTBL.flushWindowArea(x, y, w, FONT_HEIGHT);
    }

    TEXT.x++;

    if (next == 0xa)
	return ret;

    if (TEXT.geom[TEXT.x][TEXT.y] >= WIDTH - ABS_LEFT_MARGIN) {
	if (newline() < 0)
	    ret = -1;
    } else if (TEXT.geom[TEXT.x][TEXT.y] >= WIDTH - PRE_LEFT_MARGIN
	       && is_kinsoku_o(next)) {
	if (newline() < 0)
	    ret = -1;
    } else if (TEXT.geom[TEXT.x][TEXT.y] >= WIDTH - LEFT_MARGIN
	       && !is_kinsoku_c(next)) {
	if (newline() < 0)
	    ret = -1;
    }

    return ret;
}

void
debug_put_char(int c)
{
    u_char u, l;

    u = c >> 8 & 0xff;
    l = c & 0xff;

    if (c == 0xa) {
	fprintf(stderr, "<Enter: %d, %d>\n", TEXT.x, TEXT.y);
	return;
    }

    fprintf(stderr, "<%c%c>", u, l);
}

void
set_text(const u_char *text)
{
    int i, j;

    for (i = 0, j = 0; j < strlen(text);) {
	if (text[j] < 0x80) {
	    text_buf[i++] = text[j];
	    j++;
	} else {
	    text_buf[i++] = text[j] << 8 | text[j + 1];
	    j += 2;
	}
    }
    text_buf_pos = 0;
    text_buf_len = i;

    dict_lookup(text_buf, attr_buf);
}

int
proc_text()
{
    if (text_buf_pos >= text_buf_len)
	return True;

    if (CONFIG.text_speed == CONFIG_TEXT_SLOW) {
	put_char(text_buf[text_buf_pos], attr_buf[text_buf_pos],
		 text_buf[text_buf_pos + 1]);
	text_buf_pos++;
	if (text_buf_pos >= text_buf_len)
	    return True;
    } else {
	for (; text_buf_pos < text_buf_len; text_buf_pos++) {
	    if (put_char(text_buf[text_buf_pos], attr_buf[text_buf_pos],
			 text_buf[text_buf_pos + 1]) < 0) {
		dprintf((stderr, "[̤դˤڡ]\n"));
		text_buf_pos++;
		return False;
	    }
	}
	return True;
    }
    return False;
}

void
redraw_text()
{
    int i, j, w, type;

    for (i = 0; i < TEXT_HEIGHT; i++) {
	for (j = 0; j < TEXT_WIDTH; j++) {
	    if (TEXT.tvram[j][i] == 0 || TEXT.geom[j][i] > TEXT.view_width)
		break;

	    if (TEXT.attr[j][i] & ATTR_DICT_MASK) {
		if (TEXT.attr[j][i] == TEXT.highlight) {
		    type = ATTR_LINK_HL;
		} else {
		    type = ATTR_LINK;
		}
	    } else if (TEXT.attr[j][i] & ATTR_ALTN_MASK) {
		if (TEXT.attr[j][i] == TEXT.highlight) {
		    type = ATTR_ALTN_HL;
		} else {
		    type = ATTR_ALTN;
		}
	    } else if (TEXT.attr[j][i] & ATTR_LOG_MASK) {
		type = ATTR_BACKLOG;
	    } else {
		type = ATTR_NORMAL;
	    }
	    w = ALIB.putChar(TEXT.tvram[j][i], j, i, type);
	    if (w > 0)
		TEXT.geom[j + 1][i] = TEXT.geom[j][i] + w;
	}
    }
    CURSOR.move = True;
}

void
print_backlog(int from, int num)
{
    int i, x, y, ty = 0;

    if (from < 0) {
	if (HISTORY.loop) {
	    from += BACKLOG_LEN;
	} else {
	    return;
	}
    }

    clear_tvram();
    ALIB.imageDisplay(5);

    if (HISTORY.loop) {
	for (i = 0; i < num; i++, ty++) {
	    y = (from + i) % BACKLOG_LEN;
	    for (x = 0; HISTORY.tvram[x][y] != 0; x++)
		TEXT.tvram[x][ty] = HISTORY.tvram[x][y];
	    TEXT.tvram[x][ty] = 0;
	}
    } else {
	for (y = from; y < from + num; y++, ty++) {
	    for (x = 0; HISTORY.tvram[x][y] != 0; x++)
		TEXT.tvram[x][ty] = HISTORY.tvram[x][y];
	    TEXT.tvram[x][ty] = 0;
	}
    }
    for (; ty < TEXT_HEIGHT; ty++)
	TEXT.tvram[0][ty] = 0;

    for (y = 0; y < TEXT_HEIGHT; y++) {
	for (x = 0; x < TEXT_WIDTH; x++)
	    TEXT.attr[x][y] = 0 | ATTR_LOG_MASK;
    }

    redraw_text();
    REQTBL.flushWindow();
}
