/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2002 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: title.c,v 1.29 2002/03/04 16:15:12 tf Exp $
 */

#include <stdlib.h>
#include <unistd.h>

#include "aladder.h"
#include "command.h"
#include "event.h"

static Image *lButton[11], *lButton_m[11];
static Image *sButton[11], *sButton_m[11];
static Image *mascot[5][2], *mascot_m[5][2];

static int title_menu[] = { 0, 1, 2, 3, -1 };
static int game_menu[] = { 4, 5, 10, -1 };
static int room_menu[] = { 6, 7, 8, 9, 10, -1 };

static int *current_menu = NULL;

static int mascot_step = 0;
static int mascot_idx_base = 0;

#define MASCOT_PERIOD 10

typedef struct {
    char flag[16];
    int x;
    int y;
} MascotInfo;

MascotInfo mascot_info[5] = {
    {"kasumi_end=on", 313, 305},
    {"misaki_end=on", 585, 285},
    {"rin_end=on", 295, 400},
    {"sayako_end=on", 450, 395},
    {"mia_end=on", 550, 215},
};

static void
init_title_widget()
{
    int i;
    static int initialized = False;

    if (initialized)
	return;
    initialized = True;

    ALIB.imageLoad(alCore.sub, "title_c.bmp");

    for (i = 0; i < 11; i++) {
	lButton[i] = ALIB.imageNew(303, 54);
	lButton_m[i] = ALIB.imageNew(303, 54);
	ALIB.imageCopy(lButton[i], alCore.sub, 0, 0, 200, 264, 303, 54);
	ALIB.imageCopy(lButton_m[i], alCore.sub, 0, 0, 200, 318, 303, 54);
	ALIB.imageCopy(lButton[i], alCore.sub, 60, 12, 10, 29 * i + 2, 180,
		       27);

	sButton[i] = ALIB.imageNew(247, 44);
	sButton_m[i] = ALIB.imageNew(247, 44);
	ALIB.imageCopy(sButton[i], alCore.sub, 0, 0, 200, 371, 247, 44);
	ALIB.imageCopy(sButton_m[i], alCore.sub, 0, 0, 228, 421, 247, 44);
	ALIB.imageCopy(sButton[i], alCore.sub, 45, 12, 200, 23 * i + 2,
		       145, 20);
    }

    for (i = 0; i < 5; i++) {
	mascot[i][0] = ALIB.imageNew(56, 80);
	mascot[i][1] = ALIB.imageNew(56, 80);
	if (i < 4) {
	    ALIB.imageCopy(mascot[i][0], alCore.sub, 0, 0, 516,
			   160 + 80 * i, 56, 80);
	    ALIB.imageCopy(mascot[i][1], alCore.sub, 0, 0, 580,
			   160 + 80 * i, 56, 80);
	} else {
	    ALIB.imageCopy(mascot[i][0], alCore.sub, 0, 0, 353, 177, 56,
			   80);
	    ALIB.imageCopy(mascot[i][1], alCore.sub, 0, 0, 416, 177, 56,
			   80);
	}
	mascot_m[i][0] = ALIB.imageCreateMask(mascot[i][0]);
	mascot_m[i][1] = ALIB.imageCreateMask(mascot[i][1]);
    }
}

void
redraw_premenu_all()
{
    int i;
    int res = trap_area_selected() - mascot_idx_base;

    ALIB.imageAdd(alCore.vram, alCore.sub, 0, 0);

    if (res < 0)
	mascot_step = 0;

    /* ޥåȤ */
    for (i = 0; i < 5; i++) {
	int j = 0;
	if (!isset_flag(alCore.flag, mascot_info[i].flag))
	    continue;
	if (i == res && mascot_step < MASCOT_PERIOD / 2)
	    j = 1;
	ALIB.imageAddWithMask(alCore.vram, mascot[i][j], mascot_m[i][j],
			      mascot_info[i].x, mascot_info[i].y);
    }

    /* ˥塼 */
    i = 0;
    while (current_menu[i] >= 0) {
	int x, y;
	if (i == trap_area_selected()) {
	    if (i < 4) {
		x = 10;
		y = 230 + i * 55;
	    } else {
		x = 330;
		y = 230 + (i - 1) * 55;
	    }
	    ALIB.imageAddWithMask(alCore.vram, lButton[current_menu[i]],
				  lButton_m[current_menu[i]], x, y);
	} else {
	    if (i < 4) {
		x = 10 + 28;
		y = 230 + i * 55 + 5;
	    } else {
		x = 330 + 28;
		y = 230 + (i - 1) * 55 + 5;
	    }
	    ALIB.imageAddWithMask(alCore.vram, sButton[current_menu[i]],
				  sButton_m[current_menu[i]], x, y);
	}
	i++;
    }
}

static void
redraw_premenu(int full)
{
    redraw_premenu_all();

    if (full) {
	REQTBL.flushWindow();
    } else {
	REQTBL.flushWindowArea(0, 200, 320, 280);
    }
}

static void
set_premenu(int *menu)
{
    int i = 0;

    trap_area_clear();
    current_menu = menu;

    while (menu[i] >= 0) {
	if (i < 4) {
	    trap_area_add(15, 230 + i * 55, 303, 54);
	} else {
	    trap_area_add(335, 230 + (i - 1) * 55, 303, 54);
	}
	i++;
    }
    mascot_idx_base = i;

    for (i = 0; i < 5; i++) {
	trap_area_add(mascot_info[i].x, mascot_info[i].y, 56, 80);
    }

    redraw_premenu(True);
}

static void
mascot_action()
{
    mascot_step = (mascot_step + 1) % MASCOT_PERIOD;
    redraw_premenu_all();
    REQTBL.flushWindowArea(300, 200, 340, 280);
}

static void
enter_title()
{
    char music[][ARG_LEN] = {
	"Music",
	"12tenhashi.mp3",
    };
    clear_text(False);
    push_stack(STAT_TTL_TITLE);
    strcpy(SCENE.bg, "title_bg.bmp");
    ALIB.imageLoad(alCore.sub, "title_bg.bmp");
    set_premenu(title_menu);
    cmd_Music(music, 2, False);
}

static void
enter_game()
{
    push_stack(STAT_TTL_GAME);
    set_premenu(game_menu);
}

static void
enter_room()
{
    push_stack(STAT_TTL_ROOM);
    set_premenu(room_menu);
}

static int
proc_title_in()
{
    static int state = 0, s;
    static timer t;

    if (state == 0) {
	SET_CACHE(alCore.vram, "title_1.bmp");
	SET_CACHE(alCore.vram, "title_2.bmp");
	SET_CACHE(alCore.vram, "title_3.bmp");
    }

    switch (state) {
    case 0:
	ALIB.imageFill(alCore.vram, 0, 0, WIDTH, HEIGHT, BLACK);
	REQTBL.flushWindow();
	s = 0;
	state++;
	ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, BLACK);
	ALIB.imageLoad(alCore.sub, "title_1.bmp");
	return False;
    case 1:
	if (s == 0)
	    init_timer(&t);
	if (s++ < 6) {
	    get_timer(&t);
	    if (t.msec > (s + 1) * 50 && s != 6) {
		WAIT.drop_frame = True;
		return False;
	    }
	    ALIB.imageMerge(alCore.vram, alCore.sub, alCore.work,
			    s * 42 + 3);
	    REQTBL.flushWindow();
	    return False;
	}
	s = 0;
	state++;
	ALIB.imageLoad(alCore.work, "title_1.bmp");
	ALIB.imageLoad(alCore.sub, "title_2.bmp");
	return False;
    case 2:
	if (s == 0)
	    init_timer(&t);
	if (s++ < 6) {
	    get_timer(&t);
	    if (t.msec > (s + 1) * 50 && s != 6) {
		WAIT.drop_frame = True;
		return False;
	    }
	    ALIB.imageMerge(alCore.vram, alCore.sub, alCore.work,
			    s * 42 + 3);
	    REQTBL.flushWindow();
	    return False;
	}
	s = 0;
	state++;
	ALIB.imageLoad(alCore.work, "title_2.bmp");
	ALIB.imageLoad(alCore.sub, "title_3.bmp");
	return False;
    case 3:
	if (s == 0)
	    init_timer(&t);
	if (s++ < 6) {
	    get_timer(&t);
	    if (t.msec > (s + 1) * 50 && s != 6) {
		WAIT.drop_frame = True;
		return 0;
	    }
	    ALIB.imageMerge(alCore.vram, alCore.sub, alCore.work,
			    s * 42 + 3);
	    REQTBL.flushWindow();
	    return False;
	}
	s = 0;
	state++;
	ALIB.imageLoad(alCore.work, "title_3.bmp");
	ALIB.imageFill(alCore.sub, 0, 0, WIDTH, HEIGHT, WHITE);
	return False;
    case 4:
	if (s == 0)
	    init_timer(&t);
	if (s++ < 3) {
	    get_timer(&t);
	    if (t.msec > (s + 1) * 50 && s != 3) {
		WAIT.drop_frame = True;
		return False;
	    }
	    ALIB.imageMerge(alCore.vram, alCore.sub, alCore.work, s * 85);
	    REQTBL.flushWindow();
	    return False;
	}
	s = 0;
	state++;
	ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, WHITE);
	ALIB.imageLoad(alCore.sub, "title_bg.bmp");
	return False;
    case 5:
	if (s == 0)
	    init_timer(&t);
	if (s++ < 3) {
	    get_timer(&t);
	    if (t.msec > (s + 1) * 50 && s != 3) {
		WAIT.drop_frame = True;
		return False;
	    }
	    ALIB.imageMerge(alCore.vram, alCore.sub, alCore.work, s * 85);
	    REQTBL.flushWindow();
	    return False;
	}
	state++;
	break;
    default:
	break;
    }

    state = 0;
    return True;
}

static int
proc_title_end()
{
    static int state = 0;
    int ret;

    if (state == 0) {
	ALIB.imageFill(alCore.work, 0, 0, WIDTH, HEIGHT, BLACK);
	ALIB.imageCopy(alCore.sub, alCore.vram, 0, 0, 0, 0, WIDTH, HEIGHT);
    }
    ret = FadePattern(&state, True);

    if (ret)
	state = 0;

    return ret;
}

void
run_title_engine()
{
    timer t;
    int idle;

    event_queue_init();
    clear_stack();
    init_title_widget();
    push_stack(STAT_TTL_TITLE_FADE);

    init_variable();

    while (1) {
	init_timer(&t);
	if (!event_queue_is_empty()) {
	    int q = event_queue_get();

	    switch (q) {
	    case EVENT_ENTER_AREA:
		switch (CURRENT_STATE) {
		case STAT_TTL_TITLE:
		case STAT_TTL_GAME:
		case STAT_TTL_ROOM:
		    redraw_premenu(False);
		    break;
		case STAT_TTL_CGMODE:
		    redraw_cg_mode(False);
		    break;
		case STAT_TTL_PREVMODE:
		    redraw_prev_mode(False);
		    break;
		case STAT_TTL_MUSIC:
		    redraw_music_mode(False);
		    break;
		case STAT_TTL_CONFIG:
		    redraw_config_mode();
		    break;
		case STAT_MENU:
		    menu_set_selected(trap_area_selected() + 1);
		    break;
		}
		break;
	    case EVENT_LEAVE_AREA:
		switch (CURRENT_STATE) {
		case STAT_TTL_TITLE:
		case STAT_TTL_GAME:
		case STAT_TTL_ROOM:
		    redraw_premenu(False);
		    break;
		case STAT_TTL_CGMODE:
		    redraw_cg_mode(False);
		    break;
		case STAT_TTL_PREVMODE:
		    redraw_prev_mode(False);
		    break;
		case STAT_TTL_MUSIC:
		    redraw_music_mode(False);
		    break;
		case STAT_TTL_CONFIG:
		    redraw_config_mode();
		    break;
		case STAT_MENU:
		    menu_set_selected(trap_area_selected() + 1);
		    break;
		}
		break;
	    case EVENT_BUTTON1:
		dprintf((stderr, "button1\n"));
		switch (CURRENT_STATE) {
		case STAT_TTL_TITLE:
		    switch (trap_area_selected()) {
		    case 0:
			pop_stack();
			enter_game();
			break;
		    case 1:
			pop_stack();
			enter_room();
			break;
		    case 2:
			pop_stack();
			enter_config_mode();
			break;
		    case 3:
			LongJump(JUMP_END);
		    }
		    break;
		case STAT_TTL_GAME:
		    switch (trap_area_selected()) {
		    case 0:
			pop_stack();
			push_stack(STAT_TTL_TITLE_END);
			break;
		    case 1:
			pop_stack();
			strcpy(SCENE.bg, "title_bg.bmp");
			push_stack(STAT_MENU);
			set_command("#EnterMenuMode(file_load)");
			push_stack(STAT_COMMAND);
			break;
		    case 2:
			pop_stack();
			enter_title();
			break;
		    }
		    break;
		case STAT_MENU:
		    if (proc_menu())
			LongJump(JUMP_RESTART);
		    break;
		case STAT_TTL_ROOM:
		    switch (trap_area_selected()) {
		    case 0:
			pop_stack();
			enter_cg_mode();
			break;
		    case 1:
			pop_stack();
			enter_prev_mode();
			break;
		    case 2:
			pop_stack();
			enter_music_mode();
			break;
		    case 3:
			pop_stack();
			enter_title();
			break;
		    case 4:
			pop_stack();
			enter_title();
			break;
		    }
		    break;
		case STAT_TTL_CGMODE:
		    switch (trap_area_selected()) {
		    case 0:
			backward_cg_mode();
			break;
		    case 1:
			pop_stack();
			leave_cg_mode();
			enter_title();
			break;
		    case 2:
			forward_cg_mode();
			break;
		    default:
			proc_cg_mode();
			break;
		    }
		    break;
		case STAT_TTL_CGVIEW:
		    forward_cg_view();
		    break;
		case STAT_TTL_PREVMODE:
		    switch (trap_area_selected()) {
		    case 0:
			backward_prev_mode();
			break;
		    case 1:
			pop_stack();
			leave_prev_mode();
			enter_title();
			break;
		    case 2:
			forward_prev_mode();
			break;
		    default:
			proc_prev_mode();
			break;
		    }
		    break;
		case STAT_TTL_MUSIC:
		    switch (trap_area_selected()) {
		    case 0:
			pop_stack();
			enter_title();
			break;
		    case 1:
			dprintf((stderr, "stop\n"));
			break;
		    case 2:
			dprintf((stderr, "play\n"));
			break;
		    }
		    break;
		case STAT_TTL_CONFIG:
		    if (proc_config_mode()) {
			pop_stack();
			enter_title();
		    }
		    break;
		}
		break;
	    case EVENT_BUTTON2:
		dprintf((stderr, "button2\n"));
		switch (CURRENT_STATE) {
		case STAT_MENU:
		    pop_stack();
		    push_stack(STAT_TTL_TITLE_ENTRY);
		    set_command("#LeaveMenuMode");
		    push_stack(STAT_COMMAND);
		    break;
		case STAT_TTL_CGVIEW:
		    leave_cg_view();
		    break;
		}
		break;
	    default:
		dprintf((stderr, "ꤷƤʤ٥[%d]Ǥ.", q));
	    }
	}

	switch (CURRENT_STATE) {
	case STAT_EMPTY:
	    break;
	case STAT_COMMAND:
	    if (proc_command(False)) {
		pop_stack();
	    }
	    break;
	case STAT_TTL_TITLE_FADE:
	    if (proc_title_in()) {
		pop_stack();
		push_stack(STAT_TTL_MENU_FLY);
	    }
	    break;
	case STAT_TTL_MENU_FLY:
	    pop_stack();
	    enter_title();
	    break;
	case STAT_TTL_TITLE_ENTRY:
	    pop_stack();
	    enter_title();
	    break;
	case STAT_TTL_TITLE:
	case STAT_TTL_GAME:
	case STAT_TTL_ROOM:
	    mascot_action();
	    break;
	case STAT_TTL_TITLE_END:
	    if (proc_title_end()) {
		ALIB.musicFadeOut();
		init_variable();
		goto end;
	    }
	    break;
	}
	REQTBL.flip();

	get_timer(&t);
	idle = WAIT_TIME - t.msec * 1000;
	if (WAIT.drop_frame) {
	    WAIT.drop_frame = False;
	    continue;
	}
	if (idle > 0)
	    usleep(idle);
    }
  end:
}
