/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: util.c,v 1.8 2002/03/09 17:41:59 tf Exp $
 */
#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/time.h>

#include "aladder.h"

static struct {
    int duration;
    timer t;
} wait_timer;

void *
setup_signal(int signo, void (*func) (int))
{
    struct sigaction act, oact;
    act.sa_handler = func;
    sigemptyset(&act.sa_mask);
    act.sa_flags = 0;
    if (sigaction(signo, &act, &oact) < 0)
	return SIG_ERR;
    return oact.sa_handler;
}

void *
clear_signal(int signo)
{
    struct sigaction act, oact;
    act.sa_handler = SIG_DFL;
    sigemptyset(&act.sa_mask);
    act.sa_flags = 0;
    if (sigaction(signo, &act, &oact) < 0)
	return SIG_ERR;
    return oact.sa_handler;
}

/* ESC code for Kanji code conversion */
#define	ESC	((char)0x1b)
#define	SS2	((char)0x8e)
#define	SS3	((char)0x8f)

void
sjis2euc(char *sjis, char *euc)
{
    int c, c1, c2;
    u_char *s = (u_char *) sjis;
    u_char *e = (u_char *) euc;

    while ((c = *s++) != 0) {
	if (c & 0x80) {
	    if (c >= 0xa0 && c < 0xe0) {
		*e++ = SS2;
		*e++ = c;
		continue;
	    }
	    if (c == 0x80 || c >= 0xfd)
		continue;	/* ignore */
	    c2 = *s++;
	    if (c >= 0xe0)
		c -= 64;
	    c1 = ((c - 0x81) << 1) + (c2 >= 0x9f ? 1 : 0) + 0xa1;
	    c2 -= ((c2 & 0x80) ? 0x41 : 0x40);
	    if (c2 >= 94)
		c2 -= 94;
	    c2 += 0xa1;
	    *e++ = c1;
	    *e++ = c2;
	} else {
	    *e++ = c;
	}
    }
    *e = '\0';
}

int
text_len(int *text)
{
    int i;

    for (i = 0; text[i] != 0; i++);

    return i;
}

void
text_cpy(int *dst, int *src)
{
    while (*src != 0)
	*dst++ = *src++;
    *dst = 0;
}

int
text_cmp(int *text1, int *text2, int num)
{
    int i;

    for (i = 0; i < num; i++) {
	if (text1[i] != text2[i])
	    return 0;
    }
    return 1;
}

void
euc_to_text(u_char *euc, int *text)
{
    u_char *p = euc;
    int *q = text;

    while (*p) {
	if (*p < 0x80) {
	    *q++ = *p++;
	} else {
	    *q++ = *p << 8 | *(p + 1);
	    p += 2;
	}
    }
    *q = 0;
}

void
init_timer(timer *tp)
{
    gettimeofday(&tp->start, 0);
    tp->msec = 0;
}

void
get_timer(timer *tp)
{
    struct timeval tv;
    long s, us;

    gettimeofday(&tv, 0);
    s = tv.tv_sec - tp->start.tv_sec;
    us = tv.tv_usec - tp->start.tv_usec;

    tp->msec = s * 1000 + us / 1000;
}

void
init_wait_timer(int second)
{
    wait_timer.duration = second * 1000;
    init_timer(&wait_timer.t);
}

int
end_wait_timer()
{
    get_timer(&wait_timer.t);
    if (wait_timer.t.msec > wait_timer.duration)
	return True;
    return False;
}

Color
string2color(const char *string)
{
    int r, g, b;
    Color color;

    sscanf(string, "0x%02x%02x%02x", &r, &g, &b);

    color.r = r;
    color.g = g;
    color.b = b;

    return color;
}
